unit fMHA_ASI_Options;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  fIA_dialogs, Dialogs,
  StdCtrls,
  Buttons,
  ExtCtrls,
  JvExStdCtrls,
  JvCombobox,
  JvColorCombo,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons;


type
  TfrmASI_Options = class(TForm)
    pnl1: TPanel;
    lbl1: TLabel;
    lbl2: TLabel;
    pnlSample: TPanel;
    cbxColorForeground: TJvColorComboBox;
    cbxColorBackground: TJvColorComboBox;
    btnOK: TJvXPButton;
    btnCancel: TJvXPButton;

    procedure FormCreate(Sender: TObject);
    procedure ConfigureForm;

    procedure btnOkClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure cbxColorForegroundChange(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormResize(Sender: TObject); // Letters used are: C-Close, E-Exit, F-Finish later
                            // S-Save/Verify/Sign it
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmASI_Options              : TfrmASI_Options;

implementation

uses
  uIA_Const,
  uMHA_Registry,
  uIA_GUIControl;

{$R *.DFM}

var
  lvInitialize                : boolean;


procedure TfrmASI_Options.FormCreate(Sender: TObject);
begin
  Position := poDesigned;
  lvInitialize := true;
  ConfigureForm;

  lvInitialize := false;
  ActiveControl := cbxColorForeground;

  ShowModal;
end;

procedure TfrmASI_Options.ConfigureForm;
var
  i                           : Integer;
begin
  with GUI_Control do
    begin
      Scaled := True;
      AutoScroll := False;
      AutoSize := False;
      PixelsPerInch := 120;
      KeyPreview := True;
      HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
      VertScrollBar.Range := 0;
      Font.Assign(gvFont);

      Height := GetHeight(11 * ButtonHeight); //CheckHeight(250, Height);
      Width := GetWidth(4 * ButtonWidthNormal); //CheckWidth(280, Width);
      Left := GetLeft(Width);
      Top := GetTop(Height);
      if CheckWidth(Width) or CheckHeight(Height) then
        WindowState := wsMaximized;
      Refresh; // resets alClient dimensions

      for i := 0 to ComponentCount - 1 do
        begin

          if Components[i] is TLabel then
            TLabel(Components[i]).Height := TextHeight('Yy', gvFont);

          if Components[i] is TJvColorComboBox then
            begin
              TJvColorComboBox(Components[i]).Height := 2 * TextHeight('Yy',
                gvFont);
              //TJvColorComboBox(Components[i]).ItemHeight := 3 * TextHeight('Yy',
              //  gvFont) div 2;
              TJvColorComboBox(Components[i]).Width := (width div 2) -
                (Width div 6);
              TJvColorComboBox(Components[i]).Left := width div 2;
            end;
          if Components[i] is TJvXPButton then
            begin
              TJvXPButton(Components[i]).Height := ButtonHeight;
              TJvXPButton(Components[i]).Width := ButtonWidthNormal;
            end;
        end;

      btnOK.Top := Height - (3 * ButtonHeight div 2);
      btnCancel.Top := btnOK.Top;
      btnOK.Left := Width - btnOK.Width - btnOK.Width div 3;
      btnCancel.Left := btnOK.Width div 3;

      cbxColorForeground.ColorValue :=
        gvHighlightForeground;
      cbxColorBackground.ColorValue :=
        gvHighlightBackground;

      if gvUseScreenReader then
        begin
          Left := 0;
          Width := Screen.Width;
          //btnOK.caption := btnOK.Caption + ' button';
          //btnCancel.caption := btnCancel.Caption + ' button';
        end;

      Update;
    end;
end;


procedure TfrmASI_Options.btnOkClick(Sender: TObject);
begin
  gvHighlightForeground := cbxColorForeground.ColorValue; {TColor}
  gvHighlightBackground := cbxColorBackground.ColorValue; {TColor}
  MHA_Registry.SaveRegistry;
  Close;
end;

procedure TfrmASI_Options.btnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmASI_Options.cbxColorForegroundChange(Sender: TObject);
begin

  pnlSample.Color := cbxColorBackground.ColorValue;
  pnlSample.Font.Color := cbxColorForeground.ColorValue;
  if lvInitialize then
    exit;
  if cbxColorBackground.ColorValue = cbxColorForeground.ColorValue then
    begin
      MsgDlgFrm('The foreground and background cannot be the same color. The defaults will ' +
        'be reinstated.',
        'ASI Highlight Colors', mtInformation, [mbOK], 0);
       // frmASIOptions.Left, frmASIOptions.Top, frmASIOptions.Height, frmASIOptions.Width, 0);
      cbxColorForeground.ColorValue := clBlack;
      cbxColorBackground.ColorValue := clYellow;
      pnlSample.Color := cbxColorBackground.ColorValue;
      pnlSample.Font.Color := cbxColorForeground.ColorValue;
    end;
  Update;

end;

procedure TfrmASI_Options.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = vk_Escape then
    begin
      Close;
      Key := 0;
    end;
end;



procedure TfrmASI_Options.FormResize(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  // Reset these, used in placement of dialogs
  gvLeft := Left;
  gvTop := Top;
  gvHeight := Height;
  gvWidth := Width;
end;

end.

