unit fMHA_Admin;

interface

uses
  Windows,
  Messages,
  SysUtils,
  System.UITypes,
  Classes,
  Graphics,
  Controls,
  Forms,
  RzPanel,
  StdCtrls,
  ORCtrls,
  ComCtrls,
  ExtCtrls,
  RzLabel,
  JvExMask,
  JvToolEdit,
  JvCheckedMaskEdit,
  JvDatePickerEdit,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons,
  dwlCore,
  jclSysInfo,
  Mask;

type
  TfrmMHA_Admin = class(TForm)
    pnlHeader: TRzPanel;
    lblHeader: TRzLabel;
    pnlLeft: TPanel;
    grpDate: TGroupBox;
    grpOrderer: TGroupBox;
    cbxOrderedBy: TORComboBox;
    grpInterviewer: TGroupBox;
    cbxInterviewer: TORComboBox;
    pnlRight: TPanel;
    pnlSpacer2: TPanel;
    pnlSpacer0: TPanel;
    grpLocation: TGroupBox;
    cbxlocation: TORComboBox;
    pnlButtons: TPanel;
    pnlRadio: TPanel;
    lblDisplay: TLabel;
    rbOne: TRadioButton;
    rbAll: TRadioButton;
    lblPatient: TRzLabel;
    btnStaffEntry: TJvXPButton;
    dtpkAdministration: TDateTimePicker;
    btnPatientEntry: TJvXPButton;
    grpConsult: TGroupBox;
    cbxConsult: TORComboBox;

    procedure FormShow(Sender: TObject);
    procedure cbxOrderedByNeedData(Sender: TObject;
      const StartFrom: string; Direction, InsertAt: Integer);
    procedure FormCreate(Sender: TObject);
    procedure btnStaffEntryClick(Sender: TObject);
    procedure btnPatientEntryClick(Sender: TObject);
    procedure cbxInterviewerChange(Sender: TObject);
    procedure cbxOrderedByChange(Sender: TObject);
    procedure dtpkAdministrationChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbxInterviewerDropDown(Sender: TObject);
    procedure cbxOrderedByDropDown(Sender: TObject);
    procedure cbxLocationChange(Sender: TObject);
    procedure cbxlocationDropDown(Sender: TObject);
    procedure cbxlocationNeedData(Sender: TObject; const StartFrom: string;
      Direction, InsertAt: Integer);
    //procedure cbxConsultChange(Sender: TObject);
    //procedure cbxConsultDropDown(Sender: TObject);
    //procedure imgBackupClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ConfigureForm(Sender: TObject);
    function IsSecureDesktopInstalled: BOOLEAN;
  private
    { Private declarations }
    FCancelled,
      FIsStaffEntry,
      FIsAllItems: Boolean;
    FOrderedByDUZ,
      FInterviwerDUZ,
      FDateGiven,
      FLocationName,
      FLocationIEN: string;
  public
    { Public declarations }
    property Cancelled: boolean read FCancelled write FCancelled;
    property IsStaffEntry: boolean read FIsStaffEntry write FIsStaffEntry;
    property IsAllItems: boolean read FIsAllItems write FIsAllItems;
    property OrderedByDUZ: string read FOrderedByDUZ write FOrderedByDUZ;
    property InterviwerDUZ: string read FInterviwerDUZ write FInterviwerDUZ;
    property DateGiven: string read FDateGiven write FDateGiven;
    property LocationName: string read FLocationName write FLocationName;
    property LocationIEN: string read FLocationIEN write FLocationIEN;

  end;

var
  frmMHA_Admin                : TfrmMHA_Admin;

implementation

{$R *.DFM}

uses
  uIA_Const,
  uMHA_Const,
  uMHA_LocalFiles,
  fIA_dialogs, Dialogs,
  uIA_ThisUser,
  uIA_Staff_OPS,
  uIA_Consult_OPS,
  uIA_Patient,
  uIA_Utils,
  uIA_GUIControl,
  uMHA_Registry,
  uMHA_Location_IO,
  uMHA_DM;


var
  lvInitialize                : Boolean;


procedure TfrmMHA_Admin.FormCreate(Sender: TObject);
begin

  lvInitialize := True;
  Position := poDesigned;

  FCancelled := true;
  FIsStaffEntry := true;
  FOrderedByDUZ := ThisUser.DUZ;
  FInterviwerDUZ := ThisUser.DUZ;
  FDateGiven := DateTimeToStr(Now);
  FLocationName := '';
  FLocationIEN := '';

  if gvShowAllOrOneItem then
    begin
      rbAll.Checked := true;
      FIsAllItems := true;
    end
  else
    begin
      FIsAllItems := false;
      rbOne.Checked := True;
    end;

  if (DataBaseMode = dbVistA) then
    begin
      cbxOrderedBy.Text := ThisUser.Name;
      cbxInterviewer.Text := ThisUser.Name;
    end
  else
    begin
      cbxOrderedBy.Text := ThisUser.Name;
      cbxOrderedBy.Items.Clear;
      cbxOrderedBy.Enabled := False;
      cbxInterviewer.Text := ThisUser.Name;
      cbxInterviewer.Items.Clear;
      cbxInterviewer.Enabled := False;
    end;

  // NO_ID means this is a new instrument.
  if gvID <> NO_ID then
    begin
      // Edit exisiting instrument/administration
      pnlLeft.Visible := false;
      pnlLeft.Width := 0;
    end
  else
    pnlLeft.Visible := true;

  ConfigureForm(Sender);
end;


procedure TfrmMHA_Admin.FormShow(Sender: TObject);
begin
  if pnlLeft.Visible then
    begin
      dtpkAdministration.DateTime := gvDateGiven;
      dtpkAdministration.MaxDate := Now;

      if (Piece(gvLastLocation, U, 1) = '-1') or
        (Piece(gvLastLocation, U, 1) = ' ') or
        (Piece(gvLastLocation, U, 1) = '') then
        begin
          cbxLocation.InitLongList('');
          cbxLocation.ItemIndex := 0;
        end
      else
        begin
          cbxLocation.InitLongList(Piece(gvLastLocation, U, 2));
          cbxLocation.ItemIndex :=
            cbxlocation.SelectByIEN(StrToIntDef(Piece(gvLastLocation, U, 1), 0));
        end;

      FLocationIEN := IntToStr(cbxLocation.ItemIEN);
      FLocationName := cbxLocation.Text;

      cbxConsult.Items.AddStrings(Consult_OPS.LoadConsultSubSet(Patient.DFN,
        DataModule1.RPCBroker1));
      cbxConsult.DropDownCount := cbxConsult.Items.Count - 1;
    //  lvAdministration.ConsultIEN := '';
    end;

  btnPatientEntry.Enabled := IsSecureDesktopInstalled; //FIsSecureDesktopAvailable;
  lvInitialize := False; // return to normal component functioning
  Screen.Cursor := crDefault;

  if (DataBaseMode = dbVistA) then
    begin
      if pnlLeft.visible then
        begin
          if gvUseScreenReader then
            ActiveControl := grpOrderer
          else
            ActiveControl := cbxOrderedBy;
        end
      else
        ActiveControl := rbAll;
    end
  else
    begin
      if gvUseScreenReader then
        ActiveControl := grpDate
      else
        ActiveControl := dtpkAdministration;
    end;
end;

procedure TfrmMHA_Admin.ConfigureForm(Sender: TObject);
begin
  // See FormResize for left positions on buttons.
  Caption := MHA_Caption + ' in use by ' + ThisUser.Name;

  with GUI_Control do
    begin
      Scaled := True;
      AutoScroll := False;
      AutoSize := False;
      PixelsPerInch := 120;
      KeyPreview := True;
      HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
      VertScrollBar.Range := 0;
      Font.Assign(gvFont);

      with lblHeader do
        begin
          Caption := '   ' + gvNameOfInstrument;
          ShowHint := True;
          Font.Assign(gvFont);
          Font.Size := lblHeaderFontSize;
          Font.Color := clBlack;
          AutoSize := False;
          Layout := tlCenter;
          Align := alLeft;
          Color := clWhite;
          Width := pnlHeader.Width - lblpatient.Left - 5;
        end;

      with lblPatient do
        begin
          Font.Assign(gvFont);
          Font.Size := lblPatientFontSize;
          Font.Color := clBlack;
          Color := clWhite;
          Layout := tlCenter;
          Align := alRight;
          AutoSize := false;
          Caption := Patient.CaptionInfo;
          Width := TextWidth(Caption, Font);
        end;

      pnlHeader.Height := pnlHeaderHeight;

      pnlSpacer0.Width := ButtonHeight div 4;
      pnlSpacer2.Width := ButtonHeight div 4;

      grpDate.Height := 2 * ButtonHeight;
      grpOrderer.Height := grpDate.Height;
      grpInterviewer.Height := grpDate.Height;
      grpLocation.Height := grpDate.Height;
      grpConsult.Height := grpDate.Height;

      Height := GetHeight(pnlHeader.Height + (5 * grpDate.Height) +
        (3 * ButtonHeight div 2));
      Width := GetWidth(7 * ButtonWidthNormal);
      Left := GetLeft(Width);
      Top := GetTop(Height);
      if CheckWidth(Width) or CheckHeight(Height) then
        WindowState := wsMaximized;
      Refresh; // resets alClient dimensions

      pnlLeft.Width := 3 * ButtonWidthNormal;

      dtpkAdministration.Top := grpDate.Height div 3;
      cbxOrderedBy.Top := dtpkAdministration.Top;
      cbxInterviewer.Top := dtpkAdministration.Top;
      cbxLocation.Top := dtpkAdministration.Top;
      cbxConsult.Top := dtpkAdministration.Top;

      dtpkAdministration.Left := pnlSpacer0.Width;
      cbxOrderedBy.Left := dtpkAdministration.Left;
      cbxInterviewer.Left := dtpkAdministration.Left;
      cbxLocation.Left := dtpkAdministration.Left;
      cbxConsult.Left := dtpkAdministration.Left;

      dtpkAdministration.Width := grpDate.Width - (2 * pnlSpacer0.Width);
      cbxOrderedBy.Width := dtpkAdministration.Width;
      cbxInterviewer.Width := dtpkAdministration.Width;
      cbxLocation.Width := dtpkAdministration.Width;
      cbxConsult.Width := dtpkAdministration.Width;

      lblDisplay.Height := TextHeight('Yy', gvFont);
      lblDisplay.Top := lblDisplay.Height div 2;
      lblDisplay.Width := TextWidth(lblDisplay.Caption + 'XXX', gvFont);
      lblDisplay.Left := 10; //(pnlRight.Width - lblDisplay.Width) div 2;

      pnlButtons.Height := 3 * ButtonHeight div 2;

      rbAll.Height := ButtonHeight;
      rbOne.Height := ButtonHeight;
      rbAll.Width := TextWidth(rbOne.Caption + 'XXX', gvFont);
      rbOne.Width := rbAll.Width;
      rbAll.Top := 2 * lblDisplay.Height;
      rbOne.Top := 1 * lblDisplay.Height + rbAll.Top + rbAll.Height;
      rbAll.Left := 10;
      rbOne.Left := rbAll.Left;

      btnPatientEntry.Top := 5;
      btnPatientEntry.Width := ButtonWidth(btnPatientEntry.Caption);
      btnPatientEntry.Height := ButtonHeight;

      btnStaffEntry.Top := 5;
      btnStaffEntry.Width := btnPatientEntry.Width;
      btnStaffEntry.Height := ButtonHeight;
      btnStaffEntry.Left := 2;

    end;

  if gvUseScreenReader then
    begin
      Caption := 'Administer ' + gvNameOfInstrument + ' for ' + Patient.Name;
      grpOrderer.Caption := 'Ordered &By: Vista combo box';
      grpInterviewer.Caption := '&Interviewer: Vista combo box';
      grpLocation.Caption := 'Visit &Location: Vista combo box';
      grpConsult.Caption := 'Link &With Consult: Vista combo box';

      rbAll.Caption := 'Administer all &Questions at once.';
      rbOne.Caption := 'Administer &One question at a time.';

      //btnStaffEntry.caption := btnStaffEntry.Caption + ' button';
      //btnPatientEntry.caption := btnPatientEntry.Caption + ' button';

    end;
end;

procedure TfrmMHA_Admin.cbxOrderedByNeedData(Sender: TObject;
  const StartFrom: string; Direction, InsertAt: Integer);
begin
  inherited;
  (Sender as TORComboBox).ForDataUse(Staff_OPS.LoadStaffSubSet(StartFrom,
    Direction, DataModule1.RPCBroker1));
end;


function TfrmMHA_Admin.IsSecureDesktopInstalled: BOOLEAN;
begin
  Result := False;

  if (FileExists(LocalFiles.MHAPath + 'YS_MHA_VASD.exe') and // SD is present
    //(LMDSysInfo1.Platform = 'Windows NT') and //is this an NT-class OS (Pro)
    (FileExists(GetWindowsSystemFolder + '\dwlGina2.dll'))) then
    {//Special system DLL is installed}
    //if wlIsGinaInstalled = 1 {...and activated} then
      result := True;
end;


procedure TfrmMHA_Admin.btnStaffEntryClick(Sender: TObject);
begin
  if (cbxLocation.ItemIndex = -1) and (DataBaseMode = dbVista) then
    begin
      MessageDlg('Please select a "Visit Location" before proceeding.', mtError,
        [mbOK], 0);
      Exit;
    end;

  FCancelled := false;
  FIsStaffEntry := True;
  FIsAllItems := rbAll.Checked;
  Close;
end;


procedure TfrmMHA_Admin.btnPatientEntryClick(Sender: TObject);
begin
  if (cbxLocation.ItemIndex = -1) and (DataBaseMode = dbVista) then
    begin
      MessageDlg('Please select a "Visit Location" before proceeding.', mtError,
        [mbOK], 0);
      Exit;
    end;

  FCancelled := false;
  FIsStaffEntry := false;
  FIsAllItems := rbAll.Checked;
  Close;
end;

procedure TfrmMHA_Admin.cbxInterviewerChange(Sender: TObject);
var
  tmpDUZ                      : string;
begin
  tmpDUZ := IntToStr(cbxInterviewer.ItemIEN);
  FInterviwerDUZ := tmpDUZ;
end;

procedure TfrmMHA_Admin.cbxOrderedByChange(Sender: TObject);
var
  tmpDUZ                      : string;
begin
  tmpDUZ := IntToStr(cbxOrderedBy.ItemIEN);

  if not (tmpDUZ = '0')
    and not (tmpDUZ = FOrderedByDUZ) then
    FOrderedByDUZ := tmpDUZ; //.Load(tmpDUZ, DataModule1.RPCBroker1);

end;

procedure TfrmMHA_Admin.dtpkAdministrationChange(Sender: TObject);
begin
  FDateGiven := DateTimeToStr(dtpkAdministration.DateTime);
end;

procedure TfrmMHA_Admin.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin

  if pnlLeft.Visible then
    gvLastLocation := FLocationIEN + U + cbxLocation.Text;

  gvShowAllOrOneItem := rbAll.Checked;

  MHA_Registry.SaveRegistry;

  Action := caFree;
end;


procedure TfrmMHA_Admin.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = [ssALT]) and (Key = vk_F4) then
    Close;

  if Key = vk_Escape then
    begin
      Close;
      Key := 0;
    end;
end;

procedure TfrmMHA_Admin.cbxInterviewerDropDown(Sender: TObject);
begin
  // OnFormShow does not load cbxInterviewer items because the method is very
   // slow, and in most cases the Administration.AdministeredBy will indeed be the
   // User. This method will load the items the first time the user clicks
   // on the combobox's down arrow (i.e., when item.count = 0

  if (cbxInterviewer.Items.Count = 0) then
    begin
      cbxInterviewer.InitLongList(ThisUser.Name);
      // Patch 129 fix for DUZ's with greater than 9 digits ; changed StrtoInt to StrToInt64 llh
      cbxInterviewer.SelectByIEN(StrToInt64(ThisUser.DUZ));
    end;
end;

procedure TfrmMHA_Admin.cbxOrderedByDropDown(Sender: TObject);
begin
  // OnFormShow does not load cbxOrderedBy items because the method is very
   // slow, and in most cases the Administration.Orderer will indeed be the
   // User. This method will load the items the first time the user clicks
   // on the combobox's down arrow (i.e., when item.count = 0

  if (cbxOrderedBy.Items.Count = 0) then
    begin
      cbxOrderedBy.InitLongList(ThisUser.Name);
      // Patch 129 fix for DUZ's with greater than 9 digits ; changed StrtoInt to StrToInt64 llh
      cbxOrderedBy.SelectByIEN(StrToInt64(ThisUser.DUZ));
    end;
end;

procedure TfrmMHA_Admin.cbxLocationChange(Sender: TObject);
begin
  if cbxLocation.ItemIndex <> -1 then
    begin
      FLocationIEN := IntToStr(cbxLocation.ItemIEN);
      FLocationName := cbxLocation.Text;
    end
  else
    begin
      FLocationIEN := '';
      FLocationName := '';
    end;
end;

procedure TfrmMHA_Admin.cbxlocationDropDown(Sender: TObject);
begin
 // OnFormShow does not load cbxLocation items because the method is very
 // slow, and in most cases the location will not change.
 // This method will load the items the first time the user clicks
 // on the combobox's down arrow (i.e., when item.count = 0

  if (cbxLocation.Items.Count = 0) then
    begin
      cbxLocation.InitLongList(cbxLocation.text);
      cbxLocation.SelectByIEN(StrToIntDef(Piece(gvLastLocation, U, 1), 1));
    end;
end;

(*
procedure TfrmMHA_Admin.cbxConsultChange(Sender: TObject);
begin
  if cbxLocation.ItemIndex <> -1 then
    begin
      lvAdministration.ConsultIEN :=
        Piece(cbxConsult.Items[cbxConsult.ItemIndex], U, 7);
    end
  else
    begin
      lvAdministration.ConsultIEN := '';
    end;
end;

procedure TfrmMHA_Admin.cbxConsultDropDown(Sender: TObject);
begin
  // OnFormShow does not load cbxConsult items because the method is very
  // slow, and in most cases the consult will not change.
  // This method will load the items the first time the user clicks
  // on the combobox's down arrow (i.e., when item.count = 0

  if (cbxConsult.Items.Count = 0) then
    begin
      cbxConsult.Items.AddStrings(Consult_OPS.LoadConsultSubSet(Patient.DFN,
        DataModule1.RPCBroker1));
      cbxConsult.DropDownCount := cbxConsult.Items.Count - 1;
    end;
end;*)

procedure TfrmMHA_Admin.cbxlocationNeedData(Sender: TObject;
  const StartFrom: string; Direction, InsertAt: Integer);
begin
  inherited;
  (Sender as TORComboBox).ForDataUse(SubSetOfLocations(StartFrom,
    Direction, DataModule1.RPCBroker1));
end;

//procedure TfrmMHA_Admin.imgBackupClick(Sender: TObject);
//begin
//  Close;
//end;

procedure TfrmMHA_Admin.FormResize(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  btnPatientEntry.Left := pnlButtons.width - btnPatientEntry.Width -
    pnlSpacer0.Width;

  // Reset these, used in placement of dialogs
  gvLeft := Left;
  gvTop := Top;
  gvHeight := Height;
  gvWidth := Width;
end;

end.

