unit fMHA_AdministerOneItem;

// WIN7SCROLLBUG. Part of workaround for Windows 7 glitch.
// This form had an issue executing on Windows 7 because of the "Windows 7
// enforced 16-bit ranges scrolling window bug." This glitch is evident when
// creating forms with a large number of embedded components. This is usually
// not a problem with most MH instruments. However, there are some instruments
// that are very large, and adding all those components to the scrollbox exceeds
// the allowable scroll range value, thus triggering the bug.
// This bug has been corrected via a workaround that is implemented at various
// locations in this code. To find these locations, search for the
// WIN7SCROLLBUG LABEL.

interface

uses
  Windows,
  Messages,
  SysUtils,
  DateUtils,
  Classes,
  Graphics,
  Controls,
  Contnrs,
  Forms,
  Printers,
  StdCtrls,
  ComCtrls,
  ExtCtrls,
  ORCtrls,
  Mask,
  RzRadGrp,
  RzSpnEdt,
  uIA_Instrument,
  uIA_Question,
  uMHA_Administration,
  Menus,
  RzChkLst,
  RzLstBox,
  Buttons,
  ExTrackBar,
  RzLabel,
  RzPanel,
  JvValidateEdit,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons,
  JvProgressBar,
  JvXPProgressBar,
  JvExStdCtrls,
  JvEdit,
  JvExMask,
  JvToolEdit,
  JvDatePickerEdit,
  JvXPCheckCtrls;


type
  // Do not change this name: required for Secure Desktop
  TfrmMHA3_AdministerSingleItems = class(TForm)
    pnlTop: TPanel;
    pnlBottom: TPanel;
    btnFinished: TJvXPButton;
    btnCancel: TJvXPButton;
    cbSpeedTab: TJvXPCheckbox;
    lblHint: TLabel;
    pgctrlAllItems: TPageControl;
    pnlStatus: TRzStatusBar;
    prgbrStatus: TJvXPProgressBar;
    mnuFull: TMainMenu;
    mnuFile: TMenuItem;
    mnuExit: TMenuItem;
    mnuView: TMenuItem;
    mnuTools: TMenuItem;
    mnuMetric: TMenuItem;
    N2: TMenuItem;
    mnuOptions: TMenuItem;
    mnuHelp: TMenuItem;
    mnuOnlineSupport: TMenuItem;
    N1: TMenuItem;
    mnuAbout: TMenuItem;
    btnPrior: TJvXPButton;
    btnNext: TJvXPButton;
    btnReview: TJvXPButton;
    mnuPrintBlank: TMenuItem;
    N3: TMenuItem;

    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure ConfigureForm(Sender: TObject);
    procedure BuildForm(WhichOne: Integer);
    procedure BuildEvents(WhichOne: Integer);
    procedure BuildButtonPanel;

    function GetDefaultEntry(QID: integer): string;
    function GetFormerAnswer(QID: string): string;
    function GetChoiceOnly(StringIn: string): string;
    function GetLegacyValue(aQuestionID, ItemIndexStr: string): string;
    function GetChoiceID(aQuestionID, ItemIndexStr: string): string;
    function GetItemIndex(aQuestionID, ChoiceID: string): integer;
    function GetStaticText(Where: integer): string;

    //--------------------------------------------------------------------------
    procedure HideAllItems;
    function IsEnabled(TagIn: integer): Boolean;
    function IsQuestionRequired(QID: string): Boolean;
    procedure UpdateComponentsAndAnswerList(QID, Answer: string);
    procedure UpdateTallyNotAskedEtc;
    //procedure GetScaleScore;

    //--------------------------------------------------------------------------
    //procedure MakeActiveThisButton(RadioGroupName: string; KeyIn: Word);
    function KeyPressRadioGroup(K: word; CurrentRadioGrp: string): Boolean;
    function KeyPressTrackBar(K: word; LastTrackBar: string): boolean;
    function KeyPressListBox(K: word; CurrentListBox: string): boolean;
    function KeyPressComboBox(K: word; CurrentComboBox: string): Boolean;
    procedure OnExitEvent(sender: TObject);
    procedure OnChangeEvent(sender: TObject);
    procedure OnClickRadioGroup(Sender: TObject);
    procedure OnClickListBox(Sender: TObject);
    procedure OnClickComboBox(Sender: TObject);
    procedure OnClickORComboBox(Sender: TObject);
    procedure OnChangeTrackBar(Sender: TObject);
    procedure OnChangeCheckList(Sender: TObject;
      Index: Integer; NewState: TCheckBoxState);
    procedure DoubleClickMessage(Sender: TObject);
    procedure CheckForRule(QID, Answer: string);
    //function CheckForCustomOperation: Boolean;
    procedure UpdateStatusBar(NewCaption: string);
    procedure RecordAnswer(QID, aAnswerValue, aAnswerText,
      aIsMultiChoice: string; aItemIndex: Integer);
    procedure GoToThisQuestion(QuestionID: Integer);
    procedure GoToNextQuestion(aName: string);
    procedure GoToPriorQuestion(aName: string);
    function IntroOrSectionOrLabel(aName: string): Integer;

    //procedure tbctrlRightChange(Sender: TObject);
    procedure cmbStaffNeedData(Sender: TObject;
      const StartFrom: string; Direction, InsertAt: Integer);
    //procedure cmbConsultNeedData(Sender: TObject;
    //  const StartFrom: string; Direction, InsertAt: Integer);
    procedure btnCancelClick(Sender: TObject);
    procedure btnFinishedClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ProcessAnswer(QID, Answer: string);
    procedure cbSpeedTabClick(Sender: TObject);
    function VerifyAnswers: boolean;
    procedure pgctrlAllItemsChange(Sender: TObject);
    procedure btnReviewClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnPriorClick(Sender: TObject);
    procedure UpdateButtons(Count, Current: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure mnuPrintBlankClick(Sender: TObject);
    procedure mnuOptionsClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    function CloseThisOne(WhichOne: Integer): string;
    procedure mnuMetricClick(Sender: TObject);
    //procedure mnuViewClick(Sender: TObject);
    procedure mnuOnlineSupportClick(Sender: TObject);
    procedure ChangePT;
    procedure HandleProgressNote(WhichOne: integer);
    function AdminFinalCheck(AAdmin: TAdministration): string;
    procedure LoadVistaText(InstrumentName: string);
    procedure LoadStaticText(InstrumentName: string);
    function GetVistaText(Where: integer): string;
  private
    { Private declarations }
    // WIN7SCROLLBUG
    // Moved these here, as they are shared by FormCreate and FormShow
    formRectOriginal :TRect;
    formWidth, formHeight: integer;
    // END WIN7SCROLLBUG

    procedure MenuSelection(Sender: TObject);
    function SaveAllAnswers(WhichOne: Integer): boolean;
    procedure AgmentAnswerList(WhichOne, aTag, aItemIndex: Integer;
      aText: string);
    procedure RecordLongInstrument(TagInt, IndexInt: integer);
    procedure SetupLongInstrument;
  public
    { Public declarations }
  end;

var
  frmMHA3_AdministerSingleItems: TfrmMHA3_AdministerSingleItems;


implementation

{$R *.DFM}

uses
  uIA_Const,
  uMHA_Const,
  uMHA_Registry,
  fIA_dialogs, Dialogs,
  uIA_GUIControl,
  uMHA_Proctor,
  uIA_Introduction,
  uIA_SectionLabel,
  uIA_Display,
  fMHA_ReviewAnswers,
  uMHA_Administration_OPS,
  uMHA_Instrument_OPS,
  uIA_Answer,
  uIA_Scale,
  uIA_Key,
  uIA_Key_OPS,
  uIA_Utils,
  uMHA_LocalFiles, // Crash file
  uIA_Build,
  uIA_Patient,
  uIA_ThisUser,
  uIA_Staff_OPS,
  uIA_Consult_OPS,
  uMHA_Rule_OPS,
  uMHA_ResourceFile_OPS,
 // fMHA_Print_Blank,
  uMHA_DM,
  fMHA_About,
  fMHA_ProgressNote,
  fMHA_Print_Blank;


const
  lcStartComponent            : integer = 20;
  lcSpacer                    : Integer = 15;
  NO_WARNING                  : string = 'No Warning';
  WARNING                     : string = 'Warning';
  CANCEL                      : string = 'CANCEL';
  SKIP_WARNING                : string = 'Skip Warning';

var
  lvTabSheet                  : TTabSheet;
  lvScrollBox                 : TScrollBox;


  lvLongInstrumentLastItem,
    lvLongInstrumentStartQID,
    lvLongInstrumentCount,
    lvCurrentInstrument       : Integer;

  lvInitialize,
    lvIsLongInstrument        : Boolean;

  lvLongInstrumentTChoiceID,
    lvLongInstrumentFChoiceID,
    lvStaticText,
    lvVistaText,
    lvShutDownMethod,
    lvFirstQuestionID,
    lvFinalQuestionID,
    lvLastQuestionID,
    lvQuestionsNotAsked,
    lvQuestionsSkipped,
    lvRequiredQuestionsSkipped,
    lvComponentName,
    lvVerifyMessageSeen       : string;
  lvInstrument                : TInstrument;
  lvCrashFile                 : TStringList;


procedure TfrmMHA3_AdministerSingleItems.FormCreate(Sender: TObject);
begin

  // WIN7SCROLLBUG
	// Get the dialog's current location
  GetWindowRect(Self.Handle,formRectOriginal) ;
  formWidth := formRectOriginal.right - formRectOriginal.left;
  formHeight := formRectOriginal.bottom - formRectOriginal.top;

  self.Visible := false;

	// The workaround is to move the dialog to (-32768, -32768) while child window content is being created.

  MoveWindow(Self.Handle, -32768, -32768, formWidth, formHeight, FALSE) ;

	// Now create the questionnaire content
  // END WIN7SCROLLBUG

  lvInitialize := True; // defeat OnChange, OnExit & OnEnter events
  Position := poDesigned;
  lvInstrument := TInstrument.Create;
  lvCrashFile := TStringList.Create;

  gvQuestionOrder := 1;
  lvFirstQuestionID := '';
  lvLastQuestionID := '';
  lvFinalQuestionID := '';
  ConfigureForm(Sender);

  ShowModal;
end;


procedure TfrmMHA3_AdministerSingleItems.ConfigureForm(Sender: TObject);
var
  i                           : integer;
  InstrumentMenuName          : TMenuItem;

  // WIN7SCROLLBUG
    nHeight : integer;
    dlgRectMoved,
    rect : TRect;
    m_yTopDlg,
    m_yBottomLastCtrl: integer;
  // END WIN7SCROLLBUG

begin

  // WIN7SCROLLBUG
  nHeight := 18;
  GetWindowRect(Self.Handle, dlgRectMoved);  // PageControl1
  m_yTopDlg := dlgRectMoved.top;
  // END WIN7SCROLLBUG

  lvCurrentInstrument := 0;

  with GUI_Control do
    begin
      Scaled := True;
      AutoScroll := False;
      AutoSize := False;
      PixelsPerInch := 120;

      KeyPreview := True;
      HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
      VertScrollBar.Range := 0;
      Font.Assign(gvFont);

      WindowState := wsNormal;
      Height := GetHeight(ButtonHeight * 20);
      Width := GetWidth(ButtonWidthNormal * 12);
      Left := GetLeft(Width);
      Top := GetTop(Height);
      if CheckWidth(Width) or CheckHeight(Height) then
        WindowState := wsMaximized;
       // Make sure the control buttons are visible
      if (Top + Height) > Screen.WorkAreaHeight then
        Top := Screen.WorkAreaHeight - Height;

      if gvUseScreenReader then
        begin
          Left := 0;
          Width := Screen.Width;
        end;

      Refresh; // resets alClient dimensions

      Caption := TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).InstrumentName +
        ': ' + Patient.Name + '  ';

      gvScrollBarWidthWS := TextWidthbyFont(Font.handle, 'XXXX');
      prgbrStatus.Left := pnlStatus.Width div 2;
      prgbrStatus.Width := (pnlStatus.Width div 2) - 20;
      prgbrStatus.Height := 2 * TextHeight('Yy', Font);

  // Build the forms selected by the user. Each instrument gets its own
  // TabControl, ScrollBox and main-menu option.
      with pgctrlAllItems do
        for i := 0 to Administration_OPS.Administrations.Count - 1 do
          begin
            lvTabSheet := TTabSheet.Create(Self);
            with lvTabSheet do
              begin
                PageControl := pgctrlAllItems;
                Caption := TAdministration(
                  Administration_OPS.Administrations[i]).InstrumentName;
                Name := 'Tab' + IntToStr(i);
                lvTabSheet.ParentFont := False;
                lvTabSheet.Width := ClientWidth;
              end;

            lvScrollBox := TScrollBox.Create(Self);
            lvScrollBox.Name := 'Form' + IntToStr(i);
            lvScrollBox.Parent := lvTabSheet;
            lvScrollBox.Align := alClient;
            lvScrollBox.Width := lvTabSheet.Width;
            lvScrollBox.AutoScroll := True;
            lvScrollBox.VertScrollBar.visible := False;
            lvScrollBox.ParentFont := False;

            // Menu Stuff
            InstrumentMenuName := TMenuItem.Create(Self);
            InstrumentMenuName.Name := 'mnu' + IntToStr(i);
            InstrumentMenuName.Caption := lvTabSheet.Caption;
            InstrumentMenuName.OnClick := MenuSelection;
            mnuView.Add(InstrumentMenuName);

            BuildForm(i);
            BuildEvents(i);
            HideAllItems;
            // Make first question visible in each instrument
            GoToThisQuestion(StrToIntDef(Piece(lvFirstQuestionID, U, i + 1), -1));
          end;

      BuildButtonPanel;

      lvShutDownMethod := WARNING;
      lvLastQuestionID := Piece(lvFirstQUestionID, U, 1);
      lvCurrentInstrument := 0;
      pgctrlAllItems.ActivePageIndex := 0;
      pgctrlAllItemsChange(Sender);

    end; // with GUI Control

    // WIN7SCROLLBUG
    GetWindowRect(Self.Handle, rect);
    m_yBottomLastCtrl := rect.bottom + nHeight;
    // END WIN7SCROLLBUG

end;


procedure TfrmMHA3_AdministerSingleItems.FormShow(Sender: TObject);
begin

  OpenFormName := ofSingle;

  cbSpeedTab.Checked := gvSpeedTab;
  if cbSpeedTab.Checked then
    lblHint.caption := 'Hint: Use the number key of the item to speed data entry.'
  else
    lblHint.caption := 'Hint: Press carriage return to move to next question.';

  lvInitialize := False; // return to normal component functioning
  screen.cursor := crDefault;

  if Proctor.DataEntryMode = emPatient then
    begin
      mnuMetric.Visible := false;
      mnuOnLineSupport.Visible := false;
      mnuHelp.Visible := false;
    end;

  //WIN7SCROLLBUG
	// And now move the dialog back to its original location so it can be displayed.

  MoveWindow(Self.Handle, formRectOriginal.left,
  formRectOriginal.top, formWidth, formHeight, TRUE) ;
  self.Visible := true;

  // END WIN7SCROLLBUG

end;


procedure TfrmMHA3_AdministerSingleItems.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if gvCPRSShutDown then
    Application.Terminate;
end;

procedure TfrmMHA3_AdministerSingleItems.FormDestroy(Sender: TObject);
begin
//  if lvInstrument <> nil then
  lvInstrument.Free;

//  if lvCrashFile <> nil then
  lvCrashFile.Free;
end;

procedure TfrmMHA3_AdministerSingleItems.SetupLongInstrument;
// These are simple instruments (no rules, all True/False, and no skip logic) but long,
// i.e., MMPI2, MCMI2, MCMI3 & MBMD. Using a "for" loop to check all the components
// after a user-response produces sluggish reactions not seen on shorter (less than
// 100 questions) instruments.
begin
  lvLongInstrumentLastItem := 0;
  lvQuestionsSkipped := '';
  lvIsLongInstrument := false;
  lvLongInstrumentCount := lvInstrument.NumberOfQuestions - 1; // one less since start is 0, not 1

  if POS('MMPI2', lvInstrument.InstrumentName) > 0 then
    begin
      lvIsLongInstrument := true;
      lvLongInstrumentStartQID := 2188;
      lvLongInstrumentTChoiceID := '612';
      lvLongInstrumentFChoiceID := '613';
    end;

  if POS('MCMI2', lvInstrument.InstrumentName) > 0 then
    begin
      lvIsLongInstrument := true;
      lvLongInstrumentStartQID := 1211;
      lvLongInstrumentTChoiceID := '612';
      lvLongInstrumentFChoiceID := '613';
    end;

  if POS('MCMI3', lvInstrument.InstrumentName) > 0 then
    begin
      lvIsLongInstrument := true;
      lvLongInstrumentStartQID := 1386;
      lvLongInstrumentTChoiceID := '688';
      lvLongInstrumentFChoiceID := '689';
    end;

  if POS('MMP2S', lvInstrument.InstrumentName) > 0 then
    begin
      lvIsLongInstrument := true;
      lvLongInstrumentStartQID := 1818;
      lvLongInstrumentTChoiceID := '612';
      lvLongInstrumentFChoiceID := '613';
    end;

  if POS('MBMD', lvInstrument.InstrumentName) > 0 then
    begin
      lvIsLongInstrument := true;
      lvLongInstrumentStartQID := 1046;
      lvLongInstrumentTChoiceID := '612';
      lvLongInstrumentFChoiceID := '613';
    end;

  if POS('MMPI-2-RF', lvInstrument.InstrumentName) > 0 then
    begin
      lvIsLongInstrument := true;
      lvLongInstrumentStartQID := 6029;
      lvLongInstrumentTChoiceID := '612';
      lvLongInstrumentFChoiceID := '613';
    end;
end;

procedure TfrmMHA3_AdministerSingleItems.cmbStaffNeedData(Sender: TObject;
  const StartFrom: string; Direction, InsertAt: Integer);
begin
  inherited;
  if DataBaseMode = dbVista then
    (Sender as TORComboBox).ForDataUse(Staff_OPS.LoadStaffSubSet(StartFrom,
      Direction, DataModule1.RPCBroker1));
end;

//procedure TfrmMHA_AdministerOneItem.cmbConsultNeedData(Sender: TObject;
//  const StartFrom: string; Direction, InsertAt: Integer);
//begin
//  inherited;
//  if DataBaseMode = dbVista then
//    (Sender as TORComboBox).ForDataUse(Consult_OPS.LoadConsultSubSet(Patient.DFN,
//      DataModule1.RPCBroker1));
//end;


procedure TfrmMHA3_AdministerSingleItems.BuildButtonPanel;
var
  Space                       : Integer;
begin
  with GUI_Control do
    begin
      with btnNext do
        begin
          Height := ButtonHeight;
          Width := ButtonWidth('Review Answers'); // Longest caption
          Top := Height div 4;
        end;
      with btnPrior do
        begin
          Height := btnNext.Height;
          Width := btnNext.Width;
          Top := btnNext.Top;
          Enabled := False;
        end;
      with btnReview do
        begin
          Height := btnNext.Height;
          Width := btnNext.Width;
          Top := btnNext.Top;
        end;
      with btnCancel do
        begin
          Height := btnNext.Height;
          Width := btnNext.Width;
          Top := btnNext.Top;
        end;
      with btnFinished do
        begin
          Height := btnNext.Height;
          Width := btnNext.Width;
          Top := btnNext.Top;
          Enabled := False;
        end;

      with cbSpeedTab do
        begin
          Height := SetLabelHeight(gvFont.Size);
          Width := TextWidth(Caption + 'XXX', gvFont);
          Top := 3 * btnCancel.Height div 2;
        end;

      with lblHint do
        begin
          Height := SetLabelHeight(gvFont.Size);
          Width := TextWidth(Caption + 'XXX', gvFont);
          Top := 3 * btnCancel.Height div 2;
        end;

      Space := (pnlBottom.Width - (5 * btnNext.Width)) div 6;
      pnlBottom.Height := 5 * btnCancel.Height div 2;
      btnNext.Left := Space;
      btnPrior.Left := btnNext.Width + 2 * Space;
      btnReview.Left := (2 * btnNext.Width) + (3 * Space);
      btnCancel.Left := (3 * btnNext.Width) + (4 * Space);
      btnFinished.Left := (4 * btnNext.Width) + (5 * Space);
      cbSpeedTab.Left := Space;
      lblHint.Left := btnPrior.Left;
      pnlStatus.Height := GetStandardHeight(gvFont);
    end; // with GUI_Control
end;


//------------------------------------------------------------------------------
//       Highlight operations
//------------------------------------------------------------------------------

procedure TfrmMHA3_AdministerSingleItems.HideAllItems;
var
  i                           : Integer;
begin
  for i := lcStartComponent to ComponentCount - 1 do
    begin
      if Components[i] is TRzRadioGroup then
        with TRzRadioGroup(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
      if Components[i] is TEdit then
        with TEdit(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
      if Components[i] is TJvValidateEdit then
        with TJvValidateEdit(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
      if Components[i] is TComboBox then
        with TComboBox(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
      if Components[i] is TORComboBox then
        with TORComboBox(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
      if Components[i] is TJvDatePickerEdit then
        with TJvDatePickerEdit(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
      if Components[i] is TExTrackBar then
        with TExTrackBar(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
      if Components[i] is TRzSpinEdit then
        with TRzSpinEdit(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
      if Components[i] is TLabel then
        with TLabel(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
      if Components[i] is TMemo then
        with TMemo(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
      if Components[i] is TMaskEdit then
        with TMaskEdit(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
      if Components[i] is TRzCheckList then
        with TRzCheckList(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
      if Components[i] is TRzListBox then
        with TRzListBox(Components[i]) do
          begin
            if Parent = lvScrollBox then
              Visible := False;
          end;
    end;
end;

(*
procedure TfrmMHA_AdministerOneItem.CenterComponentInScrollBox(WhichOne: Integer);
var
  i                           : Integer;
begin


  for i := lcStartComponent to ComponentCount - 1 do
    begin
      if i = WhichOne then
        begin
      if Components[i] is TEdit then
        if TEdit(Components[i]).Parent = lvScrollBox then
          begin
            TEdit(Components[i]).Visible := True;
            SetFocusedControl(TEdit(Components[i]));
            Break;
          end;
      if Components[i] is TMaskEdit then
        if TMaskEdit(Components[i]).Parent = lvScrollBox then
          begin
            TMaskEdit(Components[i]).Visible := True;
            SetFocusedControl(TMaskEdit(Components[i]));
            Break;
          end;
      if Components[i] is TJvValidateEdit then
        if TJvValidateEdit(Components[i]).Parent = lvScrollBox then
          begin
            TJvValidateEdit(Components[i]).Visible := True;
            SetFocusedControl(TJvValidateEdit(Components[i]));
            Break;
          end;
      if Components[i] is TJvDatePickerEdit then
        if TJvDatePickerEdit(Components[i]).Parent = lvScrollBox then
          begin
            TJvDatePickerEdit(Components[i]).Visible := True;
            SetFocusedControl(TJvDatePickerEdit(Components[i]));
            Break;
          end;
      if Components[i] is TRzSpinEdit then
        if TRzSpinEdit(Components[i]).Parent = lvScrollBox then
          begin
            TRzSpinEdit(Components[i]).Visible := True;
            SetFocusedControl(TRzSpinEdit(Components[i]));
            Break;
          end;
          if Components[i] is TRzRadioGroup then
            if TRzRadioGroup(Components[i]).Parent = lvScrollBox then
              begin
                if not TRzRadioGroup(Components[i]).Visible then
                  begin
            //TRzRadioGroup(Components[i]).Color := clred;
                    TRzRadioGroup(Components[i]).Visible := True;
                    SetFocusedControl(TRzRadioGroup(Components[i]));

                  end;
              end;

      if Components[i] is TRzListBox then
        if TRzListBox(Components[i]).Parent = lvScrollBox then
          begin
            TRzListBox(Components[i]).Visible := True;
            SetFocusedControl(TRzListBox(Components[i]));
            Break;
          end;
      if Components[i] is TRzCheckList then
        if TRzCheckList(Components[i]).Parent = lvScrollBox then
          begin
            TRzCheckList(Components[i]).Visible := True;
            SetFocusedControl(TRzCheckList(Components[i]));
            Break;
          end;
      if Components[i] is TComboBox then
        if TComboBox(Components[i]).Parent = lvScrollBox then
          begin
            TComboBox(Components[i]).Visible := True;
            SetFocusedControl(TComboBox(Components[i]));
            Break;
          end;
      if Components[i] is TORComboBox then
        if TORComboBox(Components[i]).Parent = lvScrollBox then
          begin
            TORComboBox(Components[i]).Visible := True;
            SetFocusedControl(TORComboBox(Components[i]));
            Break;
          end;
      if Components[i] is TExTrackBar then
        if TExTrackBar(Components[i]).Parent = lvScrollBox then
          begin
            TExTrackBar(Components[i]).Visible := True;
            SetFocusedControl(TExTrackBar(Components[i]));
            Break;
          end;
      if Components[i] is TMemo then
        if TMemo(Components[i]).Parent = lvScrollBox then
          begin
            TMemo(Components[i]).Visible := True;
            SetFocusedControl(TMemo(Components[i]));
            Break;
          end;

        end;
    end;

  // Place the control in the middle of the scroll box.
  if TopIn > lvScrollBox.Height div 2 then
    lvScrollBox.VertScrollBar.Position :=
      lvScrollBox.VertScrollBar.Position + TopIn - (lvScrollBox.Height div 2);

  // Ensure the proper right tab is active.
  if tbctrlRight.Visible then
    begin
      with lvScrollBox.VertScrollBar do
        for i := 0 to (RIGHT_TAB_MAX - 1) do
          if ((Position >= lvRightTab[i]) and (Position < lvRightTab[i + 1])) then
            tbctrlRight.TabIndex := i;
    end;
end; *)

(*
procedure TfrmMHA_AdministerOneItem.HighlightRadioGroup(Sender: TObject);
begin

  if Sender is TRzRadioGroup then
    with Sender as TRzRadioGroup do
      begin
        lvComponentName := Name;
       //ShowMessage('HRG vivible');
       // Visible := True;
        //CenterComponentInScrollBox(Top);
      end;

end;

procedure TfrmMHA_AdministerOneItem.HighlightTrackBar(Sender: TObject);
begin
  HideAllItems;
  if Sender is TExTrackBar then
    with Sender as TExTrackBar do
      begin
        Visible := True;
      end;
end;

procedure TfrmMHA_AdministerOneItem.HighlightEdit(Sender: TObject);
begin
  HideAllItems;

  if Sender is TEdit then
    with Sender as TEdit do
      begin
        Visible := True;
      end;
end;

procedure TfrmMHA_AdministerOneItem.HighlightMaskEdit(Sender: TObject);
begin
  HideAllItems;
  if Sender is TMaskEdit then
    with Sender as TMaskEdit do
      begin
        Visible := True;
      end;
end;

procedure TfrmMHA_AdministerOneItem.HighlightCheckListBox(Sender: TObject);
begin
  HideAllItems;

  with Sender as TRzCheckList do
    begin
      Visible := True;

      // It makes no sense at all, but placing the following line in OnEnter
      // event where it logically belongs will cause the first item to be
      // checked, regardless of the item that is clicked, when the CheckBox is
      // entered with a click event.
      // Font.Color := gvHighlightForeground;
      // The line is now in the OnChange event.
    end;
end;

procedure TfrmMHA_AdministerOneItem.HighlightListBox(Sender: TObject);
begin
  HideAllItems;

  if Sender is TRzListBox then
    with Sender as TRzListBox do
      begin
        Visible := True;
      end;
end;

procedure TfrmMHA_AdministerOneItem.HighlightComboBox(Sender: TObject);
begin
  HideAllItems;

  if Sender is TComboBox then
    with Sender as TComboBox do
      begin
        Visible := True;
      end;
end;

procedure TfrmMHA_AdministerOneItem.HighlightORComboBox(Sender: TObject);
begin
  HideAllItems;

  if Sender is TORComboBox then
    with Sender as TORComboBox do
      begin
        Visible := True;
      end;
end;

procedure TfrmMHA_AdministerOneItem.HighlighTJvDatePickerEdit(Sender: TObject);
begin
  HideAllItems;

  if Sender is TJvDatePickerEdit then
    with Sender as TJvDatePickerEdit do
      begin
        Visible := True;
      end;
end;

procedure TfrmMHA_AdministerOneItem.HighlightSpinEdit(Sender: TObject);
begin
  HideAllItems;

  if Sender is TRzSpinEdit then
    with Sender as TRzSpinEdit do
      begin
        Visible := True;
      end;
end;

procedure TfrmMHA_AdministerOneItem.HighlightCurrency(Sender: TObject);
begin
  HideAllItems;

  if Sender is TJvValidateEdit then
    with Sender as TJvValidateEdit do
      begin
        Visible := True;
      end;
end;

function TfrmMHA_AdministerOneItem.HighlightLabel(NameIn: string): integer;
var
  i                           : Integer;
  LabelToHighlight            : string;
begin
  Result := 0;
  LabelToHighlight := NameIn;
  Delete(LabelToHighlight, 1, 3);
  LabelToHighlight := 'lbl' + LabelToHighlight;

  for i := lcStartComponent to ComponentCount - 1 do
    begin
      if Components[i] is TLabel then
        with TLabel(Components[i]) do
          begin
            if Name = LabelToHighlight then
              begin
                Visible := True;
                //Top := 20;
                Result := Height;
              end;
          end;
    end;
end;

procedure TfrmMHA_AdministerOneItem.HighlightTrackBarLabel(NameIn: string;
  WhichOne: byte);
var
  i                           : Integer;
  LabelToHighlight            : string;
begin
  LabelToHighlight := NameIn;
  Delete(LabelToHighlight, 1, 3);
  if WhichOne = 1 then
    LabelToHighlight := cTrackBarLabel1 + LabelToHighlight
  else
    LabelToHighlight := cTrackBarLabel2 + LabelToHighlight;

  for i := lcStartComponent to ComponentCount - 1 do
    begin
      if Components[i] is TLabel then
        with TLabel(Components[i]) do
          begin
            if Name = LabelToHighlight then
              begin
                Visible := True;
              end;
          end;
    end;
end;

procedure TfrmMHA_AdministerOneItem.HighlightMemo(Sender: TObject);
begin
  HideAllItems;

  if Sender is TMemo then
    with Sender as TMemo do
      begin
        Visible := True;
      end;
end;*)

//------------------------------------------------------------------------------
//       Skip Questions: Enable/Disable Questions
//------------------------------------------------------------------------------

function TfrmMHA3_AdministerSingleItems.IsEnabled(TagIn: Integer): Boolean;
// Rule_OPS.SkipQuestions is a list of QuestionIDs. If the current question
// is one the list, then make it disabled.
var
  i                           : Integer;
begin
  Result := true;

  if Pos(U + InttoStr(TagIn) + U, Rule_OPS.SkipQuestions) > 0 then
    begin
      RecordAnswer(InttoStr(TagIn), CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION,
        CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION, YES {MultiChoice}, -1);
      // Set MultiChoice to YES for all question types; this ensures the correct
      // ChoiceID is saved. If the question is visited again, the MultiChoice will
      // set to its proper setting.
      Result := False;
    end;

  // Handle questions that were once set to "skipped by rule" but are now
  // okay to answer. Remove the "1156"

  if Pos(U + InttoStr(TagIn) + U, Rule_OPS.SkipQuestions) = 0 then
    begin
      for i := 0 to TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers.Count - 1 do
        if InttoStr(TagIn) = TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).QuestionID then
          if TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue =
            CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
            RecordAnswer(InttoStr(TagIn), NOT_ASKED, NOT_ASKED, YES {MultiChoice}, -1);
      // Set MultiChoice to YES for all question types; this ensures the correct
      // ChoiceID is saved. If the question is visited again, the MultiChoice will
      // set to its proper setting.
    end;
end;


procedure TfrmMHA3_AdministerSingleItems.UpdateComponentsAndAnswerList(QID, Answer: string);
// When a OnChange event occurs there may be one or more components that may need
// to be enabled/disabled, or inconsistencies to be flagged for the user.
// Note: The "Answer, Answer" for non-multi-choice items, below, is needed: the first will
// handle the SKIPPED items, the second Answer is the text of the answer.
var
  i                           : Integer;
begin

  for i := lcStartComponent to ComponentCount - 1 do
    begin

      if Components[i] is TRzRadioGroup then
        with TRzRadioGroup(Components[i]) do
          begin
            Enabled := IsEnabled(Tag {QuestionID});
            if IntToStr(Tag) = QID then
              begin
                if ItemIndex <> -1 then
                  RecordAnswer(QID,
                    IntToStr(ItemIndex),
                    GetChoiceOnly(Items[ItemIndex]),
                    YES {MultiChoice},
                    ItemIndex);
              end;
          end;

      if Components[i] is TRzListBox then
        with TRzListBox(Components[i]) do
          begin
            Enabled := IsEnabled(Tag {QuestionID});
            if IntToStr(Tag) = QID then
              begin // ItemIndex cannot be -1 here
                RecordAnswer(QID,
                  IntToStr(ItemIndex),
                  GetChoiceOnly(Items[ItemIndex]),
                  YES {MultiChoice},
                  ItemIndex);
              end;
          end;

      if Components[i] is TRzCheckList then
        with TRzCheckList(Components[i]) do
          begin
            Enabled := IsEnabled(Tag);
            // This component has already been saved. It has special processing
            // handled at the OnChange event.
          end;

      if Components[i] is TEdit then
        with TEdit(Components[i]) do
          begin
            Enabled := IsEnabled(Tag);
            if IntToStr(Tag) = QID then
              RecordAnswer(QID, Answer, Answer, NO {MultiChoice}, -1);
          end;

      if Components[i] is TJvValidateEdit then
        with TJvValidateEdit(Components[i]) do
          begin
            Enabled := IsEnabled(Tag);
            if IntToStr(Tag) = QID then
              RecordAnswer(QID, Answer, Answer, NO {MultiChoice}, -1);
          end;

      if Components[i] is TComboBox then
        with TComboBox(Components[i]) do
          begin
            Enabled := IsEnabled(Tag);
            if IntToStr(Tag) = QID then
              RecordAnswer(QID, Answer, Text, YES {MultiChoice}, ItemIndex);
          end;

      if Components[i] is TORComboBox then
        with TORComboBox(Components[i]) do
          begin
            Enabled := IsEnabled(Tag);
            if IntToStr(Tag) = QID then
              begin
                if Pos(cComboBoxConsult, Name) > 0 then
                  if Pos(cComboBoxConsult, Name) > 0 then
                    begin
                      RecordAnswer(QID, Piece(Items[ItemIndex], U, 1),
                        Piece(Items[ItemIndex], U, 7), NO {MultiChoice}, ItemIndex);
                      TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ConsultIEN :=
                        Piece(Items[ItemIndex], U, 1);
                    end
                  else
                    RecordAnswer(QID, Answer, Text, NO {MultiChoice}, ItemIndex);
              end;
          end;

      if Components[i] is TJvDatePickerEdit then
        with TJvDatePickerEdit(Components[i]) do
          begin
            Enabled := IsEnabled(Tag);
            if IntToStr(Tag) = QID then
              RecordAnswer(QID, Answer, Answer, NO {MultiChoice}, -1);
          end;

      if Components[i] is TExTrackBar then
        with TExTrackBar(Components[i]) do
          begin
            Enabled := IsEnabled(Tag);
            if IntToStr(Tag) = QID then
              RecordAnswer(QID, Answer, Answer, NO {MultiChoice}, Position);
          end;

      if Components[i] is TRzSpinEdit then
        with TRzSpinEdit(Components[i]) do
          begin
            Enabled := IsEnabled(Tag);
            if IntToStr(Tag) = QID then
              RecordAnswer(QID, Answer, Answer, NO {MultiChoice}, -1);
          end;

      if Components[i] is TLabel then
        begin
          if (Pos(cSection, TLabel(Components[i]).Name) > 0) or
            (Pos(cIntroduction, TLabel(Components[i]).Name) > 0) then // do nothing
          else
            TLabel(Components[i]).Enabled :=
              IsEnabled(TLabel(Components[i]).Tag);
        end;

      if Components[i] is TMemo then
        with TMemo(Components[i]) do
          begin
            Enabled := IsEnabled(Tag);
            if IntToStr(Tag) = QID then
              RecordAnswer(QID, Answer, Answer, NO {MultiChoice}, -1);
          end;

      if Components[i] is TMaskEdit then
        with TMaskEdit(Components[i]) do
          begin
            Enabled := IsEnabled(Tag);
            if IntToStr(Tag) = QID then
              RecordAnswer(QID, Answer, Answer, NO {MultiChoice}, -1);
          end;
    end; // for i
end;

//------------------------------------------------------------------------------
//       Key Press and Mouse Events
//------------------------------------------------------------------------------

function TfrmMHA3_AdministerSingleItems.KeyPressListBox(K: word; CurrentListBox: string): boolean;
var
  i, Choice                   : Integer;
begin
  Result := False;

  Choice := KeyToNumber(K);
  if Choice = -2 then
    Exit;
  if Choice = -1 then
    begin
      MessageBeep(MB_ICONASTERISK);
      if gvVisualBeeps then
        MsgDlgFrm('Wrong key.', MHA_CAPTION,
          mtError, [mbOk], 0);
      Exit;
    end;

  for i := 0 to ComponentCount - 1 do
    begin
      // Find the Combo Box that has focus
      if Components[i] is TRzListBox then
        if TRzListBox(Components[i]).Name = CurrentListBox then
          begin
            Choice := AdjustKeyToItemIndex(Choice,
              TRzListBox(Components[i]).Items.Count,
              TRzListBox(Components[i]).Items[0]);
            if Choice = -1 then
              begin
                MessageBeep(MB_ICONASTERISK);
                if gvVisualBeeps then
                  MsgDlgFrm('Wrong key.', MHA_CAPTION,
                    mtError, [mbOk], 0);
                Exit;
              end;
            TRzListBox(Components[i]).ItemIndex := Choice;
            Result := true;
            ProcessAnswer(IntToStr(TRzListBox(Components[i]).Tag),
              IntToStr(TRzListBox(Components[i]).ItemIndex));
          end;
    end;
end;


function TfrmMHA3_AdministerSingleItems.KeyPressRadioGroup(K: word; CurrentRadioGrp: string): boolean;
var
  i, Choice, TempInt          : Integer;

begin
  Result := False;

  Choice := KeyToNumber(K);
  if Choice = -2 then
    Exit;
  if Choice = -1 then
    begin
      MessageBeep(MB_ICONASTERISK);
      if gvVisualBeeps then
        MsgDlgFrm('Wrong key.', MHA_CAPTION,
          mtError, [mbOk], 0);
      Exit;
    end;

  for i := 0 to ComponentCount - 1 do
    begin
      // Find the radio group that has focus
      if Components[i] is TRzRadioGroup then
        begin
          if TRzRadioGroup(Components[i]).Name = CurrentRadioGrp then
            begin
              TempInt := TRzRadioGroup(Components[i]).ItemIndex;

              Choice := AdjustKeyToItemIndex(Choice,
                TRzRadioGroup(Components[i]).Items.Count,
                TRzRadioGroup(Components[i]).Items[0]);

              if Choice = -1 then
                begin
                  MessageBeep(MB_ICONASTERISK);
                  if gvVisualBeeps then
                    MsgDlgFrm('Wrong key.', MHA_CAPTION,
                      mtError, [mbOk], 0);
                  Exit;
                end;

              TRzRadioGroup(Components[i]).ItemIndex := Choice;

              // This little slight-of-hand is needed when the TRzGroupButton
              // has focus and it is already checked.
              if TempInt = TRzRadioGroup(Components[i]).ItemIndex then
                if gvSpeedTab then
                  GoToNextQuestion(CurrentRadioGrp);

              Result := True;
            end; // Current
        end; // if radio
    end; // for i

end;


function TfrmMHA3_AdministerSingleItems.KeyPressComboBox(K: word; CurrentComboBox:
  string): Boolean;
var
  i, Choice                   : Integer;
begin
  Result := False;

  Choice := KeyToNumber(K);
  if Choice = -2 then
    Exit;
  if Choice = -1 then
    begin
      MessageBeep(MB_ICONASTERISK);
      if gvVisualBeeps then
        MsgDlgFrm('Wrong key.', MHA_CAPTION,
          mtError, [mbOk], 0);
      Exit;
    end;

  for i := 0 to ComponentCount - 1 do
    begin
      // Find the Combo Box that has focus
      if Components[i] is TComboBox then
        if TComboBox(Components[i]).Name = CurrentComboBox then
          begin
            Choice := AdjustKeyToItemIndex(Choice,
              TComboBox(Components[i]).Items.Count,
              TComboBox(Components[i]).Items[0]);
            if Choice = -1 then
              begin
                MessageBeep(MB_ICONASTERISK);
                if gvVisualBeeps then
                  MsgDlgFrm('Wrong key.', MHA_CAPTION,
                    mtError, [mbOk], 0);
                Exit;
              end;
            TComboBox(Components[i]).ItemIndex := Choice;
            TComboBox(Components[i]).Text :=
              TComboBox(Components[i]).Items[TComboBox(Components[i]).ItemIndex];
            Result := true;
          end;
    end;
end;

function TfrmMHA3_AdministerSingleItems.KeyPressTrackBar(K: word; LastTrackBar: string): boolean;
var
  i, Choice                   : Integer;
begin
  Result := False;
  Choice := KeyToNumber(K);
  if Choice = -2 then
    Exit;
  if Choice = -1 then
    begin
      MessageBeep(MB_ICONASTERISK);
      if gvVisualBeeps then
        MsgDlgFrm('Wrong key.', MHA_CAPTION,
          mtError, [mbOk], 0);
      Exit;
    end;

  for i := lcStartComponent to ComponentCount - 1 do
    begin
      // Find the track bar that has focus
      if Components[i] is TExTrackBar then
        if TExTrackBar(Components[i]).Name = LastTrackBar then
          begin
            if ((TExTrackBar(Components[i]).Max >= Choice) or
              (TExTrackBar(Components[i]).Min <= Choice)) then
              begin
                TExTrackBar(Components[i]).Position := Choice;
                Result := True;
              end;
          end;
    end;
end;

procedure TfrmMHA3_AdministerSingleItems.RecordLongInstrument(TagInt, IndexInt: integer);
// This procedure is for simple instruments (no rules, all True/False, and no skip logic) but long,
// i.e., MMPI2, MCMI2, MCMI3 & MBMD. Using a "for" loop to check all the components
// after a user-response produces sluggish reactions not seen on shorter (less than
// 100 questions) instruments.
var
  place,
    j,
    i                         : integer;
  TempStr                     : string;
begin

  j := TagInt - lvLongInstrumentStartQID;
  lvCrashFile.Clear;

  // TagInt = -1 indicates the administration is finished
  if TagInt <> -1 then
    begin
       // Handle the Skipped item that is now answered; remove it from the list of skipped items.
      if TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerValue = SKIPPED then
        begin
          TempStr := inttostr(j + 1) + ', ';
          place := POS(TempStr, lvQuestionsSkipped);
          if place > 0 then
            System.Delete(lvQuestionsSkipped, place, length(TempStr));
          TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).NumberSkipped :=
            TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).NumberSkipped - 1;
        end;

      TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerValue := inttostr(IndexInt);
      TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerItemIndex := IndexInt;
      TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).QuestionID := inttostr(TagInt);
      TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).IsMultiChoice := True;

      if IndexInt = 0 then
        begin
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerChoiceID := lvLongInstrumentTChoiceID;
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerLegacyValue := 'T';
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerText := 'True';
        end;

      if IndexInt = 1 then
        begin
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerChoiceID := lvLongInstrumentFChoiceID;
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerLegacyValue := 'F';
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerText := 'False';
        end;

      if IndexInt = -1 then
        begin
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerValue := SKIPPED;
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerChoiceID := CHOICE_ID_OF_SKIPPED_QUESTION;
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerLegacyValue := '';
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[j]).AnswerText := '';
        end;
    end; // -1

  lvQuestionsNotAsked := '';
  lvQuestionsSkipped := '';

  TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).NumberAnswered := 0;
  // patch 129 llh - need to reset number skipped as well
  TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).NumberSkipped := 0;

  for i := 0 to lvLongInstrumentCount do
    begin

    // Not answered
      if (TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue = NOT_ASKED) then
        lvQuestionsNotAsked := lvQuestionsNotAsked + inttostr(i + 1) + ', ' // designator
      else
        TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).NumberAnswered :=
          TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).NumberAnswered + 1;

      // Skipped
      if (TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue = SKIPPED) then
        begin
          lvQuestionsSkipped := lvQuestionsSkipped + InttoStr(i + 1) + ', ';
          TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).NumberSkipped :=
            TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).NumberSkipped + 1;
        end;

      lvCrashFile.Add(
        inttostr(i + lvLongInstrumentStartQID) + U +
        inttostr(i + 1) + U + // designator
        TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue + U +
        TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerLegacyValue + U +
        TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerText + U +
        'TRUE' + U +
        inttostr(TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerItemIndex) + U +
        TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerChoiceID);
    end;

  lvCrashFile.SaveToFile(LocalFiles.CrashFileName(Patient.DFN,
    TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).OrdererDUZ,
    TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).InstrumentName));

  if Length(lvQuestionsSkipped) > 1 then
    UpdateStatusBar('Questions skipped: ' + lvQuestionsSkipped)
  else
    UpdateStatusBar('  ' + lvInstrument.Copyright);
end;

procedure TfrmMHA3_AdministerSingleItems.OnClickRadioGroup(Sender: TObject);
// This will capture both clicks and keypresses within the radio group
begin
  if lvInitialize then
    Exit;

  with Sender as TRzRadioGroup do
    begin
      if gvSpeedTab and (ItemIndex = -1) then
        Exit;

      if lvIsLongInstrument then
        RecordLongInstrument(Tag, ItemIndex)
      else
        ProcessAnswer(IntToStr(Tag), IntToStr(ItemIndex));

      if gvSpeedTab then
        GoToNextQuestion(Name);
    end;
end;

procedure TfrmMHA3_AdministerSingleItems.OnClickListBox(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  with Sender as TRzListBox do
    ProcessAnswer(IntToStr(Tag), IntToStr(ItemIndex));

end;

procedure TfrmMHA3_AdministerSingleItems.OnClickComboBox(Sender: TObject);
// This picks up the use of the arrow keys, home, & end.
begin
  if lvInitialize then
    Exit;

  with Sender as TComboBox do
    begin
      ProcessAnswer(IntToStr(Tag), IntToStr(ItemIndex));
      if gvSpeedTab then
        GoToNextQuestion(Name);
    end;

end;

procedure TfrmMHA3_AdministerSingleItems.OnClickORComboBox(Sender: TObject);
// This picks up the use of the arrow keys, home, & end.
begin
  if lvInitialize then
    Exit;

  with Sender as TORComboBox do
    begin
      ProcessAnswer(IntToStr(Tag), Text);
   //         if Pos(cComboBoxConsult, Name) > 0 then
//              begin
//                RecordAnswer(IntToStr(Tag), Piece(Items[ItemIndex], U, 1),
//                  Piece(Items[ItemIndex], U, 7), NO {MultiChoice}, ItemIndex);
//                TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ConsultIEN :=
//                  Piece(Items[ItemIndex], U, 1);
//              end
//            else
//
//          end;

      if gvSpeedTab then
        GoToNextQuestion(Name);
    end;

end;

procedure TfrmMHA3_AdministerSingleItems.CheckForRule(QID, Answer: string);
var
  TempMsg                     : string;
begin
  if lvInstrument.ListOfRules.Count = 0 then
    Exit;

  TempMsg := Rule_OPS.ProcessRule(
    TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]),
    lvInstrument,
    QID, Answer);

  if TempMsg <> '' then
    MsgDlgFrm(TempMsg, MHA_CAPTION, mtInformation,
      [mbOk], 0);
end;


procedure TfrmMHA3_AdministerSingleItems.ProcessAnswer(QID, Answer: string);
begin
  CheckForRule(QID, Answer);

  UpdateComponentsAndAnswerList(QID, Answer);

//  if CheckForCustomOperation then
//    begin
//      lvInitialize := True;
//      BuildEvents; // reloads answer list, which may have changed
//      lvInitialize := False;
//    end;

  if QID = Piece(lvFinalQuestionID, U, lvCurrentInstrument) then
    UpdateTallyNotAskedEtc;
end;


function TfrmMHA3_AdministerSingleItems.GetFormerAnswer(QID: string): string;
var
  i, iCount                   : Integer;
//  TempAns                     : string;
begin
  Result := '';

  iCount := TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers.Count - 1;
  if icount < 1 then
    Exit;

  for i := 0 to iCount do
    begin
      with TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]) do
        begin
          if QID = QuestionID then
            begin
              if IsMultiChoice then // Multiple Choice
              // Do not place AnswerValue = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION in
              // the exclusion list; it is needed to disable the item.
                begin
                  if (AnswerValue = NOT_ASKED) or (AnswerValue = SKIPPED) or
                    (AnswerValue = CHOICE_ID_OF_SKIPPED_QUESTION) or
                    (AnswerValue = CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION) then
                    Exit;

                  Result := AnswerValue;
                end
              else
                begin
                  Result := AnswerText;

                  if (AnswerText = SKIPPED) or
                    (AnswerText = CHOICE_ID_OF_SKIPPED_QUESTION) or
                    (AnswerText = BLANK) then
                    result := '';
                end;

              if Result = BLANK_DATE then
                Result := '';
            end; // if
        end; // with
    end; // for

end;

(*
procedure TfrmMHA_AdministerAllItems.GetScaleScore;
var
  Score,
    i, j, k                   : Integer;

begin
  Score := 0;

  if lvInstrument.ListOfScales.Count = 0 then
    Exit;

  for i := 0 to lvInstrument.ListOfScales.Count - 1 do
    begin
      for j := 0 to lvInstrument.ListOfKeys.Count - 1 do
        begin
          if TScale(lvInstrument.ListOfScales[i]).IEN =
            TKey(lvInstrument.ListOfKeys[j]).ScaleID then
            begin
              for k := 0 to TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAnswers.Count - 1 do
                begin
                  if ((TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAnswers[k]).QuestionID) =
                    (TKey(lvInstrument.ListOfKeys[j]).QuestionID)) and
                    ((TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAnswers[k]).AnswerText) =
                    (TKey(lvInstrument.ListOfKeys[j]).Target)) then
                    Score := Score + StrToInt(TKey(lvInstrument.ListOfKeys[j]).Value);

                end; // for k
            end;
        end; // for j

      gvScoreStr := gvScoreStr + '*' + TScale(lvInstrument.ListOfScales[i]).IEN + '~' +
        IntToStr(Score);

      Score := 0;
    end; // for i
end;*)

function TfrmMHA3_AdministerSingleItems.GetStaticText(Where: integer): string;
var
  Count,
    i                         : Integer;
  TempStr                     : string;
begin
  result := '';
  if lvStaticText = '' then
    Exit;
  Count := NumberOfPieces(lvStaticText, U);

  if Count = 0 then
    Exit;

  for i := 1 to Count do
    begin
      TempStr := Piece(lvStaticText, U, i);
      if IntToStr(Where) = Piece(TempStr, EQ, 1) then
        result := Piece(TempStr, EQ, 2);
    end;

end;

function TfrmMHA3_AdministerSingleItems.GetLegacyValue(aQuestionID, ItemIndexStr: string): string;
var
  i, Index                    : Integer;

begin
  Result := ' ';

  if (ItemIndexStr = CHOICE_ID_OF_SKIPPED_QUESTION) or
    (ItemIndexStr = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION) or
    (ItemIndexStr = CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION) or
    (ItemIndexStr = NOT_ASKED) or (ItemIndexStr = SKIPPED) then
    Exit;

  Index := StrToIntDef(ItemIndexStr, -1);
  if Index = -1 then
    Exit;

  for i := 0 to lvInstrument.ListOfQuestions.Count - 1 do
    if aQuestionID = TQuestion(lvInstrument.ListOfQuestions[i]).QuestionID then
      Result := Piece(TQuestion(lvInstrument.ListOfQuestions[i]).MultiChoice.choices[Index], U, 4);
end;


function TfrmMHA3_AdministerSingleItems.GetChoiceID(aQuestionID, ItemIndexStr: string): string;
var
  i, Index                    : Integer;

begin
  if ItemIndexStr = '' then
    begin
      Result := '';
      Exit;
    end;

  Result := CHOICE_ID_OF_SKIPPED_QUESTION;

  if (ItemIndexStr = NOT_ASKED) or (ItemIndexStr = SKIPPED) then
    Exit;

  if (ItemIndexStr = CHOICE_ID_OF_SKIPPED_QUESTION) or
    (ItemIndexStr = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION) or
    (ItemIndexStr = CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION) then
    begin
      Result := ItemIndexStr;
      Exit;
    end;

  Index := StrToIntDef(ItemIndexStr, -1);

  if Index = -1 then
    Exit;

  for i := 0 to lvInstrument.ListOfQuestions.Count - 1 do
    if aQuestionID = TQuestion(lvInstrument.ListOfQuestions[i]).QuestionID then
      Result := Piece(TQuestion(lvInstrument.ListOfQuestions[i]).MultiChoice.Choices[Index], U, 2);
end;


procedure TfrmMHA3_AdministerSingleItems.RecordAnswer(QID, aAnswerValue, aAnswerText,
  aIsMultiChoice: string; aItemIndex: Integer);
// The ListOfAnswers was constructed at Instrument.create with NOT_ASKED
// for AnswerValue. This will update the answer for the question in QID.
// Answer is not saved to Vista (or file, if off-line) until user is finished.
var
  i                           : Integer;
begin
  lvLastQuestionID := QID;

  for i := 0 to TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers.Count - 1 do
    begin
      if QID =
        TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).QuestionID then
        begin

          // aAnswerValue is: multiple-choice item index (as string), SKIPPED value, etc.
          // It is blank when in use with date, edit, memo, mask-edit and check-list box
          if lvInstrument.IsLegacy then
            TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerLegacyValue :=
              GetLegacyValue(QID, aAnswerValue);

          if aIsMultiChoice = YES then
            begin
              TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerItemIndex :=
                aItemIndex;
              TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerChoiceID :=
                GetChoiceID(QID, aAnswerValue);
            end
          else
            begin
              TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerItemIndex := -1;
              TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerChoiceID := '';
            end;

          // AnswerText is: multiple-choice item (less the designator), edit text,
          // date string, Mask edit text, etc.
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue :=
            aAnswerValue;
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerText :=
            aAnswerText;
          TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).IsMultiChoice :=
            aIsMultiChoice = YES;
        end;
    end; // for i

end;

procedure TfrmMHA3_AdministerSingleItems.UpdateTallyNotAskedEtc;
// Update the tally of Not Asked, and Skipped questions.
// A "Skipped" question is defined, operationally, as a question that has been
// highlighted but no answer was given, e.g., radio group itemIndex = -1, edit
// text blank.
var
  i                           : Integer;
begin
  lvCrashFile.Clear;
  lvQuestionsNotAsked := '';
  lvQuestionsSkipped := '';

  with TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]) do
    begin
      NumberAnswered := 0;
      NumberSkipped := 0;

      if ListOfAdminAnswers.Count = 0 then
        Exit;

      lvCrashFile.Add(InstrumentName + U +
        AdministeredBy + U +
        PatientID + U +
        OrdererDUZ + U +
        LocationIEN + U +
        LocationStr + U +
        DateTimeToStr(DateGiven));

      for i := 0 to ListOfAdminAnswers.Count - 1 do
        begin
          with TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]) do
            begin
      // We're counting the number of questions that have been "touched" by the
      // user which includes skipped questions.
              if (AnswerValue = NOT_ASKED) then
                lvQuestionsNotAsked := lvQuestionsNotAsked +
                  Designator + ', '
              else
                NumberAnswered := NumberAnswered + 1;

      // Skipped answers: not to be confused with a question that the user
      // has not yet viewed. The required questions are handled in verify answers
      // later. They will be included here.
              if ((AnswerValue = SKIPPED) or
                (AnswerValue = CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION) or
                (AnswerValue = CHOICE_ID_OF_SKIPPED_QUESTION)) then
                begin
                  NumberSkipped := NumberSkipped + 1;
                  lvQuestionsSkipped := lvQuestionsSkipped +
                    Designator + ', ';
                end;

              lvCrashFile.Add(
                QuestionID + U + Designator + U + AnswerValue + U + AnswerText + U +
                AnswerLegacyValue + U + BoolToStr(IsMultiChoice) + U +
                IntToStr(AnswerItemIndex) + U + AnswerChoiceID);

      // Special case: AUDC when first question is answered "Never." The
      // instrument is finished (which may appear to be locked up). Give
      // message that the user is done.
              if InstrumentName = 'AUDC' then
                if (QuestionID = '83') and (AnswerValue = '0') then
                  MsgDlgFrm('The AUDC is complete. ' + #13 +
                    'Click on the "Finish" button to save this administration.',
                    MHA_CAPTION, mtInformation, [mbOk], 0);

            end; // with TAnswer
        end; // for i

      if Length(lvQuestionsSkipped) > 1 then
        UpdateStatusBar('Questions skipped: ' + lvQuestionsSkipped)
      else
        UpdateStatusBar('  ' + lvInstrument.Copyright);

      if NumberAnswered = 0 then
        btnFinished.Enabled := false
      else
        btnFinished.Enabled := True;

      lvCrashFile.SaveToFile(LocalFiles.CrashFileName(Patient.DFN,
        OrdererDUZ, InstrumentName));

    end; // with  TAdministration
end;


procedure TfrmMHA3_AdministerSingleItems.UpdateStatusBar(NewCaption: string);
begin
  if lvInstrument.NumberOfQuestions > 0 then
    prgbrStatus.Position := Round(100 * (TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).NumberAnswered / lvInstrument.NumberOfQuestions));

  if TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).NumberSkipped > 0 then
    begin
      prgbrStatus.BarColorFrom := 00600000;
      prgbrStatus.BarColorTo := 00600060;
    end
  else
    begin
      prgbrStatus.BarColorFrom := $0031D329;
      prgbrStatus.BarColorTo := $00ADEFAD;
    end;

  prgbrStatus.Refresh;

  pnlStatus.SimpleCaption := NewCaption;
  pnlStatus.Refresh;

end;


procedure TfrmMHA3_AdministerSingleItems.OnChangeTrackBar(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  with Sender as TExTrackBar do
    begin
      Color := gvHighlightBackground;
      Font.Color := gvHighlightForeground;
      ProcessAnswer(IntToStr(Tag), IntToStr(Position));
    end;
end;

procedure TfrmMHA3_AdministerSingleItems.OnChangeEvent(sender: TObject);
// OnChangeEvent is used primarily to handle rules for skipping
// subsequent questions, validating responses. OnChange for
// Radio Group, List Box, Check List box, Combo box and Track bar
// are processed elsewhere.
//
// TEdit, TMemo, TMaskEdit & TJvDatePickerEdit process their answers in the
// OnExit Event.
begin
  if lvInitialize then
    Exit;

  if Sender is TORComboBox then
    with Sender as TORComboBox do
      begin
        if Pos(cComboBoxConsult, Name) > 0 then
          ProcessAnswer(IntToStr(Tag), Piece(Items[ItemIndex], U, 1))
        else
          ProcessAnswer(IntToStr(Tag), Text);
      end;

  if Sender is TRzSpinEdit then
    with Sender as TRzSpinEdit do
      begin
        if Text = '' then // Blank will cause error
          ProcessAnswer(IntToStr(Tag), '')
        else
          ProcessAnswer(IntToStr(Tag), FloatToStr(Value));
      end;

  if Sender is TJvValidateEdit then // handles blank entry okay
    with Sender as TJvValidateEdit do
      ProcessAnswer(IntToStr(Tag), FloatToStr(Value));

end;

function TfrmMHA3_AdministerSingleItems.IsQuestionRequired(QID: string): Boolean;
var
  i                           : Integer;
begin
  result := False;

  for i := 0 to lvInstrument.ListOfQuestions.Count - 1 do
    if QID = TQuestion(lvInstrument.ListOfQuestions[i]).QuestionID then
      if TQuestion(lvInstrument.ListOfQuestions[i]).IsRequired then
        begin
          result := True;
          break;
        end;
end;


procedure TfrmMHA3_AdministerSingleItems.OnExitEvent(sender: TObject);
// This event is used to catch questions that are skipped and
// to handle components that are not in OnChange Event (TEdit, TMaskEdit,
// TMemo, TJvDatePickerEdit).
var
  Skip                        : Boolean;
  i                           : Integer;
begin

  if lvInitialize then
    Exit;

  if lvIsLongInstrument then
    begin
      if Sender is TRzRadioGroup then
        with Sender as TRzRadioGroup do
          begin
            if ItemIndex = -1 then
              RecordLongInstrument(Tag, -1);
          end;
      exit;
    end;

  if Sender is TMemo then
    with Sender as TMemo do
      begin
        if Length(Text) = 0 then
          begin
            if IsQuestionRequired(IntToStr(Tag)) then
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION, '', NO {not multichoice}, -1)
            else
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_QUESTION, '', NO {not multichoice}, -1)
          end
        else
          ProcessAnswer(IntToStr(Tag), Text);
      end;

  if Sender is TEdit then
    with Sender as TEdit do
      if Text = '' then
        begin
          if IsQuestionRequired(IntToStr(Tag)) then
            RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION, '', NO {not multichoice}, -1)
          else
            RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_QUESTION, '', NO {not multichoice}, -1)
        end
      else
        ProcessAnswer(IntToStr(Tag), Text);

  if Sender is TMaskEdit then
    with Sender as TMaskEdit do
      begin
        if (Text = '') or (Text = MASK_BLANK) then
          begin
            if IsQuestionRequired(IntToStr(Tag)) then
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION, '', NO {not multichoice}, -1)
            else
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_QUESTION, '', NO {not multichoice}, -1)
          end
        else
          ProcessAnswer(IntToStr(Tag), Text);
      end;

  if Sender is TJvDatePickerEdit then
    with Sender as TJvDatePickerEdit do
      begin
        if DateTimeToStr(Date) = BLANK_DATE then
          begin
            if IsQuestionRequired(IntToStr(Tag)) then
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION, BLANK_DATE, NO {not multichoice}, -1)
            else
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_QUESTION, BLANK_DATE, NO {not multichoice}, -1);
          end
        else // Non blank entry
          begin
            if Date > Now then
              begin
                ShowMessage('Sorry, future dates are not allowed in this field. ' + #13 +
                  'Today''s will be substituded.');
              //  MsgDlgFrm('Sorry, future dates are not allowed in this field. ' + #13 +
              //    'Today''s will be substituded.', MHA_Caption, mtInformation, [mbOk], 0);
                Date := Today;
              end;
            //RecordAnswer(IntToStr(Tag), '', DateToStr(Date), NO {not multichoice}, -1);
            ProcessAnswer(IntToStr(Tag), DateToStr(Date));
          end;
      end;

  if Sender is TComboBox then
    with Sender as TComboBox do
      begin
        if Text = '' then
          begin
            if IsQuestionRequired(IntToStr(Tag)) then
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION, '', YES {multichoice}, -1)
            else
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_QUESTION, '', YES {multichoice}, -1)
          end
        else
          // The combo box could have a default staff name that was not changed;
          // thus the OnExit event must be used to record it. The OnChange event is
          // needed in case there is a rule that is triggered by a selection.
          RecordAnswer(IntToStr(Tag), IntToStr(ItemIndex), Text, YES, ItemIndex);
      end;

  if Sender is TORComboBox then
    with Sender as TORComboBox do
      begin
        if Text = '' then
          begin
            TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ConsultIEN := '';
            if IsQuestionRequired(IntToStr(Tag)) then
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION, '', NO {multichoice}, -1)
            else
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_QUESTION, '', NO {multichoice}, -1)
          end
        else
        // The combo box could have a default staff name that was not changed;
        // thus the OnExit event must be used to record it. The OnChange event is
        // needed in case there is a rule that is triggered by a selection.
          begin
            if Pos(cComboBoxConsult, Name) > 0 then
              begin
                RecordAnswer(IntToStr(Tag), Piece(Items[ItemIndex], U, 1),
                  Piece(Items[ItemIndex], U, 7), NO {MultiChoice}, ItemIndex);
                TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ConsultIEN :=
                  Piece(Items[ItemIndex], U, 1);
              end
            else
              RecordAnswer(IntToStr(Tag), Text, Text, NO {MultiChoice}, ItemIndex);
          end;
      end;

  if Sender is TRzSpinEdit then
    with Sender as TRzSpinEdit do
      begin
        if Text = '' then
          begin
            if IsQuestionRequired(IntToStr(Tag)) then
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION, '', NO {Multichoice}, -1)
            else
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_QUESTION, '', NO {not multichoice}, -1);
          end
        else
          // This handles the case when a default value is present, user accepts
          // it, thus no OnChange event triggered).
          ProcessAnswer(IntToStr(Tag), FloatToStr(Value));
      end;

  if Sender is TJvValidateEdit then
    with Sender as TJvValidateEdit do
      begin
        if Text = '' then
          begin
            if IsQuestionRequired(IntToStr(Tag)) then
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION, '', NO {not multichoice}, -1)
            else
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_QUESTION, '', NO {not multichoice}, -1)
          end
        else
          // This handles the case when a default value is present, user accepts
          // it, thus no OnChange event triggered).
          ProcessAnswer(IntToStr(Tag), FloatToStr(Value));
      end;

  if Sender is TRzCheckList then
    with Sender as TRzCheckList do
      begin
        Skip := True;

        for i := 0 to Items.Count - 1 do
          if ItemChecked[i] then
            Skip := False;

        if Skip then
          begin
            if IsQuestionRequired(IntToStr(Tag)) then
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION, '', NO {not multichoice}, -1)
            else
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_QUESTION, '', NO {Multichoice}, -1);
            // The selected item was handled already in the OnChangeEvent
          end;
      end;

  if Sender is TRzListBox then
    with Sender as TRzListBox do
      begin
        if ItemIndex = -1 then
          begin
            if IsQuestionRequired(IntToStr(Tag)) then
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION, '', YES {multichoice}, -1)
            else
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_QUESTION, '', YES {Multichoice}, -1);
         // The selected item was handled already in the OnChangeEvent
          end;
      end;

  if Sender is TRzRadioGroup then
    with Sender as TRzRadioGroup do
      begin
        if ItemIndex = -1 then
          begin
            if IsQuestionRequired(IntToStr(Tag)) then
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION, '', YES {multichoice}, -1)
            else
              RecordAnswer(IntToStr(Tag), CHOICE_ID_OF_SKIPPED_QUESTION, '', YES {Multichoice}, -1);
          end;
      end;

// This is actually called when the user enters this component. This is okay
// for our purposes since the OnChange will get the real response, unless the
// user selects the position that is already dispayed (no change is triggered)
// and this will have recorded that answer. Recording twice? yep.

  if Sender is TExTrackBar then
    with Sender as TExTrackBar do
      RecordAnswer(IntToStr(Tag), IntToStr(Position), IntToStr(Position),
        NO {MultiChoice}, Position);
end;


procedure TfrmMHA3_AdministerSingleItems.OnChangeCheckList(Sender: TObject;
  Index: Integer; NewState: TCheckBoxState);
var
  TempItems,
    TempStr                   : string;
  i                           : Integer;
begin
  if lvInitialize then
    Exit;

  with sender as TRzCheckList do
    begin
      TempStr := '';

      for i := 0 to Items.Count - 1 do
        if ItemChecked[i] then
          begin
            TempStr := TempStr + '1';
            TempItems := TempItems + ' ' + Items[i];
          end
        else
          TempStr := TempStr + '0';

      if length(TempItems) > 2 then
        System.Delete(TempItems, Length(TempItems) - 1, 1);

      ProcessAnswer(IntToStr(Tag), TempStr);
      // Normally the Answer.Value is sufficient for the saving of a
      // component's answer. However, the text of items have to be saved too,
      // and check list boxes are special.
      RecordAnswer(IntToStr(Tag), TempStr, TempItems, NO, -1);

    end;
end;

function TfrmMHA3_AdministerSingleItems.GetItemIndex(aQuestionID, ChoiceID: string): integer;
var
  j, i                        : Integer;
begin
  result := -1;
  for i := 0 to lvInstrument.ListOfQuestions.Count - 1 do
    if aQuestionID = TQuestion(lvInstrument.ListOfQuestions[i]).QuestionID then
      for j := 0 to TQuestion(lvInstrument.ListOfQuestions[i]).MultiChoice.choices.Count - 1 do
        if Piece(TQuestion(lvInstrument.ListOfQuestions[i]).MultiChoice.choices[j], U, 2) =
          ChoiceID then
          result := j;
end;

function TfrmMHA3_AdministerSingleItems.GetChoiceOnly(StringIn: string): string;
var
  i                           : Integer;
begin
  Result := StringIn;
  i := Pos('.', StringIn);
  if i = 0 then
    i := Pos(')', StringIn);
  if i = 0 then
    i := Pos(']', StringIn);
  if (i > 0) and ((i + 1) < Length(StringIn)) then
    result := Trim(Copy(StringIn, i + 1, Length(StringIn)));
end;

  (*
procedure TfrmMHA_AdministerAllItems.OnExitTrackBar(Sender: TObject);
// This is actually called when the user enters this component. This is okay
// for our purposes since the OnChange will get the real response, unless the
// user selects the position that is already dispayed (no change is triggered)
// and this will have recorded that answer. Recording twice? yep.
begin
  if lvInitialize then
    Exit;

  with Sender as TExTrackBar do
    RecordAnswer(IntToStr(Tag), IntToStr(Position), IntToStr(Position),
      NO {MultiChoice});
end;*)

procedure TfrmMHA3_AdministerSingleItems.BuildForm(WhichOne: Integer);
var
  i, j,
    TempHeight                : Integer;
  TempStrList                 : TStringList;
  TempFont                    : TFont;
  TempIntro,
    TempStr                   : string;

  procedure Build_Items_Here(Designator: TChoiceIdentifier;
    var StringListIn: TStringList);
  var
    i                         : Integer;
    TempSL                    : TStringList;
  begin
    if StringListIn.Count = 0 then
      Exit;

    TempSL := TStringList.Create;

    for i := 0 to StringListIn.Count - 1 do
      begin
        case Designator of
          ciNone: TempSL.Add(Piece(StringListIn[i], U, 3));
          ciZeroBased: TempSL.Add(inttostr(i) + '. ' + Piece(StringListIn[i], U, 3));
          ciOneBased: TempSL.Add(inttostr(i + 1) + '. ' + Piece(StringListIn[i], U, 3));
        end; //case Question.ChoiceIdentifier
      end;

    StringListIn.Clear;
    StringListIn.AddStrings(TempSL);
    TempSL.Free;
  end;

begin

  lvInstrument.InstrumentName := TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName;

  // Look for the instrument on the local directory
  if not ResourceFile_OPS.ReadResourceFile(lvInstrument) then
    begin
      // Get instrument from Vista: slower
      Instrument_OPS.Load(lvInstrument);
      // Make a copy on the local drive for next time
      ResourceFile_OPS.WriteResourceFile(lvInstrument);
    end;

  TempStrList := TStringList.Create;
  TempFont := TFont.Create;

  if gvUseDesignFont then
    TempFont.Assign(Font)
  else
    TempFont.Assign(gvFont);


  //WIN7SCROLLBUG MAIN
  //This is what breaks the code.  Was set to 0, changed to 3 to get some space between questions
  gvQuestionSpace := 3 ; //3 * TextHeightByFont(TempFont.Handle, 'Yy') div 2;
  // END WIN7SCROLLBUG

  gvQuestionTop := gvQuestionSpace;
  gvScrollBarWidthWS := TextWidthbyFont(TempFont.handle, 'XXXX');

  // Build static text from Vista lookup: handled by DLL
  LoadStaticText(lvInstrument.InstrumentName);
  LoadVistaText(lvInstrument.InstrumentName);

  for i := 0 to lvInstrument.ListOfQuestions.Count - 1 do
    begin
      with TQuestion(lvInstrument.ListOfQuestions[i]) do
        begin
          TempHeight := 0;
          Inc(gvQuestionOrder);

          if i = 0 then
            lvFirstQuestionID := lvFirstQuestionID + QuestionID + U;

          if i = lvInstrument.ListOfQuestions.Count - 1 then
            lvFinalQuestionID := lvFinalQuestionID + QuestionID + U;

          // Check for a section label
          for j := 0 to lvInstrument.ListOfSections.Count - 1 do
            if (TSectionLabel(lvInstrument.ListOfSections.Items[j]).QuestionID = QuestionID) then
              begin
                Build_Section(TLabel.Create(self),
                  TSectionLabel(lvInstrument.ListOfSections.Items[j]).Display, 0,
                  TSectionLabel(lvInstrument.ListOfSections.Items[j]).SectionCaption,
                  lvScrollBox);
                //if TSectionLabel(lvInstrument.ListOfSections[j]).TabCaption <> '' then
                //  tbctrlRight.Tabs.Add(TSectionLabel(lvInstrument.ListOfSections[j]).TabCaption);
              end;

          // Build the introduction if there is one.
          if TIntroduction(lvInstrument.ListOfIntroductions[i]).iText <> '' then
            if TIntroduction(lvInstrument.ListOfIntroductions[i]).IsNew then
              Build_Instructions(TLabel.Create(self),
                TIntroduction(lvInstrument.ListOfIntroductions[i]).Display, 0,
                TIntroduction(lvInstrument.ListOfIntroductions[i]).iText, lvScrollBox);

          if (lvStaticText <> '') then
            begin
              TempStr := GetStaticText(i);
              if TempStr <> '' then
                begin
                  Inc(gvQuestionOrder);
                  if lvInstrument.ListOfIntroductions.Count > 0 then
                    Build_Instructions(TLabel.Create(self),
                      TIntroduction(lvInstrument.ListOfIntroductions[0]).Display,
                      gvQuestionOrder,
                      TempStr, lvScrollBox)
                  else
                    Build_Instructions(TLabel.Create(self),
                      Display,
                      gvQuestionOrder, TempStr, lvScrollBox);
                end;
            end;

          // Build the Question text (except for radio groups which are self
          // contained.
          if ((ResponseType = rtMultiChoice) and
            (AnswerDisplay.DisplayComponent = acRadioGroup)) then
            begin
              if gvUseScreenReader then
                begin
                  AnswerDisplay.DisplaySameLine := False;
                  if (TempIntro + qCaption <> '') then
                    begin
                      TempHeight := Build_508IntroCaption(TGroupBox.Create(self),
                        Display, TempIntro + ' ' + qCaption, qTag, ResponseType, lvScrollBox);
                      Inc(gvQuestionOrder);
                    end;
                end;
            end
          else
            begin
              if (qCaption <> '') then
                begin
                  if gvUseScreenReader then
                    begin
                      AnswerDisplay.DisplaySameLine := False;
                      if (TempIntro + Caption <> '') then
                        begin
                          TempHeight := Build_508IntroCaption(TGroupBox.Create(self),
                            Display, TempIntro + ' ' + qCaption, qTag, ResponseType, lvScrollBox);
                          Inc(gvQuestionOrder);
                        end;
                    end
                  else
                    TempHeight := Build_Label(TLabel.Create(self),
                      Display, AnswerDisplay, qCaption,
                      qHint, qTag, ResponseType, lvScrollBox);
                end;
            end;

          // Is the answer component on the same line as the question text?
          if not AnswerDisplay.DisplaySameLine then // Component on next line
            gvQuestionTop := gvQuestionTop + TempHeight;

          if ResponseType = rtCheckList then
            begin
              TempStrList.Clear;
              TempStrList.AddStrings(MultiChoice.Choices);
              Build_Items_Here(ChoiceIdentifier, TempStrList);

              Build_CheckListBox(TRzCheckList.Create(self),
                AnswerDisplay, qHint, TempHeight, qTag, lvScrollBox, TempStrList);
            end;

          if (ResponseType = rtMultiChoice) and
            (AnswerDisplay.DisplayComponent = acComboBox) then
            begin
              TempStrList.Clear;
              TempStrList.AddStrings(MultiChoice.Choices);
              Build_Items_Here(ChoiceIdentifier, TempStrList);

              Build_ComboBox(TComboBox.Create(self),
                AnswerDisplay, qHint, TempHeight, qTag, lvScrollBox, TempStrList);
            end;

          if ResponseType = rtCurrency then
            Build_Currency(TJvValidateEdit.Create(self),
              AnswerDisplay, qHint, TempHeight, qTag, qMin, qMax, lvScrollBox);

          if ResponseType = rtDate then
            Build_Date(TJvDatePickerEdit.Create(self),
              AnswerDisplay, qHint, TempHeight, qTag, lvScrollBox);

          if ResponseType = rtString then
            Build_Edit(TEdit.Create(self),
              AnswerDisplay, qHint, TempHeight, qTag, lvScrollBox);

          if (ResponseType = rtMultiChoice) and
            (AnswerDisplay.DisplayComponent = acListBox) then
            begin
              TempStrList.Clear;
              TempStrList.AddStrings(MultiChoice.Choices);
              Build_Items_Here(ChoiceIdentifier, TempStrList);

              Build_ListBox(TRzListBox.Create(self),
                AnswerDisplay, qHint, TempHeight, qTag, lvScrollBox, TempStrList);
            end;

          if ResponseType = rtMaskEdit then
            Build_MaskEdit(TMaskEdit.Create(self),
              AnswerDisplay, qHint, TempHeight, qTag, lvScrollBox);

          if ResponseType = rtMemo then
            Build_Memo(TMemo.Create(self),
              AnswerDisplay, qHint, TempHeight, qTag, lvScrollBox);

          if (ResponseType = rtMultiChoice) and
            (AnswerDisplay.DisplayComponent = acRadioGroup) then
            begin
              TempStrList.Clear;
              TempStrList.AddStrings(MultiChoice.Choices);
              Build_Items_Here(ChoiceIdentifier, TempStrList);

              if gvUseScreenReader then
                Build_ListBox(TRzListBox.Create(self),
                  AnswerDisplay, qHint, TempHeight, qTag, lvScrollBox, TempStrList)
              else
                Build_RadioGroup(TRzRadioGroup.Create(self),
                  qCaption, qHint, qTag,
                  lvInstrument.Islegacy,
                  Display, AnswerDisplay,
                  //lvInstrument.IsLegacy,
                  lvScrollBox, TempStrList);
            end;

          if ResponseType = rtInteger then
            Build_SpinEdit(TRzSpinEdit.Create(self),
              AnswerDisplay, qHint, TempHeight, qTag, qMin,
              qMax, rtInteger, lvScrollBox);

          if ResponseType = rtTrackBar then
            begin
              Build_TrackBar(TExTrackBar.Create(self),
                AnswerDisplay, qHint, TempHeight, qTag, qMin, qMax, lvScrollBox);
              if MultiChoice.Choices.Count > 0 then
                begin
                  TempStrList.Clear;
                  TempStrList.AddStrings(MultiChoice.Choices);
                  Build_Items_Here(ChoiceIdentifier, TempStrList);
                  Build_TrackLabel2(TLabel.Create(self),
                    AnswerDisplay, qTag, lvScrollBox, TempStrList);
                end;
            end;

          if (ResponseType = rtVista) and
            (AnswerDisplay.DisplayComponent = acStaff) then
            Build_StaffComboBox(TORComboBox.Create(self),
              AnswerDisplay, qHint, TempHeight, qTag, lvScrollBox);

          if (ResponseType = rtVista) and
            (AnswerDisplay.DisplayComponent = acConsult) then
            Build_ConsultComboBox(TORComboBox.Create(self),
              AnswerDisplay, qHint, TempHeight, qTag, lvScrollBox);

        end; // with TQuestion(lvInstrument.ListOfQuestions[i])
    end; // for i

  // Add a little space (bottom margin) to the form
  gvQuestionTop := gvQuestionTop + (2 * gvQuestionSpace);
  lvScrollBox.VertScrollBar.Range := gvQuestionTop;
  TempStrList.Free;
  TempFont.Free;
end;

procedure TfrmMHA3_AdministerSingleItems.AgmentAnswerList(WhichOne, aTag, aItemIndex: Integer;
  aText: string);
var
  i                           : Integer;
begin
  for i := 0 to TAdministration(Administration_OPS.Administrations[WhichOne]).ListOfAdminAnswers.Count - 1 do
    begin
      with TAnswer(TAdministration(Administration_OPS.Administrations[WhichOne]).ListOfAdminAnswers[i]) do
        begin
          if IntToStr(aTag) = QuestionID then
            begin
              AnswerItemIndex := aItemIndex;
              AnswerText := aText;
            end;
        end;
    end;
end;

procedure TfrmMHA3_AdministerSingleItems.BuildEvents(WhichOne: Integer);
// Setup the OnEnter, OnExit, etc. events for each component.
// Populate component with previous responses, if applicable.
var
  i, j, k                     : integer;
  TempStr                     : string;

begin
  Rule_OPS.SkipQuestions := '';

  if lvInstrument.ListOfQuestions.Count = 0 then
    exit;

  // This will populate ListOfAnswers with previous data, or will initialize
  // the answers, NOT_ASKED.
  Administration_OPS.LoadAnswers(lvInstrument,
    TAdministration(Administration_OPS.Administrations[WhichOne]));

  lvInitialize := True;

  for i := lcStartComponent to ComponentCount - 1 do
    begin
      if Components[i] is TEdit then
        with TEdit(Components[i]) do
          begin
            if Parent = lvScrollBox then
              begin
                OnExit := OnExitEvent;

                if TAdministration(Administration_OPS.Administrations[WhichOne]).ID <> NO_ID then
                  begin
                    TempStr := '';
                    TempStr := GetFormerAnswer(IntToStr(Tag));

                    // Check for carriage return markers
                    if POS(CARRIAGE_RETURN_MARKER, TempStr) > 0 then
                      begin
                        for k := 1 to NumberOfPieces(TempStr, CARRIAGE_RETURN_MARKER) do
                          Text := Text + (Piece(TempStr, CARRIAGE_RETURN_MARKER, k)) + CRLF;
                      end
                    else
                      Text := Tempstr;
                  end
                else
                  begin
                    TempStr := '';
                    // This gets default entry or the results from a dll call
                    TempStr := GetDefaultEntry(Tag);
                    // Check for carriage returns
                    if POS(RECORD_END, TempStr) > 0 then
                      begin
                        for k := 1 to NumberOfPieces(TempStr, RECORD_END) do
                          Text := Text + (Piece(TempStr, RECORD_END, k)) + CRLF;
                      end
                    else
                      Text := Tempstr;
                  end;

                if Text = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                  begin
                    Text := '';
                    Rule_OPS.SkipQuestions := Rule_OPS.SkipQuestions + U + IntToStr(Tag) + U;
                    Enabled := false;
                  end;
              end;
          end;
      if Components[i] is TMaskEdit then
        with TMaskEdit(Components[i]) do
          begin
            if Parent = lvScrollBox then
              begin
                OnExit := OnExitEvent;
                if TAdministration(Administration_OPS.Administrations[WhichOne]).ID <> NO_ID then
                  Text := GetFormerAnswer(IntToStr(Tag))
                else
                  Text := GetDefaultEntry(Tag);
                if Text = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                  begin
                    Text := '';
                    Rule_OPS.SkipQuestions := Rule_OPS.SkipQuestions + U + IntToStr(Tag) + U;
                    Enabled := false;
                  end;
              end;
          end;

      if Components[i] is TJvValidateEdit then
        with TJvValidateEdit(Components[i]) do
          begin
            if Parent = lvScrollBox then
              begin
                OnExit := OnExitEvent;
                OnChange := OnChangeEvent;

                if TAdministration(Administration_OPS.Administrations[WhichOne]).ID <> NO_ID then
                  Text := GetFormerAnswer(IntToStr(Tag))
                else
                  Text := GetDefaultEntry(Tag);

                if Text = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                  begin
                    Text := '';
                    Rule_OPS.SkipQuestions := Rule_OPS.SkipQuestions + U + IntToStr(Tag) + U;
                    Enabled := false;
                  end;
              end;
          end;

      if Components[i] is TJvDatePickerEdit then
        with TJvDatePickerEdit(Components[i]) do
          begin
            if Parent = lvScrollBox then
              begin
                OnExit := OnExitEvent;

                if TAdministration(Administration_OPS.Administrations[WhichOne]).ID <> NO_ID then
                  TempStr := GetFormerAnswer(IntToStr(Tag));

                if TempStr = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                  begin
                    Date := StrToDateTime(BLANK_DATE);
                    Rule_OPS.SkipQuestions := Rule_OPS.SkipQuestions + U + IntToStr(Tag) + U;
                    Enabled := false;
                  end;
              end;
          end;

      if Components[i] is TRzSpinEdit then
        with TRzSpinEdit(Components[i]) do
          begin
            if Parent = lvScrollBox then
              begin
                OnExit := OnExitEvent;
                OnChange := OnChangeEvent;
                if TAdministration(Administration_OPS.Administrations[WhichOne]).ID <> NO_ID then
                  Text := GetFormerAnswer(IntToStr(Tag))
                else
                  Text := GetDefaultEntry(Tag);
                if Text = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                  begin
                    Text := '';
                    Rule_OPS.SkipQuestions := Rule_OPS.SkipQuestions + U + IntToStr(Tag) + U;
                    Enabled := false;
                  end;
              end;
          end;
      if Components[i] is TRzRadioGroup then
        with TRzRadioGroup(Components[i]) do
          begin
            if Parent = lvScrollBox then
              begin
                OnExit := OnExitEvent;
                OnClick := OnClickRadioGroup;
                OnDblClick := DoubleClickMessage;

                if TAdministration(Administration_OPS.Administrations[WhichOne]).ID <> NO_ID then
                  begin
                    TempStr := GetFormerAnswer(IntToStr(Tag)); // ChoiceID returned

                    if TempStr = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                      begin
                        ItemIndex := -1;
                        TempStr := '';
                        Rule_OPS.SkipQuestions := Rule_OPS.SkipQuestions + U + IntToStr(Tag) + U;
                        Enabled := false;
                      end;
                    if TempStr <> '' then
                      begin
                        ItemIndex := GetItemIndex(IntToStr(Tag), TempStr);
                        AgmentAnswerList(WhichOne, Tag, ItemIndex, Items[ItemIndex]);
                      end
                  end
                else
                  ItemIndex := StrToIntDef(GetDefaultEntry(Tag), -1);
              end;
          end;
      if Components[i] is TRzListBox then
        with TRzListBox(Components[i]) do
          begin
            if Parent = lvScrollBox then
              begin
                OnExit := OnExitEvent;
                OnClick := OnClickListBox;
                OnDblClick := DoubleClickMessage;
                // No OnChange Event
                if TAdministration(Administration_OPS.Administrations[WhichOne]).ID <> NO_ID then
                  begin
                    TempStr := GetFormerAnswer(IntToStr(Tag));
                    if TempStr = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                      begin
                        TempStr := '';
                        Rule_OPS.SkipQuestions := Rule_OPS.SkipQuestions + U + IntToStr(Tag) + U;
                        Enabled := false;
                      end;
                    if TempStr <> '' then
                      begin
                        ItemIndex := GetItemIndex(IntToStr(Tag), TempStr);
                        AgmentAnswerList(WhichOne, Tag, ItemIndex, Items[ItemIndex]);
                      end;
                  end;
              end;
          end;
      if Components[i] is TRzCheckList then
        with TRzCheckList(Components[i]) do
          begin
            if Parent = lvScrollBox then
              begin
                OnChange := OnChangeCheckList;
                OnExit := OnExitEvent;
                if TAdministration(Administration_OPS.Administrations[WhichOne]).ID <> NO_ID then
                  begin
                    TempStr := GetFormerAnswer(IntToStr(Tag));
                    if TempStr = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                      begin
                        TempStr := '';
                        Rule_OPS.SkipQuestions := Rule_OPS.SkipQuestions + U + IntToStr(Tag) + U;
                        Enabled := false;
                      end;

                    for j := 0 to Items.Count - 1 do
                      if Pos(Items[j], TempStr) > 0 then
                        ItemChecked[j] := True
                      else
                        ItemChecked[j] := False;
                  end;
              end;
          end;
      if Components[i] is TComboBox then
        with TComboBox(Components[i]) do
          if Parent = lvScrollBox then
            begin
              OnExit := OnExitEvent;
              OnClick := OnClickComboBox;
              OnDblClick := DoubleClickMessage;
              if TAdministration(Administration_OPS.Administrations[WhichOne]).ID <> NO_ID then
                begin
                  TempStr := GetFormerAnswer(IntToStr(Tag));
                  if TempStr = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                    begin
                      TempStr := '';
                      Rule_OPS.SkipQuestions := Rule_OPS.SkipQuestions + U + IntToStr(Tag) + U;
                      Enabled := false;
                    end;
                  if TempStr <> '' then
                    ItemIndex := StrToIntDef(TempStr, -1);

                end
              else
                Text := GetDefaultEntry(Tag);
            end;
      if Components[i] is TORComboBox then
      // Staff & Consult Vista combo-box
        with TORComboBox(Components[i]) do
          begin
            if Parent = lvScrollBox then
              begin
                OnExit := OnExitEvent;
                OnChange := OnChangeEvent;

                if DataBaseMode = dbVista then
                  begin
                    if TAdministration(Administration_OPS.Administrations[WhichOne]).ID <> NO_ID then
                      begin
                        TempStr := GetFormerAnswer(IntToStr(Tag));
                        if Text = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                          begin
                            TempStr := '';
                            Rule_OPS.SkipQuestions := Rule_OPS.SkipQuestions + U + IntToStr(Tag) + U;
                            Enabled := false;
                          end;
                      end;

                    // Consult Combo-box
                    if Pos(cComboBoxConsult, Name) > 0 then
                      begin
                    // The standard ORComboBox routines will always sort on ascending
                    // order. Not used here.
                    //OnNeedData := cmbConsultNeedData;
                    //InitLongList('');

                        Items.AddStrings(Consult_OPS.LoadConsultSubSet(Patient.DFN,
                          DataModule1.RPCBroker1));
                        DropDownCount := Items.Count - 1;
                      end
                    else
                    // Staff Combo-box
                      begin
                        OnNeedData := cmbStaffNeedData;
                        InitLongList('A');

                        if TempStr <> '' then
                          Text := TempStr
                        else
                          Text := ThisUser.Name;
                      end;
                  end;
              end;
          end;
      if Components[i] is TExTrackBar then
        with TExTrackBar(Components[i]) do
          begin
            if Parent = lvScrollBox then
              begin
                OnExit := OnExitEvent;
                OnChange := OnChangeTrackBar;
                if TAdministration(Administration_OPS.Administrations[WhichOne]).ID <> NO_ID then
                  begin
                    TempStr := GetFormerAnswer(IntToStr(Tag));
                    if TempStr = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                      begin
                        TempStr := '';
                        Rule_OPS.SkipQuestions := Rule_OPS.SkipQuestions + U + IntToStr(Tag) + U;
                        Enabled := false;
                      end;
                    if TempStr <> '' then
                      Position := StrToIntDef(TempStr, 1);
                  end
                else
                  Position := StrToIntDef(GetDefaultEntry(Tag), -1);
              end;
          end;

      if Components[i] is TMemo then
        with TMemo(Components[i]) do
          begin
            if Parent = lvScrollBox then
              begin
                OnExit := OnExitEvent;

                if TAdministration(Administration_OPS.Administrations[WhichOne]).ID <> NO_ID then
                  begin
                    TempStr := GetFormerAnswer(IntToStr(Tag));

                    // if Text contains a CARRIAGE_RETURN_MARKER then build the lines
                    if POS(CARRIAGE_RETURN_MARKER, TempStr) > 0 then
                      begin
                        Lines.Clear;
                        for k := 1 to NumberOfPieces(TempStr, CARRIAGE_RETURN_MARKER) do
                          Lines.Append(Piece(TempStr, CARRIAGE_RETURN_MARKER, k));
                      end
                    else
                      Text := TempStr;
                  end
                else
                  begin
                    TempStr := '';
                    // This gets default entry or the results from a dll call
                    TempStr := GetDefaultEntry(Tag);
                    // Check for carriage returns
                    if POS(RECORD_END, TempStr) > 0 then
                      begin
                        Lines.Clear;
                        for k := 1 to NumberOfPieces(TempStr, RECORD_END) do
                          Lines.Append(Piece(TempStr, RECORD_END, k));
                      end
                    else
                      Text := Tempstr;
                  end;

                if Text = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                  begin
                    Text := '';
                    Rule_OPS.SkipQuestions := Rule_OPS.SkipQuestions + U + IntToStr(Tag) + U;
                    Enabled := false;
                  end;
              end;
          end;
    end; // for i
end;


function TfrmMHA3_AdministerSingleItems.IntroOrSectionOrLabel(aName: string): Integer;
// Intro's, labels, sections have the same number as the control component that
// follows it.
var
  i,
    TempInt                   : Integer;
  TempStr                     : string;
begin
  result := 0;

  TempStr := aName;
  TempStr := Copy(TempStr, 4, Length(TempStr));
  if (TempStr = '') or (TempStr = ' ') then
    Exit;
  TempInt := StrToIntDef(TempStr, -1);

  if (TempInt = -1) then
    Exit;

  Result := lcSpacer;
  for i := lcStartComponent to ComponentCount - 1 do
    begin
      if Components[i] is TLabel then
        with TLabel(Components[i]) do
          begin
            if Parent = lvScrollBox then
              begin
                if (Name = cLabel + IntToStr(TempInt)) or
                  (Name = cSection + IntToStr(TempInt)) or
                  (Name = cTrackBarLabel1 + IntToStr(TempInt)) or
                  (Name = cTrackBarLabel2 + IntToStr(TempInt)) or
                  (Name = cIntroduction + IntToStr(TempInt)) then
                  begin
                    Visible := True;
                    Top := Result;
                    Result := Top + Height;
                  end
                else
                  Visible := False;
              end;
          end;
    end; //for i
end;

procedure TfrmMHA3_AdministerSingleItems.UpdateButtons(Count, Current: Integer);
begin
  btnPrior.Enabled := True;
  btnNext.Enabled := True;

  if Current = 0 then
    btnPrior.Enabled := False;

  if Current = Count then
    begin
      btnNext.Enabled := False;
      // Several instruments presented
      if Administration_OPS.Administrations.Count > 1 then
        begin
          if TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).NumberSkipped > 0 then
            MsgDlgFrm(
              'That was the last question for this instrument.' + #13 +
              'There are one or more questions that have been skipped.' + #13 +
              'You can review your answers by clicking on the "Review Answers" button.' + #13 +
              'To start the next instrument, click on its tab.' + #13 +
              'To save your answers for all the instruments, click on the "Finish" button.',
              MHA_CAPTION, mtInformation, [mbOk], 0)
          else
            MsgDlgFrm(
              'There are no more questions for this instrument.' + #13 +
              'You can review your answers by clicking on the "Review Answers" button.' + #13 +
              'To start the next instrument, click on its tab.' + #13 +
              'To save your answers for all the instruments, click on the "Finish" button.',
              MHA_CAPTION, mtInformation, [mbOk], 0);
        end
      else
      // One instrument presented
        begin
          if TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).NumberSkipped > 0 then
            MsgDlgFrm(
              'That was the last question for this instrument.' + #13 +
              'There are one or more questions that have been skipped.' + #13 +
              'You can review your answers by clicking on the "Review Answers" button.' + #13 +
              'To save your answers, click on the "Finish" button.',
              MHA_CAPTION, mtInformation, [mbOk], 0)
          else
            MsgDlgFrm(
              'There are no more questions.' + #13 +
              'You can review your answers by clicking on the "Review Answers" button.' + #13 +
              'To save your answers, click on the "Finish" button.',
              MHA_CAPTION, mtInformation, [mbOk], 0);
        end;
    end;
end;


procedure TfrmMHA3_AdministerSingleItems.GoToNextQuestion(aName: string);
// Tabbing to next question requires that the next question is visible,
//   otherwise tabbing will move control to next visible component, not in the
//   Scroll box.
// ChildControl is populated only with control components, no
//   labels, sections, or intros.
// If the next question is disabled (skip logic), go to the next enabled
//   Question.

var
  i, j,
    Current,
    LabelHeight               : Integer;
  ChildControl                : TControl;
begin

  for i := 0 to lvScrollBox.ControlCount - 1 do
    begin
      ChildControl := lvScrollBox.Controls[i];

      // This is current active question
      if aName = ChildControl.Name then
        begin
          Current := i;
          UpdateButtons(lvScrollBox.ControlCount - 1, i);

          // Last question
          if i = lvScrollBox.ControlCount - 1 then
            begin
              UpdateTallyNotAskedEtc;
              lvComponentName := ChildControl.Name;
              //ChildControl.Visible := False;
              Exit;
            end;

          // Hide the current question
          ChildControl.Visible := False;

          ChildControl := lvScrollBox.Controls[i + 1];
          // If next control is disabled (due to skip logic), continue to next
          // enabled control.
          if not ChildControl.Enabled then
            begin
              j := 2;
              // patch 123 - added = to <=
              while (not ChildControl.Enabled) and (i + j  <= lvScrollBox.ControlCount - 1) do
                begin
                  ChildControl := lvScrollBox.Controls[i + j];
                  Inc(j);
                end;

              // All items are skipped, even the last one, so return to the
              // item that started this loop.
              //patch 123
              //if (i + j) = lvScrollBox.ControlCount then
              if ( ((i+j) = lvScrollBox.ControlCount) and (not ChildControl.Enabled) ) then
                begin
                  ChildControl := lvScrollBox.Controls[Current];
                  ChildControl.Visible := True;
                  UpdateTallyNotAskedEtc;
                  //patch 123, need to display message and put focus to finish button
                  UpdateButtons(i, i);
                  btnFinished.SetFocus;
                  Exit;
                end;
            end;

          ChildControl.Visible := True;
          LabelHeight := IntroOrSectionOrLabel(ChildControl.Name);
          ChildControl.Top := LabelHeight + lcSpacer;
          lvComponentName := ChildControl.Name;

          UpdateTallyNotAskedEtc;
          ActiveControl := lvScrollBox;
          Perform(WM_NEXTDLGCTL, 0, 0);
          Exit;
        end
      else
        ChildControl.Visible := False;
    end;
end;

procedure TfrmMHA3_AdministerSingleItems.GoToPriorQuestion(aName: string);
// See comments for GoToNextQuestion: same issues apply here.
var
  i, j,
    Current,
    LabelHeight               : Integer;
  ChildControl                : TControl;
begin

  for i := lvScrollBox.ControlCount - 1 downto 0 do
    begin
      ChildControl := lvScrollBox.Controls[i];

      if aName = ChildControl.Name then
        begin
          Current := i;

          if i > 0 then
            begin
              UpdateButtons(lvScrollBox.ControlCount - 1, i - 1);
              ChildControl.Visible := False;

              ChildControl := lvScrollBox.Controls[i - 1];
              // If next control is disabled (due to skip logic), continue to next
              // enabled control.
              if not ChildControl.Enabled then
                begin
                  j := 2;
                  while (not ChildControl.Enabled) and (i - j > 0) do
                    begin
                      ChildControl := lvScrollBox.Controls[i - j];
                      Inc(j);
                    end;
                  // All items are skipped, even to the first one, so return to the
                  // item that started this loop.
                  if (i + j) = 0 then
                    begin
                      ChildControl := lvScrollBox.Controls[Current];
                      ChildControl.Visible := True;
                      Exit;
                    end;
                end;

              ChildControl.Visible := True;

              LabelHeight := IntroOrSectionOrLabel(ChildControl.Name);
              ChildControl.Top := LabelHeight + lcSpacer;
              lvComponentName := ChildControl.Name;
              UpdateTallyNotAskedEtc;
              ActiveControl := lvScrollBox;
              Perform(WM_NEXTDLGCTL, 0, 0);
              Exit;
            end
        end
      else
        ChildControl.Visible := False;
    end;
end;


procedure TfrmMHA3_AdministerSingleItems.GoToThisQuestion(QuestionID: Integer);
var
  i,
    LabelHeight               : Integer;
begin
  lvInitialize := True;

  if (QuestionID = -1) or (QuestionID = 0) then
    Exit;

  lvLastQuestionID := IntToStr(QuestionID);
  for i := lcStartComponent to ComponentCount - 1 do
    begin
      if Components[i] is TEdit then
        if //(TEdit(Components[i]).Parent = lvScrollBox) and
          (TEdit(Components[i]).Tag = QuestionID) then
          begin
            TEdit(Components[i]).Visible := True;
            LabelHeight := IntroOrSectionOrLabel(TEdit(Components[i]).Name);
            lvComponentName := TEdit(Components[i]).Name;
            TEdit(Components[i]).Top := LabelHeight + lcSpacer;
            SetFocusedControl(TEdit(Components[i]));
            Break;
          end;
      if Components[i] is TMaskEdit then
        if //(TMaskEdit(Components[i]).Parent = lvScrollBox) and
          (TMaskEdit(Components[i]).Tag = QuestionID) then
          begin
            TMaskEdit(Components[i]).Visible := True;
            LabelHeight := IntroOrSectionOrLabel(TMaskEdit(Components[i]).Name);
            lvComponentName := TMaskEdit(Components[i]).Name;
            TMaskEdit(Components[i]).Top := LabelHeight + lcSpacer;
            SetFocusedControl(TMaskEdit(Components[i]));
            Break;
          end;
      if Components[i] is TJvValidateEdit then
        if //(TJvValidateEdit(Components[i]).Parent = lvScrollBox) and
          (TJvValidateEdit(Components[i]).Tag = QuestionID) then
          begin
            TJvValidateEdit(Components[i]).Visible := True;
            LabelHeight := IntroOrSectionOrLabel(TJvValidateEdit(Components[i]).Name);
            lvComponentName := TJvValidateEdit(Components[i]).Name;
            TJvValidateEdit(Components[i]).Top := LabelHeight + lcSpacer;
            SetFocusedControl(TJvValidateEdit(Components[i]));
            Break;
          end;
      if Components[i] is TJvDatePickerEdit then
        if //(TJvDatePickerEdit(Components[i]).Parent = lvScrollBox) and
          (TJvDatePickerEdit(Components[i]).Tag = QuestionID) then
          begin
            TJvDatePickerEdit(Components[i]).Visible := True;
            LabelHeight := IntroOrSectionOrLabel(TJvDatePickerEdit(Components[i]).Name);
            lvComponentName := TJvDatePickerEdit(Components[i]).Name;
            TJvDatePickerEdit(Components[i]).Top := LabelHeight + lcSpacer;
            SetFocusedControl(TJvDatePickerEdit(Components[i]));
            Break;
          end;
      if Components[i] is TRzSpinEdit then
        if //(TRzSpinEdit(Components[i]).Parent = lvScrollBox) and
          (TRzSpinEdit(Components[i]).Tag = QuestionID) then
          begin
            TRzSpinEdit(Components[i]).Visible := True;
            LabelHeight := IntroOrSectionOrLabel(TRzSpinEdit(Components[i]).Name);
            lvComponentName := TRzSpinEdit(Components[i]).Name;
            TRzSpinEdit(Components[i]).Top := LabelHeight + lcSpacer;
            SetFocusedControl(TRzSpinEdit(Components[i]));
            Break;
          end;
      if Components[i] is TRzRadioGroup then
        if //(TRzRadioGroup(Components[i]).Parent = lvScrollBox) and
          (TRzRadioGroup(Components[i]).Tag = QuestionID) then
          begin
            TRzRadioGroup(Components[i]).Visible := True;
            LabelHeight := IntroOrSectionOrLabel(TRzRadioGroup(Components[i]).Name);
            TRzRadioGroup(Components[i]).Top := LabelHeight + lcSpacer;
            lvComponentName := TRzRadioGroup(Components[i]).Name;
            SetFocusedControl(TRzRadioGroup(Components[i]));
            Break;
          end;
      if Components[i] is TRzListBox then
        if //(TRzListBox(Components[i]).Parent = lvScrollBox) and
          (TRzListBox(Components[i]).Tag = QuestionID) then
          begin
            TRzListBox(Components[i]).Visible := True;
            LabelHeight := IntroOrSectionOrLabel(TRzListBox(Components[i]).Name);
            lvComponentName := TRzListBox(Components[i]).Name;
            TRzListBox(Components[i]).Top := LabelHeight + lcSpacer;
            SetFocusedControl(TRzListBox(Components[i]));
            Break;
          end;
      if Components[i] is TRzCheckList then
        if //(TRzCheckList(Components[i]).Parent = lvScrollBox) and
          (TRzCheckList(Components[i]).Tag = QuestionID) then
          begin
            TRzCheckList(Components[i]).Visible := True;
            LabelHeight := IntroOrSectionOrLabel(TRzCheckList(Components[i]).Name);
            lvComponentName := TRzCheckList(Components[i]).Name;
            TRzCheckList(Components[i]).Top := LabelHeight + lcSpacer;
            SetFocusedControl(TRzCheckList(Components[i]));
            Break;
          end;
      if Components[i] is TComboBox then
        if //(TComboBox(Components[i]).Parent = lvScrollBox) and
          (TComboBox(Components[i]).Tag = QuestionID) then
          begin
            TComboBox(Components[i]).Visible := True;
            LabelHeight := IntroOrSectionOrLabel(TComboBox(Components[i]).Name);
            lvComponentName := TComboBox(Components[i]).Name;
            TComboBox(Components[i]).Top := LabelHeight + lcSpacer;
            SetFocusedControl(TComboBox(Components[i]));
            Break;
          end;
      if Components[i] is TORComboBox then
        if //(TORComboBox(Components[i]).Parent = lvScrollBox) and
          (TORComboBox(Components[i]).Tag = QuestionID) then
          begin
            TORComboBox(Components[i]).Visible := True;
            LabelHeight := IntroOrSectionOrLabel(TORComboBox(Components[i]).Name);
            lvComponentName := TORComboBox(Components[i]).Name;
            TORComboBox(Components[i]).Top := LabelHeight + lcSpacer;
            SetFocusedControl(TORComboBox(Components[i]));
            Break;
          end;
      if Components[i] is TExTrackBar then
        if //(TExTrackBar(Components[i]).Parent = lvScrollBox) and
          (TExTrackBar(Components[i]).Tag = QuestionID) then
          begin
            TExTrackBar(Components[i]).Visible := True;
            LabelHeight := IntroOrSectionOrLabel(TExTrackBar(Components[i]).Name);
            lvComponentName := TExTrackBar(Components[i]).Name;
            TExTrackBar(Components[i]).Top := LabelHeight + lcSpacer;
            SetFocusedControl(TExTrackBar(Components[i]));
            Break;
          end;
      if Components[i] is TMemo then
        if //(TMemo(Components[i]).Parent = lvScrollBox) and
          (TMemo(Components[i]).Tag = QuestionID) then
          begin
            TMemo(Components[i]).Visible := True;
            LabelHeight := IntroOrSectionOrLabel(TMemo(Components[i]).Name);
            lvComponentName := TMemo(Components[i]).Name;
            TMemo(Components[i]).Top := LabelHeight + lcSpacer;
            SetFocusedControl(TMemo(Components[i]));
            Break;
          end;
    end; // for i

  btnPrior.Enabled := True;
  btnNext.Enabled := True;
end;


function TfrmMHA3_AdministerSingleItems.GetDefaultEntry(QID: integer): string;
var
  TempStr                     : string;
  i                           : Integer;

begin
  Result := '';

  for i := 0 to lvInstrument.ListOfQuestions.Count - 1 do
    begin
      if TQuestion(lvInstrument.ListOfQuestions[i]).QuestionID = IntToStr(QID) then
        begin
          TempStr := TQuestion(lvInstrument.ListOfQuestions[i]).AnswerDisplay.DisplayMask;
          TempStr := Piece(TempStr, DELIMITER, 2);

          // This adds text from Vista calls
          // Used with SBR to view/save Pain Scores, Active Problem List
          if (lvVistaText <> '') then
            begin
              TempStr := GetVistaText(QID); // text saved to component
            // Same string is placed in ListOfAdminAnswers below
            end;
        end;
    end;

  // Check for default answers from the DLL
  for i := 0 to TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers.Count - 1 do
    begin
      if TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).QuestionID =
        IntToStr(QID) then
        begin
          if TempStr <> '' then
            begin
              TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerText := TempStr;
              TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue := TempStr;
            end
          else
            if TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue <> NOT_ASKED then
              TempStr := TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue;
        end;
    end;

  result := TempStr;
end;

procedure TfrmMHA3_AdministerSingleItems.btnCancelClick(Sender: TObject);
// This will close the form without delay, no annoying dialogs.

  function AllTabsInvisible: boolean;
  var
    i                         : integer;
  begin
    result := true;
    for i := 0 to pgctrlAllItems.PageCount - 1 do
      begin
        if pgctrlAllItems.Pages[i].TabVisible then
          result := false;
      end;
  end;

begin
  // Cancel the current instrument of several instruments

  if Administration_OPS.Administrations.Count > 1 then
    begin
      if MsgDlgFrm('Are you sure you want to cancel the ' + #13 +
        'administration of ' +
        TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).InstrumentName +
        '?' + #13 + #13 +
        'All information will be ignored.', MHA_CAPTION,
        mtConfirmation, [mbNo, mbYes], 0) = mrNo then
        exit;

      pgctrlAllItems.ActivePage.TabVisible := false;

      if AllTabsInvisible then
        begin
          lvShutDownMethod := NO_WARNING;
          gvPass := CANCEL; // used in off-line prompts
          Close;
          exit;
        end;

      pgctrlAllItems.FindNextPage(pgctrlAllItems.ActivePage, True, true);
      pgctrlAllItemsChange(Self);
    end
  else

  //Cancel the single instrument presented
    begin
      if MsgDlgFrm('Are you sure you want to cancel? ' + #13 + #13 +
        'All information will be ignored.', MHA_CAPTION,
        mtConfirmation, [mbNo, mbYes], 0) = mrNo then
        exit;

      lvShutDownMethod := NO_WARNING;
      gvPass := CANCEL; // used in off-line prompts
      Close;
    end;
end;

procedure TfrmMHA3_AdministerSingleItems.btnFinishedClick(Sender: TObject);
//var
//  i                           : Integer;
//  OK                          : Boolean;
begin
  // The heavy lifting is done at FormCloseQuery
  lvShutDownMethod := '';
  Close;

(*
  //lvShutDown := False;
  lvShutDownMethod := SKIP_WARNING;
  Ok := True;

  // Multiple Instruments: Is the user trying to save one instrument? Meaning
  // no responses have been made on the remaining instruments.
  if Administration_OPS.Administrations.Count > 1 then
    begin
      for i := 0 to Administration_OPS.Administrations.Count - 1 do
        begin
          if i <> pgctrlAllItems.ActivePageIndex then
            begin
              // This instrument has not been messed with
              if (TAdministration(Administration_OPS.Administrations[i]).NumberAnswered = 0) and
                (TAdministration(Administration_OPS.Administrations[i]).NumberSkipped = 0) then
              // Ignore the clean instruments
              else
                // Other instruments have answers
                Ok := False;
            end;
        end; // for i

      // OK = true; save the current instrument, take it off the tab sheet
      if Ok then
        begin
          CloseThisOne(lvCurrentInstrument);
          pgctrlAllItems.FindNextPage(pgctrlAllItems.ActivePage, True, True);
          pgctrlAllItemsChange(Self);

          UpdateTallyNotAskedEtc;
          Exit;
        end; // OK
    end; // if multiple presentation of instruments

  Close;
  *)
end;

function TfrmMHA3_AdministerSingleItems.CloseThisOne(WhichOne: Integer): string;
// This handles all the combinations of questions that are not asked, skipped,
// or fully answered. Required questions, left blank, are also processed here.
var
  Reply                       : TModalResult;
  ReplyStr                    : string;

  //patch 105 v65 - add message if the instrument = MINICOG and any question has been
  //            skipped that it cannot be scored.  llh
  msgStr                      : string ;

  procedure ErrorMessage;
  begin
    MsgDlgFrm(
      'This session will now close. The data already entered are saved ' + CRLF +
      'as an "Incomplete" administration in the "Instrument Administrator."' + CRLF + CRLF +
      'Once you complete the "incomplete" instrument, there may be two instances of the ' + CRLF +
      'administration of this instrument: the first with some blank items ' + CRLF +
      '(the administration with the error) and the finished "incomplete."' + CRLF +
      'The first administration should probably be deleted; see your CAC to do this.',
      TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
      mtInformation, [mbOK], 0);
  end;

begin
  result := CANCEL;
  reply := mrNone;
  screen.Cursor := crDefault;

  //patch 105 v65 add message if any skipped question in the MINICOG llh
  msgStr := '' ;
  if TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName = 'MINICOG' then
    msgStr := 'If any question has been skipped for this instrument, it cannot be scored.' ;

  // Incomplete administration: Some questions not viewed.
  // -----------------------------------------------------
  if Length(lvQuestionsNotAsked) > 1 then
    begin

      // Zero means incompletes are not allowed.
      if TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart = 0 then
        begin
          Reply := MsgDlgFrm('Some questions have not been answered in the ' +
            TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
            ' and the instrument is incomplete. ' + #13 +
            'Questions: ' + lvQuestionsNotAsked + #13 +
            'An incomplete administration will not be saved.' + #13 +
            'Are you sure you are done for now?',
            TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
            mtConfirmation, [mbYes, mbNo], 0);

          if Reply = mrYes then
            begin
              TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete := False;
              Result := OK;
              exit;
            end;

          // Return to instrument: Result = cancel;
          Exit;
        end; // DaysToRestart = 0

      // Incompletes allowed: No time limit to finish
      if TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart = -1 then
        Reply := MsgDlgFrm('Some questions have not been answered in the ' +
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
          ' and the instrument is incomplete. ' + #13 +
          'Questions: ' + lvQuestionsNotAsked + #13 + #13 +
          'You can restart this administration at any time. ' + #13 +
          'Are you sure you are done for now?',
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
          mtConfirmation, [mbYes, mbNo], 0);

      // Incompletes allowed: With a time limit to finish
      if TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart > 0 then
        Reply := MsgDlgFrm('Some questions have not been answered in the ' +
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
          ' and the instrument is incomplete. ' + #13 +
          'Questions: ' + lvQuestionsNotAsked + #13 +
          'You have ' + IntToStr(TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart) +
          ' days to finish, otherwise you will have to start over. '
          + #13 + #13 + 'Are you sure you are done for now?',
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
          mtConfirmation, [mbYes, mbNo], 0);

      TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete := False;

      if Reply = mrYes then
        begin
          if not SaveAllAnswers(WhichOne) then // Saves Admin Info & responses
            begin
              ErrorMessage;
              Result := BAD_DATA;
              exit;
            end;

          if gvUseScreenReader then
            MsgDlgFrm('Instrument saved.', 'Instrument saved.',
              mtInformation, [mbOk], 0);

          result := OK;
          Exit;
        end;

      if Reply = mrNo then // Return to instrument, to finish
        Exit;
    end;
  // End: Incomplete administration: Some questions not viewed.
  // -----------------------------------------------------

  // Incomplete administration: Some required questions have been skipped.
  // ---------------------------------------------------------------------
  if not VerifyAnswers then
    begin
      screen.Cursor := crDefault;
      if TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart = 0 then
        Reply := MsgDlgFrm('These required questions in the ' +
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
          ' have been skipped: ' + #13 +
          '  ' + lvRequiredQuestionsSkipped + #13 + #13 +
          'If you end now, the instrument can only be saved as incomplete. ' + #13 + #13 +
          'Are you sure you want to stop for now?',
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
          mtConfirmation, [mbYes, mbNo], 0)
      else
        Reply := MsgDlgFrm('These required questions in the ' +
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
          ' have been skipped: ' + #13 +
          '  ' + lvRequiredQuestionsSkipped + #13 + #13 +
          'If you end now, the instrument can only be saved as incomplete. ' +
          'You have ' + IntToStr(TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart) +
          ' days to finish, otherwise you will have to start over. ' + #13 + #13 +
          'Are you sure you want to stop for now?',
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
          mtConfirmation, [mbYes, mbNo], 0);

      if Reply = mrYes then
        begin

          if not SaveAllAnswers(WhichOne) then // Saves Admin Info & responses
            begin
              ErrorMessage;
              Result := BAD_DATA;
              exit;
            end;

          if gvUseScreenReader then
            MsgDlgFrm('Instrument saved as incomplete.',
              'Instrument saved as incomplete.',
              mtInformation, [mbOk], 0);

          result := OK;
          Exit;
        end;

      if Reply = mrNo then // Result = cancel
        Exit;
    end;
  // End: Incomplete administration: Some required questions have been skipped.
  // ---------------------------------------------------------------------


  // Incomplete administration: Some questions have been skipped.
  // ------------------------------------------------------------
  if (TAdministration(Administration_OPS.Administrations[WhichOne]).NumberAnswered =
    lvInstrument.NumberOfQuestions) and
    (TAdministration(Administration_OPS.Administrations[WhichOne]).NumberSkipped > 0) then
    begin
      Reply := MsgDlgFrm('These questions in the ' +
        TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
        ' have been skipped: ' +
        '  ' + lvQuestionsSkipped + #13 + #13 +
        'If you end now, the instrument will be saved as complete with the ' +
        'skipped question(s) as blanks.' + #13#13 + msgStr +      //patch 105 v65 added msgStr llh
        #13 + #13 + 'Are you sure you are finished?',
        TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
        mtConfirmation, [mbYes, mbNo], 0);

      if Reply = mrYes then
        begin
          TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete := True;

          if not SaveAllAnswers(WhichOne) then // Saves Admin Info & responses
            begin
              ErrorMessage;
              Result := BAD_DATA;
              exit;
            end;

          // Final Check within SaveAllAnswers revealed a blank somewhere:
          // Admin changed to Incomplete.
          if not TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete then
            begin
              result := OK;
              exit;
            end;

          ReplyStr := Administration_OPS.SaveScore(
            TAdministration(Administration_OPS.Administrations[WhichOne]));

          if ReplyStr <> '' then
            begin
              MsgDlgFrm('Problem: ' + ReplyStr, MHA_CAPTION,
                mtError, [mbOk], 0);
              ErrorMessage;
              Result := BAD_DATA;
              Exit;
            end;

          if gvUseScreenReader then
            MsgDlgFrm('Instrument saved.', 'Instrument saved.',
              mtInformation, [mbOk], 0);

          HandleProgressNote(WhichOne);

          result := OK;
          Exit;
        end;

      if Reply = mrNo then // Result = cancel
        Exit;
    end;
  // End: Incomplete administration: Some questions have been skipped.
  // ------------------------------------------------------------

  // All questions answered
  // ---------------------------------------------------------------
  if (TAdministration(Administration_OPS.Administrations[WhichOne]).NumberAnswered =
    lvInstrument.NumberOfQuestions) then
    begin
      TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete := True;

      if not SaveAllAnswers(WhichOne) then // Saves Admin Info & responses
        begin
          ErrorMessage;
          Result := BAD_DATA;
          exit;
        end;

      // Final Check revealed a blank somewhere: Admin changed to Incomplete.
      if not TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete then
        begin
          result := OK;
          exit;
        end;

      ReplyStr := Administration_OPS.SaveScore(
        TAdministration(Administration_OPS.Administrations[WhichOne]));

      // The only error that will trigger ReplyStr is HL7 troubles
      if ReplyStr <> '' then
        begin
          MsgDlgFrm('Problem: ' + ReplyStr, MHA_CAPTION,
            mtError, [mbOk], 0);
          ErrorMessage;
          Result := BAD_DATA;
          exit;
        end;

      if gvUseScreenReader then
        MsgDlgFrm('Instrument saved.', 'Instrument saved.',
          mtInformation, [mbOk], 0);

      HandleProgressNote(WhichOne);
      result := OK;
    end;
end;

(*
var
  Reply                       : TModalResult;
  ReplyStr                    : string;

  procedure ErrorMessage;
  begin
    MsgDlgFrm(
      'This session will now close. The data already entered are saved ' + CRLF +
      'as an "Incomplete" administration in the "Instrument Administrator."' + CRLF + CRLF +
      'Once you complete the "incomplete" instrument, there may be two instances of the ' + CRLF +
      'administration of this instrument: the first with some blank items ' + CRLF +
      '(the administration with the error) and the finished "incomplete."' + CRLF +
      'The first administration should probably be deleted; see your CAC to do this.',
      TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
      mtInformation, [mbOK], 0);
  end;

begin
  result := CANCEL;
  reply := mrNone;

  // Incomplete administration: Some questions not viewed
  if Length(lvQuestionsNotAsked) > 1 then
    begin

      // Zero means incompletes are not allowed.
      if TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart = 0 then
        begin
          Reply := MsgDlgFrm('Some questions have not been answered in the ' +
            TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
            ' and the instrument is incomplete. ' + #13 +
            'Questions: ' + lvQuestionsNotAsked + #13 +
            'An incomplete administration will not be saved.' + #13 +
            'Are you sure you are done for now?',
            TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
            mtConfirmation, [mbYes, mbNo], 0);

          if Reply = mrYes then
            begin
              TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete := False;
              //lvShutDownMethod := NO_WARNING;
              Result := OK;
            end;

          Result := CANCEL;
          Exit;
        end; // restart = 0

      // Incompletes allowed: No time limit to finish
      if TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart = -1 then
        Reply := MsgDlgFrm('Some questions have not been answered in the ' +
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
          ' and the instrument is incomplete. ' + #13 +
          'Questions: ' + lvQuestionsNotAsked + #13 + #13 +
          'You can restart this administration at any time. ' + #13 +
          'Are you sure you are done for now?',
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
          mtConfirmation, [mbYes, mbNo], 0);

      // Incompletes allowed: With a time limit to finish
      if TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart > 0 then
        Reply := MsgDlgFrm('Some questions have not been answered in the ' +
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
          ' and the instrument is incomplete. ' + #13 +
          'Questions: ' + lvQuestionsNotAsked + #13 +
          'You have ' + IntToStr(TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart) +
          ' days to finish, otherwise you will have to start over. '
          + #13 + #13 + 'Are you sure you are done for now?',
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
          mtConfirmation, [mbYes, mbNo], 0);

      TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete := False;

      if Reply = mrYes then
        begin
          if not SaveAllAnswers(WhichOne) then // Saves Admin Info & responses
            begin
              ErrorMessage;
              Result := BAD_DATA;
              exit;
            end;

          if gvUseScreenReader then
            MsgDlgFrm('Instrument saved.', 'Instrument saved.',
              mtInformation, [mbOk], 0);

          result := OK;
          Exit;
        end;

      if Reply = mrNo then // Return to instrument, to finish
        Exit;
    end;

  // Incomplete administration: Some required questions have been skipped
  if not VerifyAnswers then
    begin
      if TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart = 0 then
        Reply := MsgDlgFrm('These required questions in the ' +
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
          ' have been skipped: ' + #13 +
          '  ' + lvRequiredQuestionsSkipped + #13 + #13 +
          'If you end now, the instrument can only be saved as incomplete. ' + #13 + #13 +
          'Are you sure you want to stop for now?',
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
          mtConfirmation, [mbYes, mbNo], 0)
      else
        Reply := MsgDlgFrm('These required questions in the ' +
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
          ' have been skipped: ' + #13 +
          '  ' + lvRequiredQuestionsSkipped + #13 + #13 +
          'If you end now, the instrument can only be saved as incomplete. ' +
          'You have ' + IntToStr(TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart) +
          ' days to finish, otherwise you will have to start over. ' + #13 + #13 +
          'Are you sure you want to stop for now?',
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
          mtConfirmation, [mbYes, mbNo], 0);

      TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete := False;

      if Reply = mrYes then
        begin
          if not SaveAllAnswers(WhichOne) then // Saves Admin Info & responses
            begin
              ErrorMessage;
              Result := BAD_DATA;
              //Close;
              exit;
            end;

          if gvUseScreenReader then
            MsgDlgFrm('Instrument saved as incomplete.',
              'Instrument saved as incomplete.',
              mtInformation, [mbOk], 0);

          result := OK;
          Exit;
        end;

      if Reply = mrNo then
        Exit;
    end;
//ShowMessage('Num Answered: '+ inttostr(TAdministration(Administration_OPS.Administrations[WhichOne]).NumberAnswered)
//+#13+ 'Num of Q ' + inttostr(lvInstrument.NumberOfQuestions)
//+#13+ 'Num skip '+ inttostr(TAdministration(Administration_OPS.Administrations[WhichOne]).NumberSkipped));

  // Incomplete administration: Some questions have been skipped.
  if (TAdministration(Administration_OPS.Administrations[WhichOne]).NumberAnswered =
    lvInstrument.NumberOfQuestions) and
    (TAdministration(Administration_OPS.Administrations[WhichOne]).NumberSkipped > 0) then
    begin
      Reply := MsgDlgFrm('These questions in the ' +
        TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
        ' have been skipped: ' +
        '  ' + lvQuestionsSkipped + #13 + #13 +
        'If you end now, the instrument will be saved as complete with the ' +
        'skipped question(s) as blanks.' +
        #13 + #13 + 'Are you sure you are finished?',
        TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
        mtConfirmation, [mbYes, mbNo], 0);

      if Reply = mrYes then
        begin
          TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete := True;

          if not SaveAllAnswers(WhichOne) then // Saves Admin Info & responses
            begin
              ErrorMessage;
              Result := BAD_DATA;
              exit;
            end;

              // Final Check revealed a blank somewhere: Admin changed to Incomplete.
          if not TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete then
            begin
              result := OK;
              exit;
            end;

          // This saves to a consult note if needed.
          ReplyStr := Administration_OPS.SaveScoreAndHL7(lvInstrument.IsNationalTest,
            TAdministration(Administration_OPS.Administrations[WhichOne]));

          if ReplyStr <> '' then
            begin
              MsgDlgFrm('Problem: ' + ReplyStr, MHA_CAPTION,
                mtError, [mbOk], 0);
              ErrorMessage;
              Result := BAD_DATA;
              Exit;
            end;

          if gvUseScreenReader then
            MsgDlgFrm('Instrument saved.', 'Instrument saved.',
              mtInformation, [mbOk], 0);

          HandleProgressNote(WhichOne);

          result := OK;
          Exit;
        end;

      if Reply = mrNo then
        Exit;
    end;

  // All questions "answered"
  if (TAdministration(Administration_OPS.Administrations[WhichOne]).NumberAnswered =
    lvInstrument.NumberOfQuestions) then
    begin
      TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete := True;

      if not SaveAllAnswers(WhichOne) then // Saves Admin Info & responses
        begin
          ErrorMessage;
          Result := BAD_DATA;
          exit;
        end;

        // Final Check revealed a blank somewhere: Admin changed to Incomplete.
      if not TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete then
        begin
          result := OK;
          exit;
        end;

      // This saves to a consult note if needed.
      ReplyStr := Administration_OPS.SaveScoreAndHL7(lvInstrument.IsNationalTest,
        TAdministration(Administration_OPS.Administrations[WhichOne]));

      // The only error that will trigger ReplyStr is HL7 troubles
      if ReplyStr <> '' then
        begin
          MsgDlgFrm('Problem: ' + ReplyStr, MHA_CAPTION,
            mtError, [mbOk], 0);
          ErrorMessage;
          Result := BAD_DATA;
          exit;
        end;

      if gvUseScreenReader then
        MsgDlgFrm('Instrument saved.', 'Instrument saved.',
          mtInformation, [mbOk], 0);

      HandleProgressNote(WhichOne);
      result := OK;
    end;

end;
*)
(*
function TfrmMHA3_AdministerSingleItems.CloseThisOne(WhichOne: Integer): boolean;
var
  reply                       : TModalResult;
  ReplyStr                    : string;

  procedure ErrorMessage;
  begin
    MsgDlgFrm(
      'This session will close. The data already entered are saved ' + CRLF +
      'to an "Incomplete" administration in the "Instrument Administrator."' + CRLF + CRLF +
      'Once you complete the "incomplete" instrument, there will be two instances of the ' + CRLF +
      'administration of this instrument: the first with some blank items ' + CRLF +
      '(the administration with the error) and the finished "incomplete."' + CRLF +
      'The first administration should be deleted; see your CAC to do this.',
      TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
      mtInformation, [mbOK], 0);
  end;

  function SaveThisAdministration(Complete: boolean): boolean;
  begin
    result := false;
    TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete := Complete;
    if not SaveAllAnswers(WhichOne) then
      exit;
    ;
    pgctrlAllItems.pages[WhichOne].TabVisible := False;

    // This saves to a consult note if needed.
    if Complete then
      begin
        ReplyStr := Administration_OPS.SaveScoreAndHL7(lvInstrument.IsNationalTest,
          TAdministration(Administration_OPS.Administrations[WhichOne]));

        if ReplyStr <> '' then
          begin
            MsgDlgFrm('Problem: ' + ReplyStr, MHA_CAPTION,
              mtInformation, [mbOk], 0);
                //Exit;
          end;
      end;

    if gvUseScreenReader then
      MsgDlgFrm('Instrument saved.', 'Instrument saved.',
        mtInformation, [mbOk], 0);

    if Complete then
      HandleProgressNote(WhichOne);

    // Crash file no longer needed.
    LocalFiles.CleanupCrashFiles(Patient.DFN);
    result := true;
  end;

begin
  result := false;
  reply := mrNone;

  // Some questions have not been viewed. Incomplete administration
  if Length(lvQuestionsNotAsked) > 1 then
    begin
      if TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart = 0 then
        begin
          Reply := MsgDlgFrm('Some questions have not been answered in the ' +
            TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
            ' and the instrument is incomplete. ' + #13 +
            'Questions: ' + lvQuestionsNotAsked + #13 +
            'An incomplete administration will not be saved.' + #13 +
            'Are you sure you are done for now?',
            TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
            mtConfirmation, [mbYes, mbNo], 0);

          if Reply = mrYes then
            begin
              // Nothing saved
              pgctrlAllItems.pages[WhichOne].TabVisible := False;
              TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete := False;
              // Crash file no longer needed.
              LocalFiles.CleanupCrashFiles(Patient.DFN);
              result := True;
            end;
          Exit;
        end; // restart = 0

      if TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart = -1 then
        Reply := MsgDlgFrm('Some questions have not been answered in the ' +
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
          ' and the instrument is incomplete. ' + #13 +
          'Questions: ' + lvQuestionsNotAsked + #13 + #13 +
          'You can restart this administration at any time. ' + #13 +
          'Are you sure you are done for now?',
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
          mtConfirmation, [mbYes, mbNo], 0);

      if TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart > 0 then
        Reply := MsgDlgFrm('Some questions have not been answered in the ' +
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
          ' and the instrument is incomplete. ' + #13 +
          'Questions: ' + lvQuestionsNotAsked + #13 +
          'You have ' + IntToStr(TAdministration(Administration_OPS.Administrations[WhichOne]).DaysToRestart) +
          ' days to finish, otherwise you will have to start over. '
          + #13 + #13 + 'Are you sure you are done for now?',
          TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
          mtConfirmation, [mbYes, mbNo], 0);

      if Reply = mrYes then
        begin
          if not SaveThisAdministration(FALSE {IsComplete}) then
            exit;
          result := True;
          Exit;
        end;

      // User to return to instrument to finish, presumably
      if Reply = mrNo then
        Exit;
    end;

  // Verify that required questions are answered and, if needed,
  // that special questions are properly answered (MHA_Scoring_DLL).
  // Some required questions have been skipped: User will answer them.
  // Or, some answers may not be right: User will fix them.

  if not VerifyAnswers then
    exit;

  // Some questions have been skipped.
  if (TAdministration(Administration_OPS.Administrations[WhichOne]).NumberAnswered =
    lvInstrument.NumberOfQuestions) and
    (TAdministration(Administration_OPS.Administrations[WhichOne]).NumberSkipped > 0) then
    begin
      Reply := MsgDlgFrm('Some questions in the ' +
        TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName +
        ' have been skipped and are listed in status bar. ' +
        'If you end now, the instrument will be saved with the ' +
        'skipped question(s) as blanks.' +
        #13 + #13 + 'Are you sure you are finished?',
        TAdministration(Administration_OPS.Administrations[WhichOne]).InstrumentName,
        mtConfirmation, [mbYes, mbNo], 0);

      if Reply = mrYes then
        begin
          if not SaveThisAdministration(TRUE {IsComplete}) then
            exit;
          result := True;
          Exit;
        end;

      if Reply = mrNo then
        Exit;
    end;

  // All questions "answered"
  if (TAdministration(Administration_OPS.Administrations[WhichOne]).NumberAnswered =
    lvInstrument.NumberOfQuestions) then
    begin
      if not SaveThisAdministration(TRUE {IsComplete}) then
        exit;
      result := True;
    end;

end;    *)

procedure TfrmMHA3_AdministerSingleItems.HandleProgressNote(WhichOne: integer);
begin

  if DatabaseMode = dbOffLine then
    exit;

  // No progress or consult notes for restricted instruments
  if lvInstrument.R_Privilege <> '' then
    begin
      if (TAdministration(Administration_OPS.Administrations[WhichOne]).ConsultIEN <> '') then
        MsgDlgFrm(lvInstrument.InstrumentName + ' data saved. ' + #13 + #13 +
          '(A consult note was not ' +
          'written to VistA as this is a restricted instrument.)',
          lvInstrument.InstrumentName, mtInformation, [mbOk], 0)
      else
        MsgDlgFrm(lvInstrument.InstrumentName + ' data saved. ' + #13 + #13 +
          '(A progress note was not ' +
          'written to VistA as this is a restricted instrument.)',
          lvInstrument.InstrumentName, mtInformation, [mbOk], 0);
      exit;
    end;

  gvInstrumentNumber := WhichOne;
  TAdministration(Administration_OPS.Administrations[WhichOne]).IsaLegacy :=
    lvInstrument.IsLegacy;

  // If User has identified a consult note, then do not produce a progress note.
  if (TAdministration(Administration_OPS.Administrations[WhichOne]).ConsultIEN <> '') then
    begin
      if (POS('PATIENT DOES NOT HAVE', TAdministration(Administration_OPS.Administrations[WhichOne]).ConsultIEN) > 0) then
        exit;

      if Proctor.DataEntryMode = emPatient then
        begin
          Administration_OPS.SaveConsultNote(True,
            TAdministration(Administration_OPS.Administrations[WhichOne]));
          exit;
        end;

       // User can save a standard consult note, edit one, or have no consult note.
       // This form takes care of co-signer issues.
      if lvInstrument.GenerateProgressNote then
        begin
          gvAdmin := TAdministration.create;
          Administration_OPS.CopyAdmin(TAdministration(Administration_OPS.Administrations[WhichOne]),
            gvAdmin);
          try
            frmMHA_ProgressNote := TfrmMHA_ProgressNote.Create(nil);
          finally
            frmMHA_ProgressNote.Free;
          end; //try
          gvAdmin.Free;
        end;

      exit;
    end;

  //----------------  Progress note --------------------------------------
  if Proctor.DataEntryMode = emPatient then
    begin
      Administration_OPS.SaveProgressNote(True,
        TAdministration(Administration_OPS.Administrations[WhichOne]));
      exit;
    end;

  // User can save a standard progress note, edit one, or have no progress note.
  // This form takes care of co-signer issues.
  if lvInstrument.GenerateProgressNote then
    begin
      gvAdmin := TAdministration.create;
      Administration_OPS.CopyAdmin(TAdministration(Administration_OPS.Administrations[WhichOne]),
        gvAdmin);
      try
        frmMHA_ProgressNote := TfrmMHA_ProgressNote.Create(nil);
      finally
        frmMHA_ProgressNote.Free;
      end; //try
      gvAdmin.Free;
    end;
end;


procedure TfrmMHA3_AdministerSingleItems.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  Reply                       : TModalResult;
  CloseReply                  : string;
  i                           : Integer;
  AllDone                     : boolean;

begin //FormCloseQuery

  CanClose := True;

  // User presses "Cancel"
  if lvShutDownMethod = NO_WARNING then
    begin
      lvInitialize := true; // Defeat OnExit events when mnuExit or "X" is used.
      // Crash file no longer needed.
      LocalFiles.CleanupCrashFiles(Patient.DFN);
      Exit;
    end;

  // No need to show user the close options for a single form without answers.
  if ((Administration_OPS.Administrations.Count = 1) and
    (TAdministration(Administration_OPS.Administrations[0]).NumberAnswered = 1) and
    (TAdministration(Administration_OPS.Administrations[0]).NumberSkipped = 1)) then
    begin
      lvInitialize := true; // Defeat OnExit events when mnuExit is used.
      // Crash file no longer needed.
      LocalFiles.CleanupCrashFiles(Patient.DFN);
      Exit;
    end;

  // Other type of exit (not Cancel)
  if lvShutDownMethod = WARNING then
    begin
      Reply := MsgDlgFrm(
        'All of the information you have entered will be lost.' + #13 + #13 +
        '(If you prefer to save the information and complete it later, ' +
        'press the "Finish" button.)' + #13 + #13 +
        'Are you sure you want to stop now? ',
        MHA_CAPTION, mtConfirmation, [mbYes, mbNo], 0);

      if reply = mrYes then
        begin
          lvInitialize := true; // Defeat OnExit events when mnuExit is used.
          // Crash file no longer needed.
          LocalFiles.CleanupCrashFiles(Patient.DFN);
          CanClose := True;
          Exit;
        end
      else
        begin
          CanClose := False;
          Exit;
        end;
    end;

  // Active tab: Save current instrument
  CloseReply := CloseThisOne(lvCurrentInstrument);

  // User wants to keep working
  if CloseReply = CANCEL then
    begin
      CanClose := false;
      exit;
    end;

  // Saving error on this instrument
  if CloseReply = BAD_DATA then
    begin
      if Administration_OPS.Administrations.Count <> 1 then
        CanClose := false;

      // Remove this instrument from tabs
      pgctrlAllItems.pages[lvCurrentInstrument].TabVisible := False;

      lvInitialize := true;
      // Keep crash files
      exit;
    end;

  // Saving done.
  if CloseReply = OK then
    begin
      // If single administation, we are done
      if Administration_OPS.Administrations.Count = 1 then
        begin
          lvInitialize := true;
          LocalFiles.CleanupCrashFiles(Patient.DFN);
          exit;
        end;

      // Remove this instrument from tabs
      pgctrlAllItems.pages[lvCurrentInstrument].TabVisible := False;
      CanClose := false;
    end;

  { patch 105 llh - after initial test site testing a bug was found. code below sets the activepage
                    to the loop counter, displays the instrument on that tab and calls CloseThisOne
                    a 2nd time and before that instrument is displayed. This results in the 'all
                    questions have not be answered' message being displayed incorrectly as the
                    instrument has not been presented to the user yet.
                    New code to use lvCurrentInstrument, set as control to display the next
                    instrument (tab) and make it the active page until all pages (instruments)
                    have been run.

   // Check out the remaining instruments
  for i := 0 to Administration_OPS.Administrations.Count - 1 do
    begin
      //pgctrlAllItems.ActivePage.PageIndex := i;
      pgctrlAllItems.ActivePageIndex := i;
      pgctrlAllItemsChange(Self);

      // Invisible tab means the instrument is already saved
      if (pgctrlAllItems.Pages[i].TabVisible) then
        begin

          if lvIsLongInstrument then
            RecordLongInstrument(-1, -1) // Updates the tally of skipped, answered, etc.
          else
            RecordAnswer('', '', '', '', -1);

          CloseReply := CloseThisOne(i);

          if CloseReply = BAD_DATA then
            begin
              CanClose := true;
              lvInitialize := true;
              pgctrlAllItems.pages[i].TabVisible := False;
              // Keep crash files
              exit;
            end;

          if CloseReply = CANCEL then
            begin
              // Return to desired instrument
              CanClose := false;
              exit;
            end;

          if CloseReply = OK then //continue on to next instrument
            begin
              pgctrlAllItems.pages[i].TabVisible := False;
            end;
        end // if visible
      else
        continue;
    end; // for i
}

  // patch 105 llh - need to set pgctrlAllItems to next index, if it exists and display the instrument
  if (pgctrlAllItems.ActivePageIndex > -1 ) then
    pgctrlAllItemsChange(self);

  // Are all the instrument's tabs now invisible? If so we're done.
  AllDone := true;
  for i := 0 to Administration_OPS.Administrations.Count - 1 do
    begin
      if not (pgctrlAllItems.Pages[i].TabVisible) then
      else
        begin
          AllDone := false;
          break;
        end;
    end;

  if AllDone then
    begin
      lvInitialize := true;
      LocalFiles.CleanupCrashFiles(Patient.DFN);
      CanClose := true;
      lvShutDownMethod := NO_WARNING;
      Close;
    end;
end;

(*
var
  i                           : Integer;
  Reply                       : TModalResult;
begin
  CanClose := True;

  // When closing a form by clicking on System Menu ("X" in upper right corner of form),
  // the OnExit event of the last active component is not flagged.

//  if not ((LastID = '') and (LastAnswer = '')) then
//    begin
//      RecordAnswer(LastID, LastAnswer, sender);
//      LastID := '';
//      LastAnswer := '';
//    end;

  // No need to show user the close options for a single form without answers.
  if ((Administration_OPS.Administrations.Count = 1) and
    (TAdministration(Administration_OPS.Administrations[0]).NumberAnswered = 1) and
    (TAdministration(Administration_OPS.Administrations[0]).NumberSkipped = 1)) then
    begin
      lvInitialize := true; // Defeat OnExit events when mnuExit is used.
      // Crash file no longer needed.
      LocalFiles.CleanupCrashFiles(Patient.DFN);
      exit;
    end;

  if lvShutDownMethod = WARNING then
    begin
      Reply := MsgDlgFrm(
        'All of the information you have entered will be lost.' + #13 + #13 +
        '(If you prefer to save the information and complete it later, ' +
        'press the "Finish" button.)' + #13 + #13 +
        'Are you sure you want to stop now? ',
        MHA_CAPTION, mtConfirmation, [mbYes, mbNo], 0);

      if reply = mrYes then
        begin
          lvInitialize := true; // Defeat OnExit events when mnuExit is used.
      // Crash file no longer needed.
          LocalFiles.CleanupCrashFiles(Patient.DFN);
          Exit;
        end
      else
        begin
          CanClose := False;
          Exit;
        end;
    end;

  // User presses "Cancel"
  if lvShutDownMethod = NO_WARNING then
    begin
      lvInitialize := true; // Defeat OnExit events when mnuExit is used.
      // Crash file no longer needed.
      LocalFiles.CleanupCrashFiles(Patient.DFN);
      Exit;
    end;

  // Active tab: current instrument
  if CloseThisOne(lvCurrentInstrument) <> OK then
    begin
      CanClose := false;
      lvShutDownMethod := WARNING;
      exit;
    end;

  // Check out the remaining instruments
  if Administration_OPS.Administrations.Count = 1 then
    begin
      // Crash file no longer needed.
      LocalFiles.CleanupCrashFiles(Patient.DFN);
      Exit;
    end;

  for i := 0 to Administration_OPS.Administrations.Count - 1 do
    begin
      if //(i <> pgctrlAllItems.ActivePageIndex) and
      // Invisible tab means the instrument is already saved
      (pgctrlAllItems.Pages[i].TabVisible) then
        begin
          lvInstrument.InstrumentName := TAdministration(Administration_OPS.Administrations[i]).InstrumentName;

          if not ResourceFile_OPS.ReadResourceFile(lvInstrument) then
            Instrument_OPS.Load(lvInstrument);
          UpdateTallyNotAskedEtc;

          if CloseThisOne(i) <> OK then
            begin
              // Return to desired instrument
              CanClose := false;
              pgctrlAllItems.ActivePageIndex := i;
              pgctrlAllItemsChange(Self);
              Exit;
            end;
        end;
    end;
end;
*)


(*procedure TfrmMHA_AdministerOneItem.MakeActiveThisButton(
  RadioGroupName: string; KeyIn: Word);
var
  i, j                        : Integer;
  Designator                  : string;
begin

  for i := lcStartComponent to ComponentCount - 1 do
    begin
      if Components[i] is TRzRadioGroup then
        if TRzRadioGroup(Components[i]).Name = RadioGroupName then
          begin
            for j := 0 to TRzRadioGroup(Components[i]).Items.Count - 1 do
              begin
                Designator := Copy(TRzRadioGroup(Components[i]).Items[j], 1, 1);

                case KeyIn of
                  48:
                    if Designator = '0' then
                      TRzRadioGroup(Components[i]).ItemIndex := j;
                  49:
                    if Designator = '1' then
                      TRzRadioGroup(Components[i]).ItemIndex := j;
                  50:
                    if Designator = '2' then
                      TRzRadioGroup(Components[i]).ItemIndex := j;
                  51:
                    if Designator = '3' then
                      TRzRadioGroup(Components[i]).ItemIndex := j;
                  52:
                    if Designator = '4' then
                      TRzRadioGroup(Components[i]).ItemIndex := j;
                  53:
                    if Designator = '5' then
                      TRzRadioGroup(Components[i]).ItemIndex := j;
                  54:
                    if Designator = '6' then
                      TRzRadioGroup(Components[i]).ItemIndex := j;
                  55:
                    if Designator = '7' then
                      TRzRadioGroup(Components[i]).ItemIndex := j;
                  56:
                    if Designator = '8' then
                      TRzRadioGroup(Components[i]).ItemIndex := j;
                  57:
                    if Designator = '9' then
                      TRzRadioGroup(Components[i]).ItemIndex := j;
                end; // case
              end;
          end;
    end;
end;*)

procedure TfrmMHA3_AdministerSingleItems.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin

  if (Shift = [ssALT]) and (Key = vk_F4) then
    begin
      Close;
      Exit;
    end;

  if Key = vk_Escape then
    begin
      Close;
      Key := 0;
      Exit;
    end;

  if lvInitialize then
    Exit;

  if gvUseScreenReader then
    Exit;

  if (Shift = [ssShift]) or (Shift = [ssALT]) or (Shift = [ssCTRL]) then
    Exit;

  if Key = vk_Return then {- Go to next field -}
    begin
      if ActiveControl.ClassName = 'TMemo' then
        Exit;

      Key := 0; {- requires "Message" in "uses" -}
      if (ActiveControl.ClassName = 'TRzGroupButton') or
        (ActiveControl.ClassName = 'TORComboEdit') then
        GoToNextQuestion(ActiveControl.Parent.Name)
      else
        GoToNextQuestion(ActiveControl.Name);

      Exit;
    end;

// This is the case where a mouse click, or keypress, gives focus to a radio group
// item that is already checked. Number input would not normally be recognized.
  if ActiveControl.ClassName = 'TRzGroupButton' then
    begin
      KeyPressRadioGroup(Key, ActiveControl.Parent.Name);
      Exit;
    end;

  if ActiveControl.ClassName = 'TExTrackBar' then
    begin
      // These two keys will advance or decrease the position of the track-bar.
      if (Key = vk_Prior) or (Key = vk_Next) then
        begin
          Key := 0;
          Exit;
        end;

      if KeyPressTrackBar(Key, ActiveControl.Name) then
        if gvSpeedTab then
          begin
            GoToNextQuestion(ActiveControl.Name);
            exit;
          end;
    end;

  if ActiveControl.ClassName = 'TRzListBox' then
    if KeyPressListBox(Key, ActiveControl.Name) then
      if gvSpeedTab then
        begin
          GoToNextQuestion(ActiveControl.Name);
          exit;
        end;

  if ActiveControl.ClassName = 'TComboBox' then
    if KeyPressComboBox(Key, ActiveControl.Name) then
      if gvSpeedTab then
        begin
          GoToNextQuestion(ActiveControl.Name);
          exit;
        end;

  // Quirk: Pressing the ALT, Enter keys will clear a radio group, as will the
  // Buttons short-cuts
  if ActiveControl.ClassName = 'TRzRadioGroup' then
    begin
      if (Key = 67) or (Key = 78) or // C & N
        (Key = 112) or (Key = 113) or (Key = 114) or // p q r
        (Key = 13) then //or (Key = 9) then
        Exit;
      KeyPressRadioGroup(Key, ActiveControl.Name);
      Exit;
    end;
end;


procedure TfrmMHA3_AdministerSingleItems.cbSpeedTabClick(Sender: TObject);
begin
  gvSpeedTab := cbSpeedTab.Checked;
  if cbSpeedTab.Checked then
    lblHint.caption := 'Hint: Use the number key of the item to speed data entry.'
  else
    lblHint.caption := 'Hint: Press carriage return to move to next question.';
end;


function TfrmMHA3_AdministerSingleItems.VerifyAnswers: Boolean;
// False means to return to instrument to finish
// True means that the instrument is okay to save.
var
  i                           : Integer;
  AnswerString,
    ResultString              : string;
  dllHandle                   : THandle;
//  Reply                       : TModalResult;
begin
  result := false; // Default: return to instrument
  lvRequiredQuestionsSkipped := '';

  if lvIsLongInstrument then
    begin
      result := true;
      exit;
    end;

  //Check for unanwered required questions
  for i := 0 to lvInstrument.ListOfQuestions.Count - 1 do
    begin

      if (TQuestion(lvInstrument.ListOfQuestions[i]).IsRequired) then
        begin
          if (TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue =
            SKIPPED) or
            (TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue =
            NOT_ASKED) or
            (TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue =
            CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION) then
            begin
              //Inc(NumberOfSkippedRequired);
              TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue :=
                CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION;
              lvRequiredQuestionsSkipped := lvRequiredQuestionsSkipped +
                TQuestion(lvInstrument.ListOfQuestions[i]).Designator + ', ';
            end;
        end;
    end;

  // There are unanswered or skipped, required questions.
  // User cannot save as complete
  if lvRequiredQuestionsSkipped <> '' then
    begin
      TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).IsComplete := False;
      result := false;
      Exit;
    end;

  // Check for special questions
  // patch 105 added gvAuxFile which = proper C:\Program Files dir for the YS_MHA_AUX file llh
  dllHandle := LoadLibrary(PChar(gvAuxFile));

  if dllHandle = 0 then
    begin
      MsgDlgFrm('Could not find DLL used for scoring instruments.', MHA_CAPTION,
        mtWarning, [mbOk], 0);
      result := true;
      Exit;
    end
  else
    begin
      try
        @VerifyAnswersDLL := GetProcAddress(dllHandle, 'VerifyAnswers');

        // function not found
        if @VerifyAnswersDLL = nil then
          begin
            result := true;
            Exit;
          end;

        AnswerString := '';
        ResultString := '';

        for i := 0 to TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers.Count - 1 do
          AnswerString := AnswerString +
            TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).QuestionID + EQ +
            TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerValue + U + U +
            TAnswer(TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).ListOfAdminAnswers[i]).AnswerChoiceID + RECORD_END;

        if Assigned(VerifyAnswersDLL) then
          VerifyAnswersDLL(lvInstrument.InstrumentName,
            Patient.Name,
            Patient.DOB,
            Patient.Sex,
            AnswerString,
            ResultString);

        if ResultString = '' then // Continue...
        else
        // ResultString has instruction for user about an answer that
        // should be reconsidered. It may have several message which are
        // separated by "~".
          begin
            for i := 1 to NumberofPieces(ResultString, RECORD_END) do
              begin
                if (POS(IntToStr(i) + U, lvVerifyMessageSeen) = 0) and
                  (Piece(ResultString, RECORD_END, i) <> '') then
                  begin
                    if MsgDlgFrm(Piece(ResultString, RECORD_END, i) + #13 + #13 +
                      'Do you wish to change it?', MHA_CAPTION,
                      mtInformation, [mbYes, mbNo], 0) = mrYes then
                      exit
                    else
                    // User has seen message but does not want to change answer.
                    // Don't show message again.
                      lvVerifyMessageSeen :=
                        lvVerifyMessageSeen + inttoStr(i) + U;
                  end;
              end;
          end;
      finally
        FreeLibrary(dllHandle);
      end; {try..finally}

      result := true;
    end;
end;


procedure TfrmMHA3_AdministerSingleItems.MenuSelection(sender: TObject);
var
  i, j                        : Integer;
  TempStr                     : string;
begin
  j := 0;

  with Sender as TMenuItem do
    TempStr := Caption;

  for i := 0 to ComponentCount - 1 do
    if (Components[i] is TMenuItem) then
      if TempStr = TMenuItem(Components[i]).Caption then
        j := mnuView.IndexOf(TMenuItem(Components[i]));

  //lvCurrentInstrument := j;
  pgctrlAllItems.ActivePageIndex := j;
  pgctrlAllItemsChange(Sender);

end;

function TfrmMHA3_AdministerSingleItems.AdminFinalCheck(AAdmin: TAdministration): string;
begin
  result := 'Error: ' + #13;

  // Check for blanks in important fields
  with AADmin do
    begin
      if InstrumentName = '' then
        result := result + #13 + 'The instrument''s name is blank.';
      if ID = '' then
        result := result + #13 + 'The instrument''s ID is blank.';
      //FRecordID,
      //if GenerateProgNote and (ProgNoteTitle = '') then
      //  result := result + #13 + 'The Progress Note Title is blank.';
      if AdministeredBy = '' then
        result := result + #13 + 'The name of the person who administered the instrument is blank.';
      if PatientID = '' then
        result := result + #13 + 'The name of the patient is blank.';
      if OrdererDUZ = '' then
        result := result + #13 + 'The name of the person who ordered the instrument is blank.';

      if LocationIEN = '' then
        result := result + #13 + 'The ID (IEN) of the location is blank.';
      if LocationStr = '' then
        result := result + #13 + 'The name of the location is blank.';

      if (ListOfAdminAnswers.Count <> lvInstrument.NumberOfQuestions) then
        result := result + #13 + 'The number of questions and the number of answers do not match.';

      if IsComplete and (ListOfAdminAnswers.Count <> NumberAnswered) then
        result := result + #13 + 'The number of questions and the number of questions answered do not match.';
(*
      for i := 0 to ListOfAdminAnswers.Count - 1 do
        begin
          if TAnswer(aAdmin.ListOfAdminAnswers[i]).IsMultiChoice then
            begin
              if TAnswer(ListOfAdminAnswers[i]).AnswerValue = '' then
                result := result + #13 + 'The answer number ' + inttostr(i) + ' is blank.';
            end
          else
            if TAnswer(ListOfAdminAnswers[i]).AnswerText = '' then
              result := result + #13 + 'The answer number ' + inttostr(i) + ' is blank.';
        end;
*)
    end; // with AAdmin

  if length(result) < 10 then
    result := OK
  else
    result := result + #13 + #13 +
      'This administration will be saved as incomplete.';
end;

function TfrmMHA3_AdministerSingleItems.SaveAllAnswers(WhichOne: Integer): boolean;
var
  Reply,
    TempID                    : string;

  procedure HandleBlankFields;
  // Since HL7 messages cannot be blank, something must be inserted in
  // edit, memo, date, spin-edit, & currency components. Field is
  // answered, "Left blank by the user." For mask-edit, the field
  // is the masked format without text, e.g., SSN is "   -  -    ".
  // Mutiple-choice items are given the Choice ID of 1155, 1156, or 1157. That is
  // done elsewhere.
  var
    j                         : Integer;
  begin
    with TAdministration(Administration_OPS.Administrations[WhichOne]) do
      begin
        for j := 0 to ListOfAdminAnswers.Count - 1 do
          begin

            if not TAnswer(ListOfAdminAnswers[j]).IsMultiChoice then
              if (TAnswer(ListOfAdminAnswers[j]).AnswerText = '') or
                (TAnswer(ListOfAdminAnswers[j]).AnswerValue = CHOICE_ID_OF_SKIPPED_QUESTION) then
                TAnswer(ListOfAdminAnswers[j]).AnswerText := BLANK;
          end;
      end;
  end;

begin
  result := false;

  if (DataBaseMode = dbVistA) then
    begin
      Reply := AdminFinalCheck(TAdministration(Administration_OPS.Administrations[WhichOne]));

      if Reply <> OK then
        begin
          MsgDlgFrm(Reply, MHA_CAPTION, mtError, [mbOK], 0);
      // Save as an incomplete
          TAdministration(Administration_OPS.Administrations[WhichOne]).IsComplete := False;
        end;
    end;

  TempID := Administration_OPS.SaveAdmin(
    TAdministration(Administration_OPS.Administrations[WhichOne])); // New administration

  // TempID returns the ID for a new administration (when ID = NO_ID) otherwise
  // a blank is returned, unless there is an error.
  if Pos('ERROR', TempID) > 0 then
    begin
      MsgDlgFrm(
        'Sorry, a serious error has occurred. ' + #13 +
        'Saving data to file 601.84 has produced an error.' + #13 +
        TempID + #13 +
        'Please contact your local IRM or CPRS expert if this ' +
        'problem persists.',
        MHA_CAPTION, mtError, [mbOK], 0);
      Exit;
    end
  else
    begin
      if TempID <> '' then // New Admin ID
        TAdministration(Administration_OPS.Administrations[WhichOne]).ID := TempID;
    end;

  HandleBlankFields;

  if not Administration_OPS.SaveAnswers(
    TAdministration(Administration_OPS.Administrations[WhichOne])) then // Not used with ASI's.
    begin
      MsgDlgFrm(
        'Sorry, a serious error has occurred. ' + #13 +
        'Saving data to file 601.85 has produced an error.' + #13 +
        'Please contact your local IRM or CPRS expert if this ' +
        'problem persists.',
        MHA_CAPTION, mtError, [mbOK], 0);
      Exit;
    end;

  result := true;
end;

procedure TfrmMHA3_AdministerSingleItems.pgctrlAllItemsChange(Sender: TObject);
var
  i                           : Integer;
begin
  SetPiece(lvFirstQuestionID, U, lvCurrentInstrument + 1, lvLastQuestionID);

  lvCurrentInstrument := pgctrlAllItems.ActivePageIndex;

  lvInstrument.InstrumentName := TAdministration(
    Administration_OPS.Administrations[lvCurrentInstrument]).InstrumentName;

  // Load the active instrument
  if not ResourceFile_OPS.ReadResourceFile(lvInstrument) then
    Instrument_OPS.Load(lvInstrument);

  Caption := lvInstrument.InstrumentName + ': ' + Patient.Name + '  ';

  for i := 0 to ComponentCount - 1 do
    if Components[i] is TScrollBox then
      if TScrollBox(Components[i]).Name = 'Form' +
        IntToStr(pgctrlAllItems.ActivePageIndex) then
        begin
          lvInitialize := true;
          lvScrollBox := TScrollBox(Components[i]);
          //ActiveControl := lvScrollBox;
          Break;
        end;

  SetupLongInstrument;
  UpdateStatusBar('  ' + lvInstrument.Copyright);
  // This keeps track of the last question recorded on the new instrument--it could be
  // the first question.

  GoToThisQuestion(StrToIntDef(Piece(lvFirstQuestionID, U, lvCurrentInstrument + 1), -1));
  lvInitialize := False;
end;

procedure TfrmMHA3_AdministerSingleItems.btnReviewClick(Sender: TObject);
var
  TempID                      : string;
begin
  TempID := '';

  try
    frmMHA3_Want_Review := TfrmMHA3_Want_Review.Create(nil);
    frmMHA3_Want_Review.DisplayQA(
      TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]),
      lvInstrument);
    frmMHA3_Want_Review.ShowModal;
    TempID := frmMHA3_Want_Review.ChangeAnswer;
  finally
    frmMHA3_Want_Review.Free;
  end;

  if tempID <> '' then
    begin
      HideAllItems;
      GoToThisQuestion(StrToIntDef(TempID, -1));
      lvInitialize := False;
    end;
end;

procedure TfrmMHA3_AdministerSingleItems.btnNextClick(Sender: TObject);
begin
  GoToNextQuestion(lvComponentName);
end;

procedure TfrmMHA3_AdministerSingleItems.btnPriorClick(Sender: TObject);
begin
  GoToPriorQuestion(lvComponentName);
end;

procedure TfrmMHA3_AdministerSingleItems.mnuPrintBlankClick(Sender: TObject);
begin
  gvNameOfInstrument := lvInstrument.InstrumentName;

  try
    frmPrint_Instrument := TfrmPrint_Instrument.Create(nil);
  finally
    frmPrint_Instrument.Free;
  end; //try

end;

procedure TfrmMHA3_AdministerSingleItems.mnuOptionsClick(Sender: TObject);
begin
  MsgDlgFrm(
    'Sorry, once an instrument is in the data-entry phase, the font, ' +
    'highlight colors, and the like, can not be changed. However, these features ' +
    'can be customized in the previous window where the instrument was selected.',
    MHA_CAPTION, mtInformation, [mbOK], 0);
end;

procedure TfrmMHA3_AdministerSingleItems.mnuAboutClick(Sender: TObject);
begin
  frmMHA_About := TfrmMHA_About.Create(nil);
  try
    frmMHA_About.ShowModal;
  finally
    frmMHA_About.Free;
  end; //try
end;

procedure TfrmMHA3_AdministerSingleItems.DoubleClickMessage(Sender: TObject);
begin
  MsgDlgFrm('Sorry, this question does not respond to double-clicks of the mouse.',
    MHA_CAPTION, mtInformation, [mbOk], 0);
end;

procedure TfrmMHA3_AdministerSingleItems.FormResize(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  // Reset these, used in placement of dialogs
  gvLeft := Left;
  gvTop := Top;
  gvHeight := Height;
  gvWidth := Width;
end;

(*function TfrmMHA_AdministerOneItem.CheckForCustomOperation: boolean;
var
  AnswerString,
    ResultString,
    TempStr                   : string;
  i                           : Integer;
  dllHandle                   : THandle;
begin
  result := False;
  dllHandle := LoadLibrary(YS_MHA_AUX);

  if dllHandle = 0 then
    begin
      MsgDlg('Could not find DLL used for scoring instruments.', ID_CAPTION,
        mtWarning, [mbOk], 0);
      FreeLibrary(dllHandle);
      Exit;
    end
  else
    begin
      try
        @CustomDLL := GetProcAddress(dllHandle, 'Custom');

        if @CustomDLL = nil then
          begin
          // function not found
            FreeLibrary(dllHandle);
            Exit;
          end;

        AnswerString := '';
        ResultString := '';

        for i := 0 to Instrument.ListOfAnswers.Count - 1 do
          AnswerString := AnswerString +
            TAnswer(Instrument.ListOfAnswers[i]).QuestionID + EQ +
            TAnswer(Instrument.ListOfAnswers[i]).AnswerValue + U + U +
            TAnswer(Instrument.ListOfAnswers[i]).AnswerID + RECORD_END;

        if Assigned(CustomDLL) then
          CustomDLL(Instrument.InstrumentName,
            'Perkins, Lenwood (Fictitious name)', //Patient.Name,
            'March 14,1950', //Patient.DOB,
            'MALE', //Patient.Sex,
            AnswerString,
            ResultString);

        if (ResultString = '') or
          (ResultString = AnswerString) then
          begin
            FreeLibrary(dllHandle);
            Exit;
          end
        else
          begin
            for i := 0 to Instrument.ListOfAnswers.Count - 1 do
              begin
                // ResultString starts at 1, ListOfAnswers at 0
                TempStr := Piece(ResultString, RECORD_END, i + 1);
                TAnswer(Instrument.ListOfAnswers[i]).AnswerValue :=
                  Piece(Piece(TempStr, EQ, 2), U, 1);
                //TAnswer(Instrument.ListOfAnswers[i]).AnswerText :=
                //  TAnswer(Instrument.ListOfAnswers[i]).AnswerValue;
              end;
            result := True;
          end;

      finally
        FreeLibrary(dllHandle);
      end; {try..finally}
    end;

end; *)

procedure TfrmMHA3_AdministerSingleItems.mnuMetricClick(Sender: TObject);
begin
  VisitURL(MetricURL);
end;

procedure TfrmMHA3_AdministerSingleItems.mnuOnlineSupportClick(Sender: TObject);
begin
  VisitURL(MHA_ONLINE_SUPPORT_URL);
end;

procedure TfrmMHA3_AdministerSingleItems.ChangePT;
begin
  MsgDlgFrm('The patient in CPRS has been changed and does not match ' +
    'the patient in use here.' + #13 + #13 +
    'You should close this administration(s) immediately.',
    MHA_CAPTION, mtWarning, [mbOk], 0);

end;

procedure TfrmMHA3_AdministerSingleItems.LoadVistaText(InstrumentName: string);
var
  ResultString                : string;
  dllHandle                   : THandle;

begin
	// patch 105 added gvAuxFile which = proper C:\Program Files dir for the YS_MHA_AUX file llh
  dllHandle := LoadLibrary(PChar(gvAuxFile));

  if dllHandle = 0 then
    begin
      // patch 105 added gvAuxFile, removed C:\Program Files llh
      MsgDlgFrm('Could not find DLL used for displaying instruments.' +
        'The file YS_MHA_AUX.DLL is usually located in ' + gvAuxFile , '',
        mtWarning, [mbOk], 0)
    end
  else
    begin
      try
        @VistaLookupDLL := GetProcAddress(dllHandle, 'VistaLookup');

        if @VistaLookupDLL = nil then
          // function not found
          Exit;

        if Assigned(VistaLookupDLL) then
          VistaLookupDLL(
            DataModule1.RPCBroker1,
            InstrumentName,
            TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).PatientID,
            TAdministration(Administration_OPS.Administrations[lvCurrentInstrument]).OrdererDUZ,
            ResultString);

        lvVistaText := ResultString;
        // Format is:
        // #=Text^#=text

      finally
        FreeLibrary(dllHandle);
      end; {try..finally}
    end;
end;


function TfrmMHA3_AdministerSingleItems.GetVistaText(Where: integer): string;
var
  Count,
    i                         : Integer;
  TempStr                     : string;
begin
  result := '';

  if lvVistaText = '' then
    Exit;
  Count := NumberOfPieces(lvVistaText, U);

  if Count = 0 then
    Exit;

  for i := 1 to Count do
    begin
      TempStr := Piece(lvVistaText, U, i);
      if IntToStr(Where) = Piece(TempStr, EQ, 1) then
        result := Piece(TempStr, EQ, 2);
    end;

end;

procedure TfrmMHA3_AdministerSingleItems.LoadStaticText(InstrumentName: string);
var
  ResultString                : string;
  dllHandle                   : THandle;

begin
	// patch 105 added gvAuxFile which = proper C:\Program Files dir for the YS_MHA_AUX file llh
  dllHandle := LoadLibrary(PChar(gvAuxFile));

  if dllHandle = 0 then
    begin
      // patch 105 added gvAuxFile, removed C:\Program Files llh
      MsgDlgFrm('Could not find DLL used for displaying instruments.' +
        'The file YS_MHA_AUX.DLL is usually located in ' + gvAuxFile , '',
        mtWarning, [mbOk], 0)
    end
  else
    begin
      try

        @StaticLookupDLL := GetProcAddress(dllHandle, 'StaticLookup');

        if @StaticLookupDLL = nil then
          // function not found
          Exit;

        if Assigned(StaticLookupDLL) then
          StaticLookupDLL(
            InstrumentName,
            '', //PatientIEN
            '', // OrderedByDUZ
            ResultString);

        lvStaticText := ResultString;
        // Format is:
        // #=Text^#=text

      finally
        FreeLibrary(dllHandle);
      end; {try..finally}
    end;
end;

end.

