unit fMHA_Options;

//llh patch 129 -- remove references to GAF in Options menu
// removed GAF from rgpMenu.items[3]


interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  StdCtrls,
  Buttons,
  ExtCtrls,
  ComCtrls,
  RzCmboBx,
  RzPanel,
  RzLabel,
  JvXPCheckCtrls,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons,
  Dialogs,
  uIA_GUIControl,
  RzRadGrp;


type
  TfrmMHA_Options = class(TForm)
    pgctrlOptions: TPageControl;
    tabFont: TTabSheet;
    tabHighlight: TTabSheet;
    pnlHighlight: TPanel;
    lblHighFore: TLabel;
    lblHighBack: TLabel;
    pnlSample: TPanel;
    cbxColorForeground: TRzColorComboBox;
    tabMisc: TTabSheet;
    cbxColorBackground: TRzColorComboBox;
    pnlMisc: TPanel;
    pnlButton: TPanel;
    lblSpeedTab: TLabel;
    pnlSpacer: TPanel;
    pnlFont: TPanel;
    lblCurrentFont: TLabel;
    lblFont: TLabel;
    pnlSpacer0: TPanel;
    lblOriginalFont: TLabel;
    pnlRadio: TPanel;
    btnDone: TJvXPButton;
    btnCancel: TJvXPButton;
    chkVisualBeep: TJvXPCheckbox;
    chkScreenReader: TJvXPCheckbox;
    chkSpeedTab: TJvXPCheckbox;
    chkImage: TJvXPCheckbox;
    chkFontOverride: TJvXPCheckbox;
    FontDialog1: TFontDialog;
    btnCPRSFont: TJvXPButton;
    btnWinFont: TJvXPButton;
    btnNewFont: TJvXPButton;
    tabMenu: TTabSheet;
    rgpMenu: TRzRadioGroup;

    procedure btnOKClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure cbxColorForeground1Change(Sender: TObject);
    procedure ChangeSample;
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure ConfigureForm;
    procedure CreateFont(Sender: TObject);
    procedure FontSetToWindows(Sender: TObject);
    procedure FontSetToCPRS(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure pgctrlOptionsChange(Sender: TObject);
    procedure btnCPRSFontClick(Sender: TObject);
    procedure btnWinFontClick(Sender: TObject);
    procedure btnNewFontClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMHA_Options              : TfrmMHA_Options;

implementation

{$R *.DFM}

uses
  uIA_Const,
  uMHA_Const,
  fIA_dialogs,
  uMHA_Registry;

var
  lvInitialize                : Boolean;
  lvOriginalFont              : TFont;

  NonClientMetrics            : TNonClientMetrics;
  WinFont                     : TFont;
  CPRSFont                    : HFONT;
  CPRS_Font                   : TFont;


procedure TfrmMHA_Options.FormCreate(Sender: TObject);
begin
  // Keep Position out of ConfigureForm when ConfigureForm can be called
  // more than once with the form.
  Position := poDesigned;
  lvInitialize := True;

  // Retain the original font in case the user cancels after
  // viewing other fonts.
  lvOriginalFont := TFont.Create;
  lvOriginalFont.Assign(gvFont);

  // Used to get Windows menu font
  NonClientMetrics.cbSize := sizeof(NonClientMetrics);
  WinFont := TFont.Create;

  // Used for CPRS font
  CPRS_Font := TFont.Create;

  if gvCPRS_Instance_Handle <> 0 then // Window found
    begin
      CPRSFont := SendMessage(gvCPRS_Instance_Handle, WM_GETFONT, 0, 0);
      CPRS_Font.handle := CPRSFont;
    end
  else
    begin
      // Default settings for CPRS font if CPRS not open.
      CPRS_Font.Name := 'MS Sans Serif';
      CPRS_Font.Size := 8;
      CPRS_Font.Color := clBlack;
      CPRS_Font.Style := [];
    end;

  if Pos('0', gvShowOptionTabs) > 0 then
    tabFont.TabVisible := True
  else
    tabFont.TabVisible := False;
  if Pos('1', gvShowOptionTabs) > 0 then
    tabHighlight.TabVisible := True
  else
    tabHighlight.TabVisible := False;
  if Pos('2', gvShowOptionTabs) > 0 then
    tabMisc.TabVisible := True
  else
    tabMisc.TabVisible := False;

  ConfigureForm;
  ShowModal;
end;


procedure TfrmMHA_Options.FormShow(Sender: TObject);
begin

  MHA_Registry.ReadRegistry;
  cbxColorForeground.SelectedColor :=
    gvHighlightForeground;
  cbxColorBackground.SelectedColor :=
    gvHighlightBackground;

  chkVisualBeep.Checked := gvVisualBeeps;
  chkImage.checked := gvShowImage;

  chkScreenReader.Checked := gvUseScreenReader;
  chkSpeedtab.Checked := gvSpeedTab;
  chkFontOverride.Checked := gvUseDesignFont;

  rgpMenu.ItemIndex := gvStart;

  lvInitialize := False;

  ChangeSample;

  // Active control is set to first component of active page, unless
  // tabFont where proper font button is active.
  pgctrlOptionsChange(Sender);
  Screen.Cursor := crDefault;

  ActiveControl := pgctrlOptions;
end;


procedure TfrmMHA_Options.ConfigureForm;
const
  BestHeight                  : Integer = 450;
  BestWidth                   : Integer = 600;
var
  i,
    TempHeight,
    TempWidth                 : Integer;

begin
  Font.Assign(gvFont);

  Scaled := True;
  AutoScroll := False;
  AutoSize := False;
  PixelsPerInch := 120;

  KeyPreview := True;
  HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
  VertScrollBar.Range := 0;

  Caption := 'User Preferences ';

  with GUI_Control do
    begin
      ConfigureGUI(gvFont);

      // This is widest caption (plus 25%); make it fit on one line.
      TempWidth := (5 * TextWidth(chkSpeedTab.Caption, gvFont) div 4) +
        chkSpeedTab.Left;
      if TempWidth > pnlMisc.Width then
        Width := TempWidth + 25
      else
        Width := BestWidth;

      Width := GetWidth(Width);
      TempHeight := 16 * TextHeight('Yy', gvFont) +
        (3 * ButtonHeight); {pnlButton.Height plus tab height}

      Height := GetHeight(TempHeight);
      Left := GetLeft(Width);
      Top := GetTop(Height);
      if CheckWidth(Width) or CheckHeight(Height) then
        WindowState := wsMaximized;
      Refresh; // resets alClient dimensions

      //------ Common components in all tabs -----------------------------------

      for i := 0 to ComponentCount - 1 do
        begin

          if Components[i] is TLabel then
            TLabel(Components[i]).Height := TextHeight('Yy', gvFont);
          if Components[i] is TJvXPCheckBox then
            TJvXPCheckBox(Components[i]).Height := TextHeight('Yy', gvFont);
          if Components[i] is TEdit then
            TEdit(Components[i]).Height :=
              (3 * TextHeight('Yy', gvFont)) div 2;
          if Components[i] is TRzColorComboBox then
            begin
              TRzColorComboBox(Components[i]).Height := 2 * TextHeight('Yy',
                gvFont);
              TRzColorComboBox(Components[i]).ItemHeight := 3 * TextHeight('Yy',
                gvFont) div 2;
              TRzColorComboBox(Components[i]).Width := (width div 2) -
                (Width div 6);
              TRzColorComboBox(Components[i]).Left := width div 2;
            end;
          if Components[i] is TJvXPButton then
            begin
              TJvXPButton(Components[i]).Height := ButtonHeight;
              TJvXPButton(Components[i]).Width :=
                ButtonWidth(TJvXPButton(Components[i]).Caption);
            end;
        end;

      pnlButton.Height := (3 * ButtonHeight) div 2;
      btnDone.Top := (pnlButton.Height - ButtonHeight) div 2;
      btnCancel.Top := btnDone.Top;
      btnDone.Left := pnlButton.Width - btnDone.Width - btnDone.Width div 3;
      btnCancel.Left := btnDone.Width div 3;

      //------ Font Tab  -------------------------------------------------------

      pnlSpacer0.Height := TextHeight('Yy', gvFont);
      lblCurrentFont.Caption := '      Font on display: ' + gvFont.Name +
        '   Size: ' + IntToStr(gvFont.Size);
      lblCurrentFont.Font.Assign(gvFont);

      // WIN7SCROLLBUG - need to make the label height more to accomodate new message llh
      lblCurrentFont.Height := 33 ;
      lblFont.Height := 7 * TextHeight('Yy', gvFont);
      //lblFont.Height := 4 * TextHeight('Yy', gvFont);

      btnCPRSFont.Top := pnlRadio.Height div 2;
      btnWinFont.Top := btnCPRSFont.Top;
      btnNewFont.Top := btnCPRSFont.Top;

      btnCPRSFont.Left := btnCPRSFont.Width div 16;
      btnWinFont.Left := (pnlRadio.Width - btnWinFont.Width) div 2;
      btnNewFont.Left := pnlRadio.Width - btnNewFont.Width - btnCPRSFont.Left;

      //------ Highlight Tab ---------------------------------------------------

      pnlSample.Font.Assign(gvFont); // needed
      pnlSample.Font.Color := cbxColorForeground.SelectedColor;
      pnlSample.Color := cbxColorBackground.SelectedColor;
      lblHighFore.Anchors := [];
      lblHighBack.Anchors := [];
      cbxColorForeGround.Anchors := [];
      cbxColorBackGround.Anchors := [];
      lblHighFore.Width := TextWidth('Highlight Color Background: ', gvFont);
      lblHighBack.Width := lblHighFore.Width;
      cbxColorForeGround.Width := (pnlHighlight.Width div 2) - ButtonWidthNormal div 4;
      cbxColorBackGround.Width := cbxColorForeGround.Width;

      lblHighFore.Top := pnlHighLight.Height div 8;
      cbxColorForeGround.Top := lblHighFore.Top;
      lblHighBack.Top := pnlHighLight.Height div 2;
      cbxColorBackGround.Top := lblHighBack.Top;
      pnlSample.Top := 3 * (pnlHighLight.Height div 4);
      pnlSample.Height := 2 * lblHighFore.Height;

      lblHighFore.Left := (pnlHighlight.Width div 2) - lblHighFore.Width;
      lblHighBack.Left := lblHighFore.Left;
      cbxColorForeGround.Left := pnlHighlight.Width div 2;
      cbxColorBackGround.Left := cbxColorForeGround.Left;

      //------ Misc Tab --------------------------------------------------------

      chkVisualBeep.Top := 1 * (pnlMisc.Height div 18);
      chkScreenReader.Top := 3 * (pnlMisc.Height div 18);
      chkSpeedtab.top := 5 * (pnlMisc.Height div 18);
      lblSpeedtab.top := 6 * (pnlMisc.Height div 18);

      chkImage.top := 8 * (pnlMisc.Height div 18);
      chkFontOverride.top := 11 * (pnlMisc.Height div 18);
      // Align alBottom
      lblOriginalFont.Height := 3 * TextHeight('Yy', gvFont);

      chkSpeedTab.Visible := True;
      lblSpeedTab.Visible := True;
      chkImage.Visible := True;

    //----------- Menu ---------------------------------------------------------
      //llh patch 129 removed GAF, adjust itemindex from 5 to 4
      rgpMenu.ItemEnabled[0] := True;
      rgpMenu.ItemHeight := TextHeight('Yy', gvFont);

    end; // with GUI_Control
end;


procedure TfrmMHA_Options.ChangeSample;
begin
  if tabHighlight.Visible then
    begin
      pnlSample.Font.Assign(gvFont);
      pnlSample.Font.Color := cbxColorForeground.SelectedColor;
      pnlSample.Color := cbxColorBackground.SelectedColor;
    end;

  lblCurrentFont.Font.Assign(gvFont);
  lblCurrentFont.Caption := '      Font on display: ' +
    gvFont.Name +
    '   Size: ' + IntToStr(gvFont.Size);
  Update;
end;

procedure TfrmMHA_Options.btnOKClick(Sender: TObject);
begin

  gvVisualBeeps := chkVisualBeep.Checked;
  gvShowImage := chkImage.Checked;

  gvUseScreenReader := chkScreenReader.Checked;
  gvSpeedTab := chkSpeedTab.Checked;
  gvUseDesignFont := chkFontOverride.Checked;
  gvHighlightForeground := cbxColorForeground.SelectedColor;
  gvHighlightBackground := cbxColorBackground.SelectedColor;

  gvStart := rgpMenu.ItemIndex;

  GUI_Control.NeedGUIUpdate := true;

  MHA_Registry.SaveRegistry;
  Close;
end;

procedure TfrmMHA_Options.btnCancelClick(Sender: TObject);
begin
  lvInitialize := True;
  // Do not make change if user is just messing around
  gvFont.Assign(lvOriginalFont);
  Close;
end;

procedure TfrmMHA_Options.cbxColorForeground1Change(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  if cbxColorBackground.SelectedColor = cbxColorForeground.SelectedColor then
    begin
      MsgDlgFrm('The foreground and background cannot be the same color. Default '
        + 'colors will be substituted.',
        MHA_Caption, mtInformation, [mbOK], 0);
      lvInitialize := True;
      cbxColorForeground.SelectedColor := clBlack;
      cbxColorBackground.SelectedColor := clYellow;
      lvInitialize := false;
    end;
  ChangeSample;
end;

procedure TfrmMHA_Options.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = [ssALT]) and (Key = vk_F4) then
    Close;

  if Key = vk_Escape then
    begin
      Close;
      Key := 0;
    end;
end;

procedure TfrmMHA_Options.CreateFont(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  FontDialog1.Font := gvFont;

  if FontDialog1.Execute then
    begin
      gvFont.Assign(FontDialog1.Font);

      ConfigureForm;
    end;

end;

procedure TfrmMHA_Options.FontSetToWindows(Sender: TObject);
var
  NonClientMetrics            : TNonClientMetrics;
begin
  if lvInitialize then
    Exit;

  NonClientMetrics.cbSize := sizeof(NonClientMetrics);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @NonClientMetrics, 0) then
    begin
      gvFont.Handle :=
        CreateFontIndirect(NonClientMetrics.lfMessageFont);

      ConfigureForm;
    end;
end;

procedure TfrmMHA_Options.FontSetToCPRS(Sender: TObject);
var
  CPRSFont                    : HFONT;
begin
  if lvInitialize then
    Exit;

  if gvCPRS_Instance_Handle <> 0 then // Window found
    begin
      CPRSFont := SendMessage(gvCPRS_Instance_Handle, WM_GETFONT, 0, 0);
      gvFont.Handle := CPRSFont;
    end
  else
    begin
          // Default settings for CPRS font
      gvFont.Name := 'MS Sans Serif';
      gvFont.Size := 8;
      gvFont.Color := clBlack;
      gvFont.Style := [];
    end;

  ConfigureForm;

end;

procedure TfrmMHA_Options.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  lvOriginalFont.Free;
  CPRS_Font.Free;
  WinFont.Free;

  //Action := caFree;
end;

procedure TfrmMHA_Options.pgctrlOptionsChange(Sender: TObject);
begin
  // Set the active control; important for 508 screen readers.
  if pgctrlOptions.ActivePage = tabFont then
    begin
      ActiveControl := btnDone;
    end;
  if pgctrlOptions.ActivePage = tabHighlight then
    begin
      ChangeSample;
      ActiveControl := cbxColorForeground;
    end;
  if pgctrlOptions.ActivePage = tabMisc then
    ActiveControl := chkVisualBeep;
end;

procedure TfrmMHA_Options.btnCPRSFontClick(Sender: TObject);
begin
  FontSetToCPRS(Sender);
end;

procedure TfrmMHA_Options.btnWinFontClick(Sender: TObject);
begin
  FontSetToWindows(Sender);
end;

procedure TfrmMHA_Options.btnNewFontClick(Sender: TObject);
begin
  CreateFont(Sender);
end;

procedure TfrmMHA_Options.FormResize(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  // Reset these, used in placement of dialogs
  gvLeft := Left;
  gvTop := Top;
  gvHeight := Height;
  gvWidth := Width;
end;

end.

