unit uMHA_Battery_OPS;

interface

uses
  SysUtils,
  Contnrs,
  uMHA_Battery,
  fIA_dialogs, Dialogs,
  Classes;

type
  TBattery_OPS = class
  private
    FListOfBatteries: TObjectList;
  protected
  public
    constructor Create;
    destructor Destroy; override;

    property ListOfBatteries: TObjectList read FListOfBatteries
      write FListOfBatteries;

    procedure Clear(aBattery: TBattery);
    function LoadBatteries: string;
    function SaveBattery(aBattery: TBattery): string;
    function DeleteBattery(aBatteryName: string): string;
    function BatteryNameUnique(NameIn: string): Boolean;
    function RenameBattery(OldName, NewName: string): string;
  end; { TBattery_OPS }

var
  Battery_OPS                 : TBattery_OPS;


implementation

uses
  uIA_Const,
  uIA_Utils,
  uMHA_DM,
  uMHA_Battery_IO;


constructor TBattery_OPS.Create;
begin
  inherited Create;
  FListOfBatteries := TObjectList.Create;
end;


destructor TBattery_OPS.Destroy;
begin
  FListOfBatteries.Free;
  inherited Destroy;
end;

procedure TBattery_OPS.Clear(aBattery: TBattery);
begin
  aBattery.InstrumentsInBattery.Clear;
end;

function TBattery_OPS.LoadBatteries: string;
// Populate ListOfBatteries
begin
  FListOfBatteries.Clear;
  result := BatteryReadList(DataModule1.RPCBroker1); // uMHA_Battery_IO
end;


function TBattery_OPS.SaveBattery(aBattery: TBattery): string;
begin
  Result := BatteryWrite(aBattery, DataModule1.RPCBroker1); // uMHA_Battery_IO
end;

function TBattery_OPS.DeleteBattery(aBatteryName: string): string;
var
  i                           : Integer;
  TempName,
    TempID                    : string;
begin
  Result := OK;

  if ((aBatteryName = '') or (aBatteryName = ' ')) then
    exit;

  TempName := aBatteryName;

  // Remove battery from ListOfObjects and the M-side.
  for i := 0 to FListOfBatteries.Count - 1 do
    begin
      if Trim(UpperCase(TempName)) =
        Trim(UpperCase(TBattery(FListOfBatteries[i]).BatteryName)) then
        begin
          TempID := TBattery(FListOfBatteries[i]).BatteryID;
          FListOfBatteries.Delete(i);

          // remove battery from M-side
          if TempID <> '' then
            Result := BatteryDelete(TempID, DataModule1.RPCBroker1); // See uMHA_Battery_IO
          Break;
        end;
    end;

end;


function TBattery_OPS.BatteryNameUnique(NameIn: string): Boolean;
// Unique name is "true"
var
  i                           : Integer;
  TempName                    : string;

begin
  Result := True;
  TempName := NameIn;

  for i := 0 to FListOfBatteries.Count - 1 do
    if Trim(UpperCase(TempName)) =
      Trim(UpperCase(TBattery(FListOfBatteries[i]).BatteryName)) then
      begin
        Result := False;
        Break;
      end;
end;


function TBattery_OPS.RenameBattery(OldName, NewName: string): string;
var
  TempName                    : string;
  i                           : Integer;
begin
  Result := '';
  TempName := OldName;

  for i := 0 to FListOfBatteries.Count - 1 do
    begin
      if Trim(UpperCase(TempName)) =
        Trim(UpperCase(TBattery(FListOfBatteries[i]).BatteryName)) then
        begin
          Result := BatteryRename(NewName,
            TBattery(FListOfBatteries[i]).BatteryID, DataModule1.RPCBroker1);
          Exit;
        end;
    end;
end;

initialization
  Battery_OPS := TBattery_OPS.Create;

finalization
  Battery_OPS.Free;

end.

