unit uMHA_Const;

interface

uses
  Trpcb,
  uMHA_Administration,
  Classes;

type
  TDatabaseMode = (dbVista, dbOffLine, dbNil);
  TOpenForm = (ofMain, ofAll, ofSingle, ofASI, ofASI_508, ofGAF, ofSingleMenu,
    ofSelect, ofResult, ofUpload, ofASIMenu);

// Input patient data: DOB, gender, etc.
// Output is new answer list.
  TDefaultAnswersDLL = procedure(
    InstrumentName,
    UserName,
    StationCode,
    StationSuffix,
    PatientName,
    PatientSSN,
    PatientDOB,
    PatientSex,
    AnswerString: string;
    var
    ResultString: string); stdcall;

// Ensure the answers are correct.
// Output is error message: Blank output means okay.
  TVerifyAnswersDLL = procedure(
    InstrumentName,
    PatientName,
    PatientDOB,
    PatientSex,
    // llh patch 129 --- issue with AUX dll not working AnwserString defined as a String
    //                   in other areas of application
    //AnswerString: AnsiString ; //string;
    AnswerString: String ; //string;
    var
    ResultString: string); stdcall;


// Modify acceptable anwsers to a preferred or legacy format
// Output is new answer list.
  TGroomAnswersDLL = procedure(
    InstrumentName,
    PatientName,
    PatientDOB,
    PatientSex,
    AnswerString: string;
    var
    ResultString: string); stdcall;


// Score answers for special instruments: *ScaleName^ScaleIEN^ScaleScore*
  TScoreAnswersDLL = procedure(
    InstrumentName,
    PatientName,
    PatientDOB,
    PatientSex,
    // llh patch 129 --- issue with AUX dll not working
    //AnswerString: AnsiString ; //string;
    AnswerString: String ; //string;
    var
    ResultString: string); stdcall;


// Progress Note for special instruments: report is output.
  TProgressNoteDLL = procedure(
    InstrumentName,
    PatientName,
    PatientDOB,
    PatientSex,
    AnswerString: string;
    var
    ResultString: string); stdcall;

// Special operations, custom controls
// Output is new answer list.
  TCustomDLL = procedure(
    InstrumentName,
    PatientName,
    PatientDOB,
    PatientSex,
    AnswerString: string;
    var
    ResultString: string); stdcall;


// Output is string, placed on form when built
  TStaticLookupDLL = procedure(
    InstrumentName,
    PatientIEN,
    OrderedByDUZ: string;
    var
    ResultString: string); stdcall;

// Output is string, placed on form when built
  TVistaLookupDLL = procedure(
    Broker: TRPCBroker;
    InstrumentName,
    PatientIEN,
    OrderedByDUZ: string;
    var
    ResultString: string); stdcall;

  TSpecialResultsDLL = procedure(
    var
    ResultString: string); stdcall;

const
 {assign a nil - not loaded function}
  GroomAnswersDLL             : TGroomAnswersDLL = nil;
  VerifyAnswersDLL            : TVerifyAnswersDLL = nil;
  DefaultAnswersDLL           : TDefaultAnswersDLL = nil;
  ScoreAnswersDLL             : TScoreAnswersDLL = nil;
  ProgressNoteDLL             : TProgressNoteDLL = nil;
  CustomDLL                   : TCustomDLL = nil;
  StaticLookupDLL             : TStaticLookupDLL = nil;
  VistaLookupDLL              : TVistaLookupDLL = nil;
  SpecialResultsDLL           : TSpecialResultsDLL = nil;

var
  DataBaseMode                : TDataBaseMode;
  OpenFormName                : TOpenForm;

const
  MHA_Caption                 = 'Mental Health Assistant';
  OFFLINE_CAPTION             = '  (Off-line Mode)';
  YS_MHA_AUX                  = '\Vista\Common Files\YS_MHA_AUX.dll';
  MHA_ONLINE_SUPPORT_URL      = 'http://vaww.mentalhealth.domain/';
  MetricURL                   =
    'http://vaww.mentalhealth.domain/mha/metric.asp';
  CONSULT_NOTE_TITLE          = 'MENTAL HEALTH CONSULT NOTE';
  PROGRESS_NOTE_TITLE         = 'MENTAL HEALTH DIAGNOSTIC STUDY NOTE';

  // MH Vista File numbers
  FILE_TESTS_AND_SURVEYS      = '601.71';
  FILE_QUESTIONS              = '601.72';
  FILE_INTRODUCTIONS          = '601.73';
  FILE_RESPONSE_TYPES         = '601.74';
  FILE_CHOICES                = '601.75';
  FILE_CHOICETYPES            = '601.751';
  FILE_INSTRUMENT_CONTENT     = '601.76';
  FILE_BATTERIES              = '601.77';
  FILE_BATTERY_CONTENTS       = '601.78';
  FILE_BATTERY_USERS          = '601.781';
  FILE_SKIPPED_QUESTIONS      = '601.79';
  FILE_SECTIONS               = '601.81';
  FILE_RULES                  = '601.82';
  FILE_INSTRUMENTRULES        = '601.83';
  FILE_ADMINISTRATIONS        = '601.84';
  FILE_ANSWERS                = '601.85';
  FILE_SCALEGROUPS            = '601.86';
  FILE_SCALES                 = '601.87';
  FILE_DISPLAY                = '601.88';
  FILE_CHOICEIDENTIFIERS      = '601.89';
  FILE_SCORING_KEYS           = '601.91';
  FILE_MH_RESULTS             = '601.92';
  FILE_REPORT                 = '601.93';

  ASI_FULL                    = 'Full';
  ASI_LITE                    = 'Lite';
  ASI_FOLLOWUP                = 'Follow-up';
  ASI_MV                      = 'ASI-MV';

var
  gvASIWidth,
    gvSelectWidth,
    gvResultWidth,
    gvGAFWidth,
    gvSingleWidth,
    gvASIHeight,
    gvSelectHeight,
    gvResultHeight,
    gvGAFHeight,
    gvSingleHeight,
    gvASIGeneral,
    gvASIMedical,
    gvASIEmploy,
    gvASIDrug,
    gvASILegal,
    gvASIFamily,
    gvASISocial,
    gvASIPsych,
    gvLastASIProgram,
    gvStart // Menu selection
                              : Integer;

  gvShowOptionTabs            : string;

  gvID,
    gvLastLocation,
    gvPass,
    //gvJAWS_Dir,
    gvSpecialResultsList,
    gvUpdateSeries,
    gvDLL_Version,
    gvCoSignerIEN,
    gvProgressNoteIEN
                              : string;

  gvShowAllOrOneItem,
    gvVisualBeeps,
    gvDefaultSize,
    gvShowHints,
    gvShowImage,
    gvShowScreenReaderMsg,
    gvCheckResourceFiles,
    gvCPRSShutDown
                              : Boolean;

  gvDateGiven,
    gvUpdate                  : TDateTime;

  gvAPICalls                  : TStringList;

  gvAdmin                     : TAdministration;

  gvASI_Type                  : integer; // 1-Full, 2-Lite, 3-Followup, 4 - MV

  // patch 105 - Get Environment Variable to determine which program file directory should
  //             be accessed for finding the YS_MHA_AUX.dll file and set into global Aux File var  llh
  gvEnvVar,
    gvAuxFile                 : string ;


implementation

end.

