
unit uMHA_DM;

interface

uses
  Windows,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  fIA_dialogs, Dialogs,
  Trpcb,
  RPCConf1,
  Diaccess,
  Fmcmpnts,
  Db,
  StdCtrls,
  Messages,
  Fmcntrls;    // removed StGenLog -- use Codesite instead

type

  TDataModule1 = class(TDataModule)
    FMValidator1: TFMValidator;
    FMFiler1: TFMFiler;
    ASI_FMGets1: TFMGets;
    RPCBroker1: TRPCBroker;
   // procedure DataModuleCreate(Sender: TObject);
   // procedure DataModuleDestroy(Sender: TObject);
  private
    { Private declarations }
    FServerVersion: string;
    //FClientVersion: string;
    FServerRequired: string;
    function GetMHAClientVersion: string;
    function GetMHAServerVersion: string;
    function GetMHAServerRequired: string;
  public
    { Public declarations }
    //Server Methods
    function ServerConnectToVista: BOOLEAN;
    //procedure ServerSetBrokerIPandPort(AIP: string; APort: integer);
    function ConnectToVista(Sender: TObject): BOOLEAN;
    property MHAClientVersion: string read GetMHAClientVersion;
    property MHAServerVersion: string read GetMHAServerVersion;
    property MHAServerRequired: string read GetMHAServerRequired;
    function ClientVersion(const AFileName: string): string;
  end;

var
  DataModule1                 : TDataModule1;

implementation

uses
  uIA_Const,
  uMHA_Const,
  uIA_Utils,
  uMHA_LocalFiles;

{$R *.DFM}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//           Server/Client synchronization methods
//               some code borrowed from CPRS
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

const
  //Following constants borrowed from CPRS

  TX_VER1                     = 'This is version ';
  TX_VER2                     = ' of Mental Health Assistant.';
  TX_VER3                     = CRLF + 'The Vista-MHA version is ';
  TX_VER_REQ                  = ' version Vista-MHA is required.';
  TX_VER_OLD                  = CRLF + 'It is strongly recommended that you upgrade.';
  TX_VER_OLD2                 = CRLF +
    'The program cannot be run online until MHA is upgraded.';
  TX_VER_NEW                  = CRLF +
    'The program cannot be run online until Vista-MHA is upgraded. Most likely '+
    'a new patch has not been installed.';

  { values that can be passed to FileVersionValue }
  FILE_VER_FILEVERSION        = '\StringFileInfo\040904E4\FileVersion';
  FILE_VER_INTERNALNAME       = '\StringFileInfo\040904E4\InternalName';
  FILE_VER_PRODUCTVERSION     = '\StringFileInfo\040904E4\ProductVersion';

function TDataModule1.ClientVersion(const AFileName: string): string; //CPRS Code
var
  ASize, AHandle              : DWORD;
  Buf                         : string;
  FileInfoPtr                 : Pointer; //PVSFixedFileInfo;
begin
  Result := '';
  ASize := GetFileVersionInfoSize(PChar(AFileName), AHandle);
  if ASize > 0 then
    begin
      SetLength(Buf, ASize);
      GetFileVersionInfo(PChar(AFileName), AHandle, ASize, Pointer(Buf));
      VerQueryValue(Pointer(Buf), '\', FileInfoPtr, ASize);
      with TVSFixedFileInfo(FileInfoPtr^) do
        Result := IntToStr(HIWORD(dwFileVersionMS)) + '.' +
          IntToStr(LOWORD(dwFileVersionMS)) + '.' +
          IntToStr(HIWORD(dwFileVersionLS)) + '.' +
          IntToStr(LOWORD(dwFileVersionLS));
    end;
end;


function ServerVersion: string;
// fix to patch 129, network path issue
var
  chr1 :  char;
  chr2 :  char;
begin
  if not DataModule1.RPCBroker1.Connected then
    exit;

  with DataModule1.RPCBroker1 do
    begin
      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ VERSRV: Successful');
      RemoteProcedure := 'YTQ VERSRV';
      Param[0].Mult['"YSB"'] := 'YS BROKER1';

      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ VERSRV: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ VERSRV: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ VERSRV: Error. Default value set to 0.0.0.0.');
          result := '0.0.0.0';
        end
      else
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ VERSRV: Successful');
          result := Piece(results[1], U, 1); //Server Version Number
          //patch 129 temporary fix -- allow server to optionally specify the DLL path
          if (results.Count > 4) and (results[4] <> '') then
          begin
            // llh fix to deficiency in patch 129, need to take into account network paths
            gvAuxFile := results[4];
            chr1 := gvAuxFile[1]  ;
            chr2 := gvAuxFile[2];
            // network drive entered or full path entered in MHA AUX parameter
            if ( (chr1 = '\') and (chr2 = '\') ) or
              // A = 65, Z= 90, a = 97 z = 122
              ( (ord(chr1) > 64) and (ord(chr1) < 91) ) or
              ( (ord(chr1) > 96) and (ord(chr1) < 123) )then exit ;
            if gvAuxFile[1] = '\' then gvAuxFile := gvEnvVar +  gvAuxFile ;
          end;
        end;
    end;
end;

function FileVersionValue(const AFileName, AValueName: string): string;
type
  PValBuf = ^TValBuf;
  TValBuf = array[0..255] of Char;
var
  VerSize, ValSize, AHandle   : DWORD;
  VerBuf                      : Pointer;
  ValBuf                      : PValBuf;
begin
  Result := '';
  VerSize := GetFileVersionInfoSize(PChar(AFileName), AHandle);
  if VerSize > 0 then
    begin
      GetMem(VerBuf, VerSize);
      try
        GetFileVersionInfo(PChar(AFileName), AHandle, VerSize, VerBuf);
        VerQueryValue(VerBuf, PChar(AValueName), Pointer(ValBuf), ValSize);
        SetString(Result, ValBuf^, ValSize);
      finally
        FreeMem(VerBuf);
      end;
    end;
end;

function CompareVersion(const A, B: string): Integer;
var
  NumA, NumB                  : Integer;
begin
  NumA := (StrToInt(Piece(A, '.', 1)) * 16777216) +
    (StrToInt(Piece(A, '.', 2)) * 65536) +
    (StrToInt(Piece(A, '.', 3)) * 256) +
    StrToInt(Piece(A, '.', 4));
  NumB := (StrToInt(Piece(B, '.', 1)) * 16777216) +
    (StrToInt(Piece(B, '.', 2)) * 65536) +
    (StrToInt(Piece(B, '.', 3)) * 256) +
    StrToInt(Piece(B, '.', 4));
  Result := NumA - NumB;
end;

function TDataModule1.GetMHAServerVersion: string;
begin
  if FServerVersion = '' then
    FServerVersion := ServerVersion;
  result := FServerVersion;
end;

function TDataModule1.GetMHAClientVersion: string;
begin
(*
  if FClientVersion = '' then
    FClientVersion := ClientVersion(Application.ExeName);
  result := FClientVersion;*)
  result := ClientVersion(Application.ExeName);
end;

function TDataModule1.GetMHAServerRequired: string;
begin
  if FServerRequired = '' then
    FServerRequired :=
      Piece(FileVersionValue(Application.ExeName, FILE_VER_INTERNALNAME), ' ',
      1);
  result := FServerRequired;
end;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//           Server Methods
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

function TDataModule1.ConnectToVista(Sender: TObject): BOOLEAN;
var
  ServerStr                   : string;
  Port                        : integer;
begin
  Result := False;
  DataBaseMode := dbNil;

  { BEGIN CPRS Tools menu = ParamCount}
  if (ParamCount > 4) then
    begin
    // Get server and port from commandline
      ServerStr := Piece(ParamStr(1), EQ, 2);
      Port := StrToIntDef(Piece(ParamStr(2), EQ, 2), 9200);

      RPCBroker1.Server := ServerStr;
      RPCBroker1.ListenerPort := Port;

      if not ServerConnectToVista then
        begin
          // Unable to connect to Vista. User may want to work off-line
          exit;
        end
      else
        begin
        // This doesn't get a chance to display. The RPC broker catches the
        // error, displays a message and closes down this application.
          if not RPCBroker1.CreateContext('YS BROKER1') then //Set MHA context
            begin
              ShowMessage('Unable to create YS BROKER1 context in Vista. ' + #13 + #13 +
                'Insure that the option YS BROKER1 [YS BROKER1] is on the ' + #13 +
                'Mental Health users'' secondary menus, as well as the secondary ' + #13 +
                'menus of all users who currently have the OR CPRS GUI CHART ' + #13 +
                'option. ');
              Application.Terminate;
              exit;
            end;

          DataBaseMode := dbVista;
          Result := True;
        end;

    //VERSION CHECK CODE FOLLOWS
      if (MHAServerVersion = '0.0.0.0') then
        begin
          ShowMessage('Unable to determine current version of server.');
          DataBaseMode := dbNil;
          Exit;
        end;

      if (MHAServerRequired = '') then
        begin
          ShowMessage('SERVERREQUIRED not identified in Project Options.');
          DataBaseMode := dbNil;
          Exit;
        end;

      if RPCBroker1.Connected then //Only if connected is any of this relevant
        begin
      // If Server is newer than Required, then Client needs updating
          if (CompareVersion(MHAServerVersion, MHAServerRequired) > 0) then
            begin
              Showmessage(TX_VER1 + MHAClientVersion + TX_VER2 + CRLF +
                MHAServerRequired + TX_VER_REQ + TX_VER3 + MHAServerVersion + '.' +
                TX_VER_OLD2);
              DataBaseMode := dbNil;
              Exit;
            end;

      // If Server older than Required, then Server needs updating
          if (CompareVersion(MHAServerVersion, MHAServerRequired) < 0) then
            begin
              ShowMessage(TX_VER1 + MHAClientVersion + TX_VER2 + CRLF +
                MHAServerRequired + TX_VER_REQ + TX_VER3 + MHAServerVersion + '.' +
                TX_VER_NEW);
              DataBaseMode := dbNil;
              Exit;
            end;
        end;

    end; //Param.Count > 4
end;


function TDataModule1.ServerConnectToVista: BOOLEAN;
begin
  Result := True;
  try
    RPCBroker1.Connected := True;

    if POS('Sign-on was not completed', RPCBroker1.RPCBError) > 0 then
      begin
        result := false;
        Halt;
      end;

  except
    on E: EBrokerError do
      begin
        Result := False;
        Halt;
      end;
  end;

end;


end.

