unit uMHA_Import_IO;

interface

uses
  classes,
  SysUtils,
  dialogs,
  Trpcb,
  graphics,
  uIA_Const,
  uMHA_Const,
  uIA_Display,
  uIA_Key,
  uIA_ScaleGroup,
  uIA_Scale,
  uIA_Rule,
  uIA_Question,
  uIA_Instrument;

function WriteInstrumentName(aInstrumentName: string;
  Broker: TRPCBroker): string;

function WriteInstrumentDescription(aInstrument: TInstrument;
  Broker: TRPCBroker): string;

function WriteQuestion(aInstrument: TInstrument;
  IntroID, IntroDisplayID: string;
  AQuestion: TQuestion; Broker: TRPCBroker): string;

function WriteMultiChoiceItems(ThisQuestion: TQuestion;
  Broker: TRPCBroker): string;

function WriteChoice(aChoice: string;
  LegacyValue: Char;
  var aChoiceIEN: string; Broker: TRPCBroker): string;

function WriteScaleGroup(aInstrumentID: string; AScaleGroup: TScaleGroup; Broker: TRPCBroker): string;
function WriteScale(AScale: TScale; Broker: TRPCBroker): string;
function WriteScoringKey(AKey: TKey; Broker: TRPCBroker): string;
function WriteReport(aReport, aInstrumentID, ReportIEN: string; Broker: TRPCBroker): string;
function WriteDisplay(ADisplay: TDisplay;
  const IsAnswerDisplay: BOOLEAN; Broker: TRPCBroker): string;
function WriteIntroText(aIntro: string; aIntroIEN: string;
  Broker: TRPCBroker): string;
function WriteSection(aIEN, aInstrumentID, aFirstQuestion, aTabCaption,
  aSectionCaption, aDisplayID: string; Broker: TRPCBroker): string;
function WriteRule(ARule: TRule; Broker: TRPCBroker): string;
function WriteRuleInstrument(RuleID, QuestionID, InstrumentID: string;
  Broker: TRPCBroker): string;
function WriteSkippedQuestions(InstrumentID, RuleID, ASkippedQuestionIDs: string;
  Broker: TRPCBroker): string;

//function InstrumentReadIEN(aInstrumentName: string; Broker: TRPCBroker): string;

implementation

uses
  uIA_Question_OPS,
  uIA_Rule_OPS,
  uMHA_Import,
  uMHA_LocalFiles,
  uIA_Utils;


function WriteInstrumentName(aInstrumentName: string;
  Broker: TRPCBroker): string;
var
  i                           : integer;
begin

  with Broker do
    begin
      CreateContext('YS BROKER1');

      ClearParameters := True;
      ClearResults := True;
      LocalFiles.UpdateAPILog('Call to YTQ TEST ADD: ' +
        ' CODE=' + aInstrumentName);

      RemoteProcedure := 'YTQ TEST ADD';
      Param[0].Mult['"CODE"'] := aInstrumentName;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ TEST ADD: Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ TEST ADD: Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if (Results[0] = '[ERROR]') then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ TEST ADD: Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          Result := 'ERROR: YTQ TEST ADD FAILED.' + #13 +
            Results[1];
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ TEST ADD: Successful');
          Result := OK; //When successful this is returned:
        end;
    end;
end;


function WriteInstrumentDescription(aInstrument: TInstrument;
  Broker: TRPCBroker): string;
var
  i                           : integer;

  function CheckVistaString(StringIn: string): string;
  // Ensure the Vista data is erased when the user removes the text.
  begin
    if Length(StringIn) = 0 then
      Result := '@' // Blank in MUMPS
    else
      Result := StringIn;
  end;

begin

  with Broker, aInstrument do
    begin
        //// Stub ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
        FILE_TESTS_AND_SURVEYS +
        ' instrument description');
      RemoteProcedure := 'YTQ STUB';
      Param[0].Mult['"FILEN"'] := FILE_TESTS_AND_SURVEYS;   //601.71
      Param[0].Mult['"IEN"'] := IEN; // Don't use InstrumentName here
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_TESTS_AND_SURVEYS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_TESTS_AND_SURVEYS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

//      if results[0] = '[ERROR]' then
//        begin
//          LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//            FILE_TESTS_AND_SURVEYS + ': Error ');
//          for i := 0 to Results.Count - 1 do
//            LocalFiles.UpdateAPILog('   ' + Results[i]);
//          result := 'ERROR: YTQ STUB: ' + FILE_TESTS_AND_SURVEYS + ' FAILED. ' + #13 +
//            results[1];
//          exit;
//        end;

      //// Save ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;
      LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
        FILE_TESTS_AND_SURVEYS +
        ' .01=' + InstrumentName);

      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_TESTS_AND_SURVEYS;

      // Note that the Mult and piece 1 are matched whenever possible,
      // but they need not be the same number. What matters is that the right
      // side of the := match the field number on the M side.

      Param[0].Mult['"1"'] := '.01^' + InstrumentName;
      Param[0].Mult['"2"'] := '2^' + CheckVistaString(PrintTitle);
      Param[0].Mult['"3"'] := '3^' + CheckVistaString(Version);
      Param[0].Mult['"4"'] := '4^' + CheckVistaString(Author);
      Param[0].Mult['"5"'] := '5^' + CheckVistaString(Publisher);
      Param[0].Mult['"6"'] := '7.5^' + CheckVistaString(Reference);
      Param[0].Mult['"7"'] := '7^' + CheckVistaString(PublicationDate);

      if A_PRIVILEGE <> '' then
        Param[0].Mult['"8"'] := '8^' + A_PRIVILEGE
      else
        Param[0].Mult['"8"'] := '8^@'; // blank

      if R_PRIVILEGE <> '' then
        Param[0].Mult['"9"'] := '9^' + R_PRIVILEGE
      else
        Param[0].Mult['"9"'] := '9^@'; // blank

      if IsOperational = 'Y' then
        Param[0].Mult['"10"'] := '10^' + 'Y';
      if IsOperational = 'N' then
        Param[0].Mult['"10"'] := '10^' + 'N';
      if IsOperational = 'U' then
        Param[0].Mult['"10"'] := '10^U'; // Under Development

      if RequiresLicense then
        Param[0].Mult['"11"'] := '11^' + 'Y'
      else
        Param[0].Mult['"11"'] := '11^' + 'N';

      Param[0].Mult['"12"'] := '12^' + CheckVistaString(Purpose);
      Param[0].Mult['"13"'] := '13^' + CheckVistaString(NormSample);
      Param[0].Mult['"14"'] := '14^' + CheckVistaString(TargetPopulation);
      Param[0].Mult['"15"'] := '15^' + CheckVistaString(EnteredBy);
      Param[0].Mult['"16"'] := '16^' + EntryDate;
      Param[0].Mult['"17"'] := '17^' + CheckVistaString(LastEditedBy);
      Param[0].Mult['"18"'] := '18^' + LastEditDate;

      if IsNationaltest then
        Param[0].Mult['"19"'] := '19^' + 'Y'
      else
        Param[0].Mult['"19"'] := '19^' + 'N';

      if LicenseIsCurrent then
        Param[0].Mult['"20"'] := '20^' + 'Y'
      else
        Param[0].Mult['"20"'] := '20^' + 'N';

      Param[0].Mult['"21"'] := '21^' + CheckVistaString(Copyright);

      if RequiresSignature then
        Param[0].Mult['"22"'] := '22^' + 'Y'
      else
        Param[0].Mult['"22"'] := '22^' + 'N';

      if IsLegacy then
        Param[0].Mult['"23"'] := '23^' + 'Y'
      else
        Param[0].Mult['"23"'] := '23^' + 'N';

      if SubmitToNationalDB then
        Param[0].Mult['"24"'] := '24^' + 'Y'
      else
        Param[0].Mult['"24"'] := '24^' + 'N';

      if IsCopyrighted then
        Param[0].Mult['"25"'] := '25^' + 'Y'
      else
        Param[0].Mult['"25"'] := '25^' + 'N';

      if WriteFullText then
        Param[0].Mult['"26"'] := '26^' + 'Y'
      else
        Param[0].Mult['"26"'] := '26^' + 'N';

      if DaysToRestart > -1 then
        Param[0].Mult['"27"'] := '27^' + IntToStr(DaysToRestart)
      else
        Param[0].Mult['"27"'] := '27^2'; // Two days default

      if GenerateProgressNote then
        Param[0].Mult['"28"'] := '28^' + 'Y'
      else
        Param[0].Mult['"28"'] := '28^' + 'N';

      Param[0].Mult['"29"'] := '29^' + CheckVistaString(TIU_Title);

      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' + FILE_TESTS_AND_SURVEYS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' + FILE_TESTS_AND_SURVEYS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if (Results[0] = '[ERROR]') then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' + FILE_TESTS_AND_SURVEYS + ': Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          Result := 'ERROR: YTQ SAVE: ' + FILE_TESTS_AND_SURVEYS + ' FAILED. ' + #13 +
            Results[1];
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' + FILE_TESTS_AND_SURVEYS + ': Successful');
          Result := OK; // When successful this is returned:
        end;
    end; //with datamodule
end;


function WriteQuestion(aInstrument: TInstrument;
  IntroID, IntroDisplayID: string;
  AQuestion: TQuestion; Broker: TRPCBroker): string;
var
  QuestText,
    YS                        : string;
  i                           : Integer;
  TempFont                    : TFont;

begin
  Result := 'OK';

  with Broker, AQuestion, Question_OPS do
    begin
      //// Stub ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
        FILE_QUESTIONS +
        ' questions');
      RemoteProcedure := 'YTQ STUB';
      Param[0].Mult['"FILEN"'] := FILE_QUESTIONS; // 601.72
      Param[0].Mult['"IEN"'] := QuestionID;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_QUESTIONS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_QUESTIONS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

//      if results[0] = '[ERROR]' then
//        begin
//          LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//            FILE_QUESTIONS + ': Error ');
//          for i := 0 to Results.Count - 1 do
//            LocalFiles.UpdateAPILog('   ' + Results[i]);
//          result := 'ERROR: YTQ STUB: ' + FILE_QUESTIONS + ' FAILED. ' + #13 +
//            results[1];
//          exit;
//        end;

      //// Save ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
        FILE_QUESTIONS +
        ' new question');
      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_QUESTIONS; // 601.72
      Param[0].Mult['"IEN"'] := QuestionID;

      Param[0].Mult['"2"'] := '2^' + IntroID + '^1'; //Introduction.ID;

      Param[0].Mult['"3"'] := '3^' + ConvertRespTypeToNumberStr(ResponseType) +'^1';

      Param[0].Mult['"4"'] := '4^' + ChoiceTypeID + '^1';

      Param[0].Mult['"5"'] := '5^' + IntToStr(qMin);

      Param[0].Mult['"6"'] := '6^' + IntToStr(qMax);

      if IsRequired then
        Param[0].Mult['"7"'] := '7^Y'
      else
        Param[0].Mult['"7"'] := '7^N';

      Param[0].Mult['"8"'] := '8^' + qHint;

      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_QUESTIONS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_QUESTIONS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
            FILE_QUESTIONS + ': Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          result := 'ERROR: YTQ SAVE: ' + FILE_QUESTIONS + ' FAILED. ' + #13 +
            results[1];
          exit;
        end;

      /////////////////////// SAVE WORD-PROCESSING FIELD SEPARATELY //////////////////

   //   if IsNewQuestion then //get IEN of saved NEW question
   //   QuestionID := Piece(Results[1], U, 2); //'Update ok^33^+'

      ClearParameters := True;
      ClearResults := True;
      QuestText := '';

      LocalFiles.UpdateAPILog('Call to YTQ WP FILER: ' +
        'FILEN= ' + FILE_QUESTIONS +
        ' IEN= ' + QuestionID);

      //Save field #1 separately, since it's a WP field
      RemoteProcedure := 'YTQ WP FILER';
      Param[0].Mult['"FILEN"'] := FILE_QUESTIONS; // 601.72
      Param[0].Mult['"IEN"'] := QuestionID;
      Param[0].Mult['"FIELD"'] := '1';

      // Wrap lines so they are 200 char or less. Do not break within a word.
      QuestText := WrapString(qText, 200); // uIA_Utils

      if QuestText <> '' then
        begin
          for i := 1 to NumberOfPieces(QuestText, U) do
            begin
              YS := '"' + IntToStr(i) + '"';
              Param[0].Mult[YS] := Piece(QuestText, U, i);
            end;

          Param[0].PType := List;
          try
            Call;

          except
            on E: EBrokerError do
              begin
                LocalFiles.UpdateAPILog('Call to YTQ WP FILER: Error ');
                LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
                exit;
              end;
            on E: EAccessViolation do
              begin
                LocalFiles.UpdateAPILog('Call to YTQ WP FILER: Error ');
                LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
                exit;
              end;
          end; // try

          if results[0] = '[ERROR]' then
            begin
              LocalFiles.UpdateAPILog('Call to YTQ WP FILER: Error ');
              for i := 0 to Results.Count - 1 do
                LocalFiles.UpdateAPILog('   ' + Results[i]);
              result := 'ERROR: YTQ WP FILER: ' + FILE_QUESTIONS + ' FAILED. ' + #13 +
                results[1];
              exit;
            end;
        end;

      ///////////////// ASSOCIATE QUESTION WITH INSTRUMENT CONTENTS FILE /////////////

      //CONDENSED DATA DICTIONARY---MH INSRUMENT CONTENT FILE (#601.76)UCI: VAH,ROU
      //
      //STORED IN: ^YTT(601.76,                                   02/13/04    PAGE 2
      //--------------------------------------------------------------------------------
      //
      //FIELD     FIELD
      //NUMBER    NAME
      //
      //.01       INSTRUMENT CONTENT id (RNJ5,0X), [0;1]
      //1         INSRUMENT (RP601.71'), [0;2]
      //2         QUESTION SEQUENCE (NJ5,0), [0;3]
      //3         QUESTION ID (P601.72'), [0;4]
      //4         DESIGNATOR (F), [0;5]
      //5         QUESTION TEXT (Cmw), [ ; ]
      //6         CHOICETYPE (CJ8), [ ; ]
      //7         QUESTIONDISPLAYID (P601.88'), [0;6]
      //8         INTRODISPLAYID (P601.88'), [0;7]
      //9         MHCHOICEDISPLAYID (P601.88'), [0;8]

      // invoke API to associate New Question with current Instrument
      ClearParameters := True;
      ClearResults := True;

      // If this is a new instrument, we need to set a default style value
      TempFont := TFont.Create;
      TempFont.Name := 'MS Sans Serif';
      TempFont.Size := 8;
      TempFont.Color := clWindowText;
      TempFont.Style := [];

      if Display.DisplayID = '' then
        Display.DisplayID :=
          Import.GetDisplayID(
          TempFont,
          taLeftJustify,
          1 {Left},
          False, // Has answer component
          NA, 0, NA);

      if IntroDisplayID = '' then
        IntroDisplayID :=
          Import.GetDisplayID(
          TempFont,
          taLeftJustify,
          1 {Left},
          False, // Has answer component
          NA, 0, NA);

      if AnswerDisplay.DisplayID = '' then
        AnswerDisplay.DisplayID :=
          Import.GetDisplayID(
          TempFont,
          taLeftJustify,
          1 {Left},
          True, // Has answer component
          NA, 1, RADIOGROUP_AC);

      //// Stub ////////
      LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
        FILE_INSTRUMENT_CONTENT +
        ' instrument-content');
      RemoteProcedure := 'YTQ STUB';
      Param[0].Mult['"FILEN"'] := FILE_INSTRUMENT_CONTENT; // 601.76
      Param[0].Mult['"IEN"'] := InstrumentContentsID;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_INSTRUMENT_CONTENT + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_INSTRUMENT_CONTENT + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

//      if results[0] = '[ERROR]' then
//        begin
//          LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//            FILE_INSTRUMENT_CONTENT + ': Error ');
//          for i := 0 to Results.Count - 1 do
//            LocalFiles.UpdateAPILog('   ' + Results[i]);
//          result := 'ERROR: YTQ STUB: ' + FILE_INSTRUMENT_CONTENT + ' FAILED. ' + #13 +
//            results[1];
//          exit;
//        end;

      //// Save ////////
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
        FILE_INSTRUMENT_CONTENT +
        ' IEN^= ' + InstrumentContentsID);

      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_INSTRUMENT_CONTENT; // 601.76
      Param[0].Mult['"IEN"'] := InstrumentContentsID;
      Param[0].Mult['"1"'] := '1^' + aInstrument.IEN +'^1';
      //Param[0].Mult['"1"'] := '1^' + '`' + aInstrument.IEN;
      //    if Instrument.NumberOfQuestions = 0 then
      //      Param[0].Mult['"2"'] := '2^' + '1' // First question added to Instrument
      //    else

      Param[0].Mult['"2"'] := '2^' + InstrumentContentsSequenceField;
      Param[0].Mult['"3"'] := '3^' +  QuestionID + '^1';
      Param[0].Mult['"4"'] := '4^' + Designator;
      Param[0].Mult['"7"'] := '7^' +  Display.DisplayID + '^1';
      Param[0].Mult['"8"'] := '8^' +  IntroDisplayID + '^1';
      Param[0].Mult['"9"'] := '9^' +  AnswerDisplay.DisplayID + '^1';

      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_INSTRUMENT_CONTENT + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_INSTRUMENT_CONTENT + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
            FILE_INSTRUMENT_CONTENT + ': Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          result := 'ERROR: YTQ SAVE: ' + FILE_INSTRUMENT_CONTENT + ' FAILED. ' + #13 +
            results[1];
          exit;
        end;

      //InstrumentContentsID :=
      //  Piece(Broker.Results[1], U, 2);

    end; //with rpcBroker

end; {WriteQuestion}


function WriteChoice(aChoice: string; LegacyValue: Char;
  var aChoiceIEN: string; Broker: TRPCBroker): string;
// Searches for choice text already present and returns the IEN if it is present.
// If new, returns the new IEN.
var
  i                           : integer;
begin

  with Broker do
    begin
      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      LocalFiles.UpdateAPILog('Call to YTQ CHOICE ADD: ' +
        ' TEXT= ' + aChoice);
      RemoteProcedure := 'YTQ CHOICE ADD';
      Param[0].Mult['"TEXT"'] := aChoice;
      Param[0].Mult['"LEGACY"'] := LegacyValue;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ CHOICE ADD: Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ CHOICE ADD: Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ CHOICE ADD: Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          result := 'ERROR: YTQ CHOICE ADD FAILED.' + #13 +
            results[1];
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ CHOICE ADD: Successful');
          aChoiceIEN := Piece(results[1], U, 1); //ChoiceIEN
          result := OK;
        end;
    end;
end;


function WriteChoiceIdentifier(aChoiceIdentifierAsString: string;
  aChoiceType: string; Broker: TRPCBroker): string;
var
  i                           : integer;
begin
  with Broker do
    begin
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;
      //// Stub ////////
      LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
        FILE_CHOICEIDENTIFIERS +
        ' choice identifiers');
      RemoteProcedure := 'YTQ STUB';
      Param[0].Mult['"FILEN"'] := FILE_CHOICEIDENTIFIERS; // 601.89
      Param[0].Mult['"IEN"'] := aChoiceType;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_CHOICEIDENTIFIERS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_CHOICEIDENTIFIERS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      //// Save ////////
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
        FILE_CHOICEIDENTIFIERS +
        ' IEN^= ' + aChoiceType);

      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_CHOICEIDENTIFIERS; // 601.89
      Param[0].Mult['"1"'] := '.01^' + aChoiceType + '^1'; // IEN
      Param[0].Mult['"2"'] := '1^' + aChoiceIdentifierAsString; // 'N', '1' or '0'
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if (Results[0] = '[ERROR]') then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          Result := 'ERROR: YTQ SAVE FAILED.' + #13 +
            results[1];
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE: Successful');
          Result := OK;
        end;
    end;

(*
  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      LocalFiles.UpdateAPILog('Call to YTQ CHOICEIDENT: ' +
        ' CT= ' + aChoiceType);
      RemoteProcedure := 'YTQ CHOICEIDENT';

      Param[0].Mult['"CT"'] := aChoiceType + '^1'; // IEN

      Param[0].Mult['"ID"'] := aChoiceIdentifierAsString; // 'N', '1' or '0'
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ CHOICEIDENT: Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ CHOICEIDENT: Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if (Results[0] = '[ERROR]') then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ CHOICEIDENT: Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          Result := 'ERROR: YTQ CHOICEIDENT FAILED.' + #13 +
            results[1];
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ CHOICEIDENT: Successful');
          Result := OK;
        end;
    end;
*)
end;


function WriteChoiceType(aChoices: TStrings; Broker: TRPCBroker;
  var aChoiceTypeIEN: string): string;
var
  i                           : integer;
  YS                          : string;
begin

  with Broker do
    begin
      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      LocalFiles.UpdateAPILog('Call to YTQ CHOICETYPE ADD');
      RemoteProcedure := 'YTQ CHOICETYPE ADD';

      for i := 0 to aChoices.Count - 1 do
        begin
          YS := '"' + IntToStr(i + 1) + '"';
          Param[0].Mult[YS] := aChoices[i];
        end;

      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ CHOICETYPE ADD: Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ CHOICETYPE ADD: Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ CHOICETYPE ADD: Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          result := 'ERROR: YTQ CHOICETYPE ADD FAILED.' + #13 +
            results[1];
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ CHOICETYPE ADD: Successful');
          aChoiceTypeIEN := Piece(results[1], U, 1); //ChoiceTypeIEN
          result := OK;
        end;
    end;
end;


function WriteMultiChoiceItems(ThisQuestion: TQuestion; Broker: TRPCBroker): string;
// Used by Import and Editor_QA.
// Format of Question.MultiChoice.Choices is text of choice only (no legacy value,
//   sequence, designator or ID).

var
  i                           : integer;
  TempCh                      : Char;
  TempIEN,
    Process,
    TempStr                   : string;
  TempList                    : TStringList;
begin
  Result := OK;

  with ThisQuestion do
    begin

      if MultiChoice.NumberOfChoices = 0 then
        Exit;

      TempList := TStringList.Create;

      // Write each multi-choice string to DB. TempIEN returns the IEN, which is
      // added to a StringList of IENs.
      for i := 0 to MultiChoice.Choices.Count - 1 do
        begin
          TempIEN := '';
          if ChoiceIdentifier = ciOneBased then
            TempStr := IntToStr(i + 1)
          else
            TempStr := IntToStr(i);

          TempCh := TempStr[1];
          // TempCh is the "LegacyValue." This is used in Clinical reminders. There
          // is a problem. The legacy value is a single character and was initially
          // designed to handle radio groups in legacy tests. The number of multi-
          // choice items was usually in the 3-5 range, however, combo box and
          // list box components can have many more items and using this scheme
          // will not produce unique characters after ten items.
          Process := WriteChoice(MultiChoice.Choices[i], TempCh, TempIEN, Broker);

          if Process <> OK then
            begin
              Result := Process;
              Break;
              Exit;
            end
          else
            TempList.Add(TempIEN);
        end; // for i

      // Save each IEN to DB in table that connects the list of Multi-Choice items
      // to the identifier for this question
      if TempList.Count > 0 then
        begin
          TempIEN := '';

          Process := WriteChoiceType(TempList, Broker, {var} TempIEN);

          if Process <> OK then
            begin
              Result := Process;
              TempList.Free;
              Exit;
            end;

          ChoiceTypeID := TempIEN;

          TempStr := 'N';
          if ChoiceIdentifier = ciZeroBased then
            TempStr := '0';
          if ChoiceIdentifier = ciOneBased then
            TempStr := '1';

          Process := WriteChoiceIdentifier(TempStr, ChoiceTypeID, Broker);
          if Process <> OK then
            begin
              Result := process;
              TempList.Free;
              Exit;
            end;

        end;
      TempList.Free;
    end; // with ThisQuestion

end;

function WriteReport(aReport, aInstrumentID, ReportIEN: string; Broker: TRPCBroker): string;
// Returns "OK" when working properly
var
  i                           : Integer;
    YS,
    ReportText                : string;
begin

  with Broker do
    begin
      //// Stub ////////
      LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
        FILE_REPORT + ' new report');
      RemoteProcedure := 'YTQ STUB';
      Param[0].Mult['"FILEN"'] := FILE_REPORT; // 601.93
      Param[0].Mult['"IEN"'] := ReportIEN;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_REPORT + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_REPORT + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

//      if results[0] = '[ERROR]' then
//        begin
//          LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//            FILE_REPORT + ': Error ');
//          for i := 0 to Results.Count - 1 do
//            LocalFiles.UpdateAPILog('   ' + Results[i]);
//          result := 'ERROR: YTQ STUB: ' + FILE_REPORT + ' FAILED. ' + #13 +
//            results[1];
//          exit;
//        end;

      //// Save ////////
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
        FILE_REPORT +
        ' IEN = ' + ReportIEN);
      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_REPORT; // 601.93

  //    Param[0].Mult['"1"'] := '.01^NEW^1';
  //    Param[0].Mult['"1"'] := '.01^' + ReportIEN + '^1';
      Param[0].Mult['"IEN"'] := ReportIEN;
      Param[0].Mult['"1"'] := '1^`' + aInstrumentID + '^1';
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
              FILE_REPORT + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
              FILE_REPORT + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
            FILE_REPORT + ': Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          result := 'ERROR: YTQ SAVE ' + FILE_REPORT + ' FAILED.' + #13 +
            results[1];
          exit;
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
            FILE_REPORT + ': Successful');
          result := OK;
        end;

      /////////////////////// SAVE WORD-PROCESSING FIELD SEPARATELY //////////////////
      ClearParameters := True;
      ClearResults := True;
      ReportText := aReport;

      LocalFiles.UpdateAPILog('Call to YTQ WP REPORT: ' +
        FILE_REPORT +
        ' IEN= ' + ReportIEN);

      RemoteProcedure := 'YTQ WP FILER';
      Param[0].Mult['"FILEN"'] := FILE_REPORT; // 601.93

      Param[0].Mult['"IEN"'] := ReportIEN;
      Param[0].Mult['"FIELD"'] := '2';

      // Wrap lines so they are 200 char or less. Do not break within a word.
      ReportText := WrapString(ReportText, 200);

      for i := 1 to NumberOfPieces(ReportText, U) - 1 do
        begin
          YS := '"' + IntToStr(i) + '"';
          Param[0].Mult[YS] := Piece(ReportText, U, i);
          //ShowMessage(Piece(ReportText, U, i));
        end;

      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ WP REPORT: Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ WP REPORT: Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try
      if Pos('ERROR', results[0]) > 0 then
        result := 'ERROR: YTQ WP FILER: ' + FILE_REPORT + ' FAILED.' + #13 +
          results[1]
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ WP REPORT: Successful');
          result := OK;
        end;
    end;
end;


function WriteScoringKey(AKey: TKey; Broker: TRPCBroker): string;
var
  i                           : integer;
begin

  with Broker, AKey do
    begin
      //// Stub ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
        FILE_SCORING_KEYS + ' scoring key');
      RemoteProcedure := 'YTQ STUB';
      Param[0].Mult['"FILEN"'] := FILE_SCORING_KEYS;
      Param[0].Mult['"IEN"'] := ScaleID;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_SCORING_KEYS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_SCORING_KEYS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

//      if results[0] = '[ERROR]' then
//        begin
//          LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//            FILE_REPORT + ': Error ');
//          for i := 0 to Results.Count - 1 do
//            LocalFiles.UpdateAPILog('   ' + Results[i]);
//          result := 'ERROR: YTQ STUB: ' + FILE_SCORING_KEYS + ' FAILED. ' + #13 +
//            results[1];
//          exit;
//        end;

      //// Save ////////
      ClearParameters := True;
      ClearResults := True;
      LocalFiles.UpdateAPILog('Call to YTQ SAVE for ' + FILE_SCORING_KEYS);

      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_SCORING_KEYS;
    //  Param[0].Mult['"6"'] := '.01^NEW^1';
      Param[0].Mult['"1"'] := '1^' + ScaleID + '^1';
      Param[0].Mult['"2"'] := '2^' + QuestionID + '^1';
      Param[0].Mult['"3"'] := '3^' + AnswerText;
      Param[0].Mult['"4"'] := '4^' + Value;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE' + FILE_SCORING_KEYS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE' + FILE_SCORING_KEYS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE' + FILE_SCORING_KEYS + ': Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          result := 'ERROR: YTQ SAVE: ' + FILE_SCORING_KEYS + ' FAILED.' + #13 +
            results[1];
          LocalFiles.UpdateAPILog('Call to YTQ SAVE' + FILE_SCORING_KEYS + ': '
            + Results[1]);
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE' + FILE_SCORING_KEYS + ': Successful');
          result := OK;
        end;

    end;
end;

function WriteScale(AScale: TScale; Broker: TRPCBroker): string;
var
  i                           : integer;
begin

  with Broker, AScale do
    begin
     //// Stub ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
        FILE_SCALES + ' scales');
      RemoteProcedure := 'YTQ STUB';
      Param[0].Mult['"FILEN"'] := FILE_SCALES; // 601.87
      Param[0].Mult['"IEN"'] := GroupIEN;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_SCALES + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_SCALES + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

//      if results[0] = '[ERROR]' then
//        begin
//          LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//            FILE_SCALES + ': Error ');
//          for i := 0 to Results.Count - 1 do
//            LocalFiles.UpdateAPILog('   ' + Results[i]);
//          result := 'ERROR: YTQ STUB: ' + FILE_SCALES + ' FAILED. ' + #13 +
//            results[1];
//          exit;
//        end;

      //// Save ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;
      LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
        FILE_SCALES +
        ' GroupIEN= ' + GroupIEN + ' ScaleName= ' + ScaleName);

      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_SCALES; // 601.87
      //Param[0].Mult['"6"'] := '.01^NEW^1';
      Param[0].Mult['"1"'] := '1^' + GroupIEN + '^1';
      Param[0].Mult['"2"'] := '2^' + IntToStr(Sequence);
      Param[0].Mult['"3"'] := '3^' + ScaleName;
      Param[0].Mult['"4"'] := '4^' + XLabel;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
              FILE_SCALES + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
              FILE_SCALES + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
            FILE_SCALES + ': Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          result := 'ERROR: YTQ SAVE: ' + FILE_SCALES + ' FAILED. ' + #13 +
            results[1];
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
            FILE_SCALES + ': Successful');
          result := OK;
        end;
    end;
end;

function WriteScaleGroup(aInstrumentID: string; AScaleGroup: TScaleGroup;
  Broker: TRPCBroker): string;
var
  i                           : integer;
begin

  with Broker, AScaleGroup do
    begin
      //// Stub ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
        FILE_SCALEGROUPS + ' scale groups');
      RemoteProcedure := 'YTQ STUB';
      Param[0].Mult['"FILEN"'] := FILE_SCALEGROUPS;
      Param[0].Mult['"IEN"'] := aInstrumentID;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_SCALEGROUPS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_SCALEGROUPS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

//      if results[0] = '[ERROR]' then
//        begin
//          LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//            FILE_SCALEGROUPS + ': Error ');
//          for i := 0 to Results.Count - 1 do
//            LocalFiles.UpdateAPILog('   ' + Results[i]);
//          result := 'ERROR: YTQ STUB: ' + FILE_SCALEGROUPS + ' FAILED. ' + #13 +
//            results[1];
//          exit;
//        end;

      //// Save ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;
      LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
        FILE_SCALEGROUPS +
        ' 1^= ' + aInstrumentID);

      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_SCALEGROUPS;
      //Param[0].Mult['"12"'] := '.01^NEW^1';
      Param[0].Mult['"1"'] := '1^' + aInstrumentID + '^1';
      Param[0].Mult['"2"'] := '2^' + GroupName;
      Param[0].Mult['"3"'] := '3^' + IntToStr(Sequence);
      Param[0].Mult['"4"'] := '4^' + OrdinateTitle;
      Param[0].Mult['"5"'] := '5^' + IntToStr(OrdinateMin);
      Param[0].Mult['"6"'] := '6^' + IntToStr(OrdinateIncrement);
      Param[0].Mult['"7"'] := '7^' + IntToStr(OrdinateMax);
      Param[0].Mult['"8"'] := '8^' + IntToStr(Grid1Ordinate);
      Param[0].Mult['"9"'] := '9^' + IntToStr(Grid2Ordinate);
      Param[0].Mult['"10"'] := '10^' + IntToStr(Grid3Ordinate);
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_SCALEGROUPS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_SCALEGROUPS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
            FILE_SCALEGROUPS + ': Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          result := 'ERROR: YTQ SAVE: ' + FILE_SCALEGROUPS + ' FAILED. ' + #13 +
            results[1];
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
            FILE_SCALEGROUPS + ': Successful');
          result := OK;
        end;
    end;
end;

(*
function GetScalesFromVista(aInstrumentName: string; Broker: TRPCBroker): string;
var
  i                           : integer;
begin
  // Results look like this.
  // Group1= 100000^AllanS 1^Allan's Sizes^1^Ego Strength^0^5^50^10^30^40~
  // Scale1= 100000^100000^1^Ego Size^ES~
  // Scale2= 100001^100000^2^Shoe Size^Sz~
  // Group2= 100005^AllanS 1^Bob^1^^0^10^100^-1^-1^0~
  // Group3= 100001^AllanS 1^Allan's Bank Statement^2^Cash^22^4^88^40^60^70~
  // Scale3= 100002^100001^1^Cash Flow~
  // Scale4= 100003^100001^2^DebT^D~
  // Group4= 100007^AllanS 1^Bob^100^^0^10^100^-1^-1^0  //First record is unique~

  with Broker do
    begin

      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ GET SCALES';
      Param[0].Mult['"CODE"'] := aInstrumentName;
      Param[0].PType := List;
      Call;

      if not (Results[0] = DATA) then
        begin
          Result := BAD_DATA;
          Exit;
        end;

      Result := '';

      //Start at 1 to skip [DATA] line
      for i := 1 to Results.Count - 1 do
        Result := Result + Results[i] + RECORD_END;

    end; //with RCPBroker

  Result := DropTrailer(Result);

end; *)

function WriteDisplay(ADisplay: TDisplay;
  const IsAnswerDisplay: BOOLEAN; Broker: TRPCBroker): string;

var
  i                           : integer;
  NewIEN                      : string;

  function AlignmentToStr(AAlignment: TAlignment): string;
  begin
    case AAlignment of
      taLeftJustify: Result := LEFT_JUSTIFY;
      taCenter: Result := CENTER_JUSTIFY;
      taRightJustify: Result := RIGHT_JUSTIFY;
    end;
  end;


  function AnswerComponentToStr(AAnswerComponent: TAnswerComponent): string;
  begin
    Result := 'NA';

    case AAnswerComponent of

    //DATA TYPE OF COMPONENT: SET OF CODES//
    //INTERNALLY-STORED CODE: RA//   WILL STAND FOR: RadioGroup//
    //INTERNALLY-STORED CODE: CK//   WILL STAND FOR: CheckList//
    //INTERNALLY-STORED CODE: CO//   WILL STAND FOR: ComboBox//
    //INTERNALLY-STORED CODE: LI//   WILL STAND FOR: List//
    //INTERNALLY-STORED CODE: ED//   WILL STAND FOR: Edit//
    //INTERNALLY-STORED CODE: MA//   WILL STAND FOR: Mask Edit//
    //INTERNALLY-STORED CODE: SP//   WILL STAND FOR: Spin Edit//
    //INTERNALLY-STORED CODE: ME//   WILL STAND FOR: Memo//
    //INTERNALLY-STORED CODE: DA//   WILL STAND FOR: Date Picker//
    //INTERNALLY-STORED CODE: TR//   WILL STAND FOR: Track Bar//
    //INTERNALLY-STORED CODE: OR//   WILL STAND FOR: ORcombo Box//
    //INTERNALLY-STORED CODE: NA//   WILL STAND FOR: Not Applicable//

      acComboBox: Result := COMBOBOX_AC;
      acCheckListBox: Result := CHECKLIST_AC;
      acListBox: Result := LISTBOX_AC;
      acRadioGroup: Result := RADIOGROUP_AC;
      acEdit: Result := EDIT_AC;
      acMaskEdit: Result := MASKEDIT_AC;
      acSpinEdit: Result := SPINEDIT_AC;
      acMemo: Result := MEMO_AC;
      acDatePicker: Result := DATEPICKER_AC;
      acTrackBar: Result := TRACKBAR_AC;
      acORCombo: Result := ORSTAFFCOMBOBOX_AC;
      acStaff: Result := ORSTAFFCOMBOBOX_AC;
      acConsult: Result := ORCONSULTCOMBOBOX_AC;
    end;
  end;

begin

  with Broker, ADisplay do //Display_OPS do
    begin
      //// Stub ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
        FILE_DISPLAY + ' display');
      RemoteProcedure := 'YTQ STUB';
      Param[0].Mult['"FILEN"'] := FILE_DISPLAY;
      Param[0].Mult['"IEN"'] := DisplayFont.Name;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_DISPLAY + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_DISPLAY + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

//      if results[0] = '[ERROR]' then
//        begin
//          LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//            FILE_DISPLAY + ': Error ');
//          for i := 0 to Results.Count - 1 do
//            LocalFiles.UpdateAPILog('   ' + Results[i]);
//          result := 'ERROR: YTQ STUB: ' + FILE_DISPLAY + ' FAILED. ' + #13 +
//            results[1];
//          exit;
//        end;

      //// Save ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;
      LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
        FILE_DISPLAY +
        ' 1^= ' + DisplayFont.Name);
      RemoteProcedure := 'YTQ SAVE';

      Param[0].Mult['"FILEN"'] := FILE_DISPLAY; // Display Properties file
      //Param[0].Mult['"12"'] := '.01^NEW^1'; // 12

      Param[0].Mult['"1"'] := '1^' + DisplayFont.Name; //  FONTNAME;
      if fsBold in DisplayFont.Style then
        Param[0].Mult['"2"'] := '2^Y'
      else
        Param[0].Mult['"2"'] := '2^N';
      if fsItalic in DisplayFont.Style then
        Param[0].Mult['"3"'] := '3^Y'
      else
        Param[0].Mult['"3"'] := '3^N';
      if fsUnderline in DisplayFont.Style then
        Param[0].Mult['"4"'] := '4^Y'
      else
        Param[0].Mult['"4"'] := '4^N';
      Param[0].Mult['"5"'] := '5^' + IntToStr(DisplayFont.Size); //'18' FONTSIZE;
      Param[0].Mult['"6"'] := '6^' + ColorToString(DisplayFont.Color); //'clRed' FONTCOLOR;
      Param[0].Mult['"7"'] := '7^' + AlignmentToStr(DisplayAlignment); //'Left' ALIGNMENT;
      Param[0].Mult['"8"'] := '8^' + IntToStr(DisplayLeft); //'0' LEFT;

      if IsAnswerDisplay then
        begin
          Param[0].Mult['"9"'] := '9^' + DisplayMask; //'NA' MASK;
          Param[0].Mult['"10"'] := '10^' + IntToStr(DisplayColumns); //'NA' COLUMNS
          Param[0].Mult['"11"'] := '11^' +
            AnswerComponentToStr(DisplayComponent); //'LI' COMPONENT;
        end;

      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_DISPLAY + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_DISPLAY + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if (Results[0] = '[ERROR]') then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
            FILE_DISPLAY + ': Error ');
          for i := 1 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          Result := 'ERROR: YTQ SAVE: ' + FILE_DISPLAY + ' FAILED.' + #13 +
            Results[1];
          exit;
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
            FILE_DISPLAY + ': Successful');
              //IEN of new record
          NewIEN := Piece(Results[1], U, 2);
          Result := NewIEN;
        end;
    end; // with
end;

function WriteIntroText(aIntro, aIntroIEN: string;
  Broker: TRPCBroker): string;
// Returns "OK^IEN" when working properly
var
  i                           : Integer;
  TempIEN,
    YS,
    IntroText                 : string;

begin
  if aIntro = '' then
    Exit;

  TempIEN := aIntroIEN;

  with Broker do
    begin
      // New introduction, get IEN
      if aIntroIEN = '' then
        begin
          //// Stub ////////
          CreateContext('YS BROKER1');
          ClearParameters := True;
          ClearResults := True;

          LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
            FILE_INTRODUCTIONS + ' introductions');
          RemoteProcedure := 'YTQ STUB';
          Param[0].Mult['"FILEN"'] := FILE_INTRODUCTIONS;
          Param[0].Mult['"IEN"'] := aIntroIEN;
          Param[0].PType := List;
          try
            Call;

          except
            on E: EBrokerError do
              begin
                LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
                  FILE_INTRODUCTIONS + ': Error ');
                LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
                exit;
              end;
            on E: EAccessViolation do
              begin
                LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
                  FILE_INTRODUCTIONS + ': Error ');
                LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
                exit;
              end;
          end; // try

//          if results[0] = '[ERROR]' then
//            begin
//              LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//                FILE_INTRODUCTIONS + ': Error ');
//              for i := 0 to Results.Count - 1 do
//                LocalFiles.UpdateAPILog('   ' + Results[i]);
//              result := 'ERROR: YTQ STUB: ' + FILE_INTRODUCTIONS + ' FAILED. ' + #13 +
//                results[1];
//              exit;
//            end;

      //// Save ////////
          ClearParameters := True;
          ClearResults := True;
          CreateContext('YS BROKER1');
          LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
            FILE_INTRODUCTIONS +
            ' new introduction');
          RemoteProcedure := 'YTQ SAVE';
          Param[0].Mult['"FILEN"'] := FILE_INTRODUCTIONS; //Intro text file
         // Param[0].Mult['"1"'] := '.01^NEW^1';

          Param[0].Mult['"1"'] := '1^' + aIntroIEN + '^1';
          Param[0].PType := List;
          try
            Call;

          except
            on E: EBrokerError do
              begin
                LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
                  FILE_INTRODUCTIONS + ': Error ');
                LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
                exit;
              end;
            on E: EAccessViolation do
              begin
                LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
                  FILE_INTRODUCTIONS + ': Error ');
                LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
                exit;
              end;
          end; // try

          if results[0] = '[ERROR]' then
            begin
              LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
                FILE_INTRODUCTIONS + ': Error ');
              for i := 0 to Results.Count - 1 do
                LocalFiles.UpdateAPILog('   ' + Results[i]);
              result := 'ERROR: YTQ SAVE ' + FILE_INTRODUCTIONS + ' FAILED.' + #13 +
                results[1];
              exit;
            end
          else
            begin
              TempIEN := piece(Broker.Results[1], U, 2);
              LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
                FILE_INTRODUCTIONS + ': Successful');
              result := OK;
            end;
        end;

      // Safety checks, don't use a "Bad" number.
      if (TempIEN = '') then
        Exit;

      i := StrToIntDef(TempIEN, -1);
      if i = -1 then
        Exit;

      /////////////////////// SAVE WORD-PROCESSING FIELD SEPARATELY //////////////////
      ClearParameters := True;
      ClearResults := True;
      IntroText := aIntro;
      LocalFiles.UpdateAPILog('Call to YTQ WP FILER: ' +
        FILE_INTRODUCTIONS +
        ' IEN= ' + TempIEN);

      //DebugString.Add(FILE_INTRODUCTIONS + ' IEN = ' + TempIEN);
      RemoteProcedure := 'YTQ WP FILER';
      Param[0].Mult['"FILEN"'] := FILE_INTRODUCTIONS; //Intro text file
      Param[0].Mult['"IEN"'] := TempIEN + '^1';
      Param[0].Mult['"FIELD"'] := '1';

      // Wrap lines so they are 200 char or less. Do not break within a word.
      IntroText := WrapString(IntroText, 200);

      for i := 1 to NumberOfPieces(IntroText, U) do
        begin
          if Piece(IntroText, U, i) <> '' then
            begin
              YS := '"' + IntToStr(i) + '"';
              Param[0].Mult[YS] := Piece(IntroText, U, i) + ' ';
            end;
        end;

      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ WP FILER ' +
              FILE_INTRODUCTIONS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ WP FILER ' +
              FILE_INTRODUCTIONS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ WP FILER ' +
            FILE_INTRODUCTIONS + ': Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          result := 'ERROR: YTQ WP FILER: ' + FILE_INTRODUCTIONS + ' FAILED.' + #13 +
            results[1];
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ WP FILER ' +
            FILE_INTRODUCTIONS + ': Successful');
          result := OK + U + TempIEN;
        end;
    end;

end;

function WriteSection(aIEN, aInstrumentID, aFirstQuestion, aTabCaption,
  aSectionCaption, aDisplayID: string; Broker: TRPCBroker): string;
var
  i                           : integer;
begin
  Result := OK;

  if aSectionCaption = '' then
    Exit;

  with Broker do
    begin
        //// Stub ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
        FILE_SECTIONS + ' sections');
      RemoteProcedure := 'YTQ STUB';
      Param[0].Mult['"FILEN"'] := FILE_SECTIONS;
      Param[0].Mult['"IEN"'] := aIEN;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_SECTIONS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_SECTIONS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

//      if results[0] = '[ERROR]' then
//        begin
//          LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//            FILE_SECTIONS + ': Error ');
//          for i := 0 to Results.Count - 1 do
//            LocalFiles.UpdateAPILog('   ' + Results[i]);
//          result := 'ERROR: YTQ STUB: ' + FILE_SECTIONS + ' FAILED. ' + #13 +
//            results[1];
//          exit;
//        end;

      //// Save ////////

      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
        FILE_SECTIONS +
        ' 1^= ' + aInstrumentID);
      RemoteProcedure := 'YTQ SAVE';

      Param[0].Mult['"FILEN"'] := FILE_SECTIONS;

      //if aIEN = '' then
      //  Param[0].Mult['"8"'] := '.01^NEW^1'
      //else
      Param[0].Mult['"IEN"'] := aIEN;
      Param[0].Mult['"1"'] := '1^' + aInstrumentID + '^1';
      Param[0].Mult['"2"'] := '2^' + aFirstQuestion + '^1';
      Param[0].Mult['"3"'] := '3^' + aTabCaption; // 50 char max
      Param[0].Mult['"4"'] := '4^' + aSectionCaption; // 50 char max
      Param[0].Mult['"6"'] := '6^' + aDisplayID + '^1';

      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_SECTIONS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_SECTIONS + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if (Results[0] = '[ERROR]') then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
            FILE_SECTIONS + ': Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          Result := 'ERROR: YTQ SAVE: ' + FILE_SECTIONS + ' FAILED.' + #13 +
            Results[1];
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
            FILE_SECTIONS + ': Successful');
          Result := OK; //When successful this is returned:
        end;
    end;
end;

function WriteRule(ARule: TRule; Broker: TRPCBroker): string;
var
  i                           : integer;
begin

  with Broker, ARule, Rule_OPS do
    begin
      if FirstQuestionID = '' then
        begin
          result := 'ERROR: YTQ SAVE: ' + FILE_RULES + ' FAILED: No ID for ' +
            'first question of rule.';
          Exit;
        end;

      //// Stub ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
        FILE_RULES + ' rules');
      RemoteProcedure := 'YTQ STUB';
      Param[0].Mult['"FILEN"'] := FILE_RULES;  // 601.82
      Param[0].Mult['"IEN"'] := ID;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_RULES + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_RULES + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

//      if results[0] = '[ERROR]' then
//        begin
//          LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//            FILE_RULES + ': Error ');
//          for i := 0 to Results.Count - 1 do
//            LocalFiles.UpdateAPILog('   ' + Results[i]);
//          result := 'ERROR: YTQ STUB: ' + FILE_RULES + ' FAILED. ' + #13 +
//            results[1];
//          exit;
//        end;

      //// Save ////////
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;
      LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
        FILE_RULES +
        ' 1^= ' + FirstQuestionID);

      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_RULES; // 601.82
      Param[0].Mult['"IEN"'] := ID + '^1';
      Param[0].Mult['"1"'] := '1^' + FirstQuestionID + '^1';
      Param[0].Mult['"2"'] := '2^' + FirstValue;
      Param[0].Mult['"3"'] := '3^'
        + DataTypeToString(FirstValueDataType);
      Param[0].Mult['"4"'] := '4^' + QuestionOperatorToStr(ARule, True);
      Param[0].Mult['"5"'] := '5^' + BoolOperatorToString(ARule);

      if SecondQuestionID <> '' then
        begin
          Param[0].Mult['"6"'] := '6^' + SecondQuestionID + '^1';

          Param[0].Mult['"7"'] := '7^' + SecondValue;
          Param[0].Mult['"8"'] := '8^'
            + DataTypeToString(SecondValueDataType);
          Param[0].Mult['"9"'] := '9^' + QuestionOperatorToStr(ARule, False);
        end;

      Param[0].Mult['"10"'] := '10^' + MessageText;
      //Param[0].Mult['"11"'] := '11^' +
      //  BoolToMStr(Question.Rule.IsConsistencyCheck);

      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_RULES + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_RULES + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
            FILE_RULES + ': Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          result := 'ERROR: YTQ SAVE: ' + FILE_RULES + ' FAILED ' + results[1];
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
            FILE_RULES + ': Successful');
          result := OK;// + U + Piece(Results[1], U, 2);
      // If the rule.ID existed, then it is not passed here, only new ID's.
        end;
    end;
end;

function WriteRuleInstrument(RuleID, QuestionID, InstrumentID: string;
  Broker: TRPCBroker): string;
var
  i                           : integer;
begin

  if RuleID = '' then
    begin
      Result := 'ERROR: WriteRuleInstrument failed because '
        + 'Rule.ID is null.';
      Exit;
    end;

  if QuestionID = '' then
    begin
      Result := 'ERROR: WriteRuleInstrument failed because '
        + 'QuestionID is null.';
      Exit;
    end;

  with Broker do
    begin
      //// Stub ////////
      (*
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
        FILE_INSTRUMENTRULES + ' instrument-rules');
      RemoteProcedure := 'YTQ STUB';
      Param[0].Mult['"FILEN"'] := FILE_INSTRUMENTRULES; //601.83
      Param[0].Mult['"IEN"'] := InstrumentID;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_INSTRUMENTRULES + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
              FILE_INSTRUMENTRULES + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

//      if results[0] = '[ERROR]' then
//        begin
//          LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//            FILE_INSTRUMENTRULES + ': Error ');
//          for i := 0 to Results.Count - 1 do
//            LocalFiles.UpdateAPILog('   ' + Results[i]);
//          result := 'ERROR: YTQ STUB: ' + FILE_INSTRUMENTRULES + ' FAILED. ' + #13 +
//            results[1];
//          exit;
//        end;

      //// Save ////////
      *)
      CreateContext('YS BROKER1');
      ClearParameters := True;
      ClearResults := True;

      LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
        FILE_INSTRUMENTRULES +
        ' 1^= ' + InstrumentID);
      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_INSTRUMENTRULES; //601.83
      Param[0].Mult['"4"'] := '.01^NEW^1';
      Param[0].Mult['"1"'] := '1^`' + InstrumentID + '^1';
      Param[0].Mult['"2"'] := '2^`' + QuestionID + '^1';
      Param[0].Mult['"3"'] := '3^`' + RuleID + '^1';

      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_INSTRUMENTRULES + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
              FILE_INSTRUMENTRULES + ': Error ');
            LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[ERROR]' then
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
            FILE_INSTRUMENTRULES + ': Error ');
          for i := 0 to Results.Count - 1 do
            LocalFiles.UpdateAPILog('   ' + Results[i]);
          result := 'ERROR: YTQ SAVE: ' + FILE_INSTRUMENTRULES + ' FAILED ' +
            results[1];
        end
      else
        begin
          LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
            FILE_INSTRUMENTRULES + ': Successful');
          result := OK;
        end;
    end;
end;

function WriteSkippedQuestions(InstrumentID, RuleID, ASkippedQuestionIDs: string;
  Broker: TRPCBroker): string;
// ASkippedQuestionIDs is a simple string with IDs, delimited by ^.
var
  i, j, k                        : Integer;
  TempStr,
    TempSkipStr               : string;
begin

  TempSkipStr := ASkippedQuestionIDs;
  Result := OK;

  // Handle the spurious input
  if (RuleID = '') or (TempSkipStr = '') then
    Exit;

  for i := 1 to NumberOfPieces(TempSkipStr, U) do
    begin
      TempStr := Piece(TempSkipStr, U, i);
      if TempStr <> '' then
        begin
          with Broker do
            begin
            (*
              //// Stub ////////
              CreateContext('YS BROKER1');
              ClearParameters := True;
              ClearResults := True;

              LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
                FILE_SKIPPED_QUESTIONS + ' skip questions');
              RemoteProcedure := 'YTQ STUB';
              Param[0].Mult['"FILEN"'] := FILE_SKIPPED_QUESTIONS; // 601.79
              Param[0].Mult['"IEN"'] := InstrumentID;
              Param[0].PType := List;
              try
                Call;

              except
                on E: EBrokerError do
                  begin
                    LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
                      FILE_SKIPPED_QUESTIONS + ': Error ');
                    LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
                    exit;
                  end;
                on E: EAccessViolation do
                  begin
                    LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
                      FILE_SKIPPED_QUESTIONS + ': Error ');
                    LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
                    exit;
                  end;
              end; // try

//              if results[0] = '[ERROR]' then
//                begin
//                  LocalFiles.UpdateAPILog('Call to YTQ STUB ' +
//                    FILE_SKIPPED_QUESTIONS + ': Error ');
//                  for k := 0 to Results.Count - 1 do
//                    LocalFiles.UpdateAPILog('   ' + Results[k]);
//                  result := 'ERROR: YTQ STUB: ' + FILE_SKIPPED_QUESTIONS + ' FAILED. ' + #13 +
//                    results[1];
//                  exit;
//                end;

              //// Save ////////
              *)
              CreateContext('YS BROKER1');
              ClearParameters := True;
              ClearResults := True;

              LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
                FILE_SKIPPED_QUESTIONS +
                ' 1^= ' + InstrumentID);
              RemoteProcedure := 'YTQ SAVE';
              Param[0].Mult['"FILEN"'] := FILE_SKIPPED_QUESTIONS; // 601.79
              Param[0].Mult['"4"'] := '.01^NEW^1';
              Param[0].Mult['"1"'] := '1^' + InstrumentID + '^1';
              Param[0].Mult['"2"'] := '2^' + RuleID + '^1';
              Param[0].Mult['"3"'] := '3^' + TempStr + '^1';

              Param[0].PType := List;

              try
                Call;

              except
                on E: EBrokerError do
                  begin
                    LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
                      FILE_SKIPPED_QUESTIONS + ': Error ');
                    LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
                    exit;
                  end;
                on E: EAccessViolation do
                  begin
                    LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
                      FILE_SKIPPED_QUESTIONS + ': Error ');
                    LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
                    exit;
                  end;
              end; // try

              if results[0] = '[ERROR]' then
                begin
                  LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
                    FILE_SKIPPED_QUESTIONS + ': Error ');
                  for j := 0 to Results.Count - 1 do
                    LocalFiles.UpdateAPILog('   ' + Results[j]);
                  result := 'ERROR: YTQ SAVE: ' + FILE_SKIPPED_QUESTIONS + ' FAILED ' +
                    results[1];
                end
              else
                begin
                  LocalFiles.UpdateAPILog('Call to YTQ SAVE ' +
                    FILE_SKIPPED_QUESTIONS + ': Successful');
                  result := OK;
                end;
            end;
        end;
    end;
end;

end.

