unit uMHA_LocalFiles;

interface

{$WARN UNIT_PLATFORM OFF}

uses
  classes,
  Windows,
  FileCtrl,
  fIA_dialogs, Dialogs,
  ShellAPI,
  SysUtils,
  jclSysInfo;


type
  TLocalFiles = class
  private
    FAnswerFilesPath,
      FInstrumentFilesPath,
      FTempFolderName,
      FASI_RecoveryFile,
      FOffLinePatientFile,
      FMHAPath: string; //va_SecureDesktop

    procedure CheckLocalDirectories;

  protected
  public
    constructor Create;

    function BuildAdministrationFileName(aDFN,
      aDateTimeGiven, aInstrumentName: string): string;
    function InstrumentResourceFileName(AInstrumentName,
      ALastEditFMDate: string): string;
    function OffLineAdministrationFilesExist: Boolean;
    function AnswerFileName(aDFN, aDUZ: string): string;
    function CrashFileName(aDFN, aDUZ, aInstrName: string): string;

    function CrashFileExists(aDFN, aDUZ: string): Boolean;
    // patch 105 removed reference, function not used llh
    //function MHA_AUX_DLL_Exists: string;
    function Check_DLL_Version: boolean;
    function PC_DLL_Version: string;
    procedure CleanupCrashFiles(aDFN: string);
    procedure CleanupDatedOfflineFiles(aName: string);
    procedure RemoveOffLineFilesForPt(aDFN: string);

    property MHAPath: string read FMHAPath write FMHAPath; //va_SecureDesktop
    property TempFolderName: string read FTempFolderName write FTempFolderName;
    property AnswerFilesPath: string read FAnswerFilesPath write FAnswerFilesPath;
    property InstrumentFilesPath: string read FInstrumentFilesPath write FInstrumentFilesPath;
    property ASI_RecoveryFile: string read FASI_RecoveryFile write FASI_RecoveryFile;
    property OffLinePatientFile: string read FOffLinePatientFile write FOffLinePatientFile;


    //* procedure UpdateAPILog(MessageIn: string);

  //published
  end; { TLocalFiles }

var
  LocalFiles                  : TLocalFiles;

implementation


uses
  uIA_Const,
  uMHA_Const,
  uIA_Utils,
  uMHA_DM,
  uMHA_Administration_OPS;


constructor TLocalFiles.Create;
begin
  inherited Create;
  CheckLocalDirectories;
end;

procedure TLocalFiles.CheckLocalDirectories;

  procedure CheckThis(ADir: string);
  begin
    if not DirectoryExists(ADir) then
      if not CreateDir(ADir) then
        raise Exception.Create('Cannot create ' + ADir);
  end;

begin
  FTempFolderName := GetAppDataFolder + '\MHA3\';
  CheckThis(FTempFolderName);

  FInstrumentFilesPath := TempFolderName + 'Instrument Files';
  CheckThis(FInstrumentFilesPath);

  FAnswerFilesPath := TempFolderName + 'Answer Files';
  CheckThis(FAnswerFilesPath);

  FASI_RecoveryFile := FAnswerFilesPath + '\Crash.txt';
  FOffLinePatientFile := FInstrumentFilesPath + '\B457-T12.dll';

  FMHAPath := ExtractFilePath(ParamStr(0));
end;

//* procedure TLocalFiles.UpdateAPILog(MessageIn: string);
//* begin
//*   gvAPICalls.Add(MessageIn);
//*   gvAPICalls.SaveToFile('MHA3_Debug.log');
//* end;

function TLocalFiles.AnswerFileName(aDFN, aDUZ: string): string;
// Patient.DFN and Administration ID used to construct name
begin
  Result := FAnswerFilesPath + '\' + aDFN + FILE_NAME_DELIMITER
    + aDUZ + '.' + ANSWER_FILE_EXT;
end;

function TLocalFiles.CrashFileName(aDFN, aDUZ, aInstrName: string): string;
begin
  Result := FAnswerFilesPath + '\' +
    aDFN + FILE_NAME_DELIMITER +
    aDUZ + FILE_NAME_DELIMITER +
    aInstrName + '.' +
    CRASH_FILE_EXT;
end;


function TLocalFiles.BuildAdministrationFileName(ADFN,
  ADateTimeGiven, AInstrumentName: string): string;
begin
  Result := FAnswerFilesPath + '\' + ADFN + FILE_NAME_DELIMITER
    + ADateTimeGiven + FILE_NAME_DELIMITER
    + AInstrumentName + '.' + ADMIN_FILE_EXT;
end;

function TLocalFiles.InstrumentResourceFileName(AInstrumentName,
  ALastEditFMDate: string): string;
var
  tmpDateStr                  : string;
begin
  if not IsFMDateTime(ALastEditFMDate) then
    tmpDateStr := '2000101' //the FMDate for 01/01/1900
  else
    tmpDateStr := Piece(ALastEditFMDate, '.', 1)
      + Piece(ALastEditFMDate, '.', 2);

  Result := FInstrumentFilesPath + '\' + AInstrumentName +
    FILE_NAME_DELIMITER + tmpDateStr + '.' + RESOURCE_FILE_EXT;

end;

//function TLocalFiles.InstrumentFrmFileName(AInstrumentName: string;
//  ALastEditFMDate: string): string;
//begin
//  Result := InstrumentFileNameStem(AInstrumentName, ALastEditFMDate) + '.frm';
//end;

//function TLocalFiles.InstrumentFileNameStem(AInstrumentName,
//  ALastEditFMDate: string): string;
//var
//  Dir, tmpDateStr             : string;
//begin
//
//  Dir := FInstrumentFilesPath + '\' + AInstrumentName;
//
//  if not IsFMDateTime(ALastEditFMDate) then
//    tmpDateStr := '2000101' //the FMDate for 01/01/1900
//  else
//    tmpDateStr := Piece(ALastEditFMDate, '.', 1)
//      + Piece(ALastEditFMDate, '.', 2);
//
//  Result := Dir + FILE_NAME_DELIMITER + tmpDateStr;
//end;


function TLocalFiles.CrashFileExists(aDFN, aDUZ: string): Boolean;
var
  sr                          : TSearchRec;
  Instrname                   : string;
begin
  Result := False;

  if (FindFirst(LocalFiles.AnswerFilesPath + '\*.' +
    CRASH_FILE_EXT, faAnyFile, sr) = 0) then
    begin
      if (Pos(aDFN + FILE_NAME_DELIMITER, sr.Name) > 0) and
        (Pos(FILE_NAME_DELIMITER + aDUZ, sr.Name) > 0) then
        begin
          Result := True;
          InstrName := Piece(sr.Name, FILE_NAME_DELIMITER, 3);

          InstrName := Copy(InstrName, 1, Length(InstrName) - 4);
          if Administration_OPS.ReadCrashFile(aDFN, aDUZ, InstrName) then
            SysUtils.DeleteFile(LocalFiles.AnswerFilesPath + '\' + sr.Name);
        end;

      while (FindNext(sr) = 0) do
        begin
          if (Pos(aDFN + FILE_NAME_DELIMITER, sr.Name) > 0) and
            (Pos(FILE_NAME_DELIMITER + aDUZ, sr.Name) > 0) then
            begin
              InstrName := Piece(sr.Name, FILE_NAME_DELIMITER, 3);
              InstrName := Copy(InstrName, 1, Length(InstrName) - 4);
              if Administration_OPS.ReadCrashFile(aDFN, aDUZ, InstrName) then
                SysUtils.DeleteFile(LocalFiles.AnswerFilesPath + '\' + sr.Name);
            end;
        end;
    end;

  SysUtils.FindClose(sr);
end;

//patch 105 removed, not needed, use gvEnvVar instead llh
{
function TLocalFiles.MHA_AUX_DLL_Exists: string;
begin
  Result := 'Not Found';

   // Windows XP
  if FileExists('C:\Program Files\Vista\Common Files\YS_MHA_AUX.dll') then
    begin
      result := 'Win XP';
      exit;
    end;

   // Windows 7
  if FileExists('C:\Program Files (x86)\Vista\Common Files\YS_MHA_AUX.dll') then
    begin
      result := 'Win 7';
      exit;
    end;
end;
}

function TLocalFiles.PC_DLL_Version: string;
//var
  //TempVersion                 : string;
begin
  // patch 105 - modified to use gvEnvVar variable to find YS_MHA_AUX.dll llh
  //result := DataModule1.ClientVersion(gvEnvVar + '\Vista\Common Files\YS_MHA_AUX.dll');

  //patch 129 - fix for network drive being used for the AUX dll
  result := DataModule1.ClientVersion(gvAuxFile);

  if result = '' then result := 'Not Found' ;

  {TempVersion := MHA_AUX_DLL_Exists;

  if TempVersion = 'Not Found' then
    begin
      result := 'Not Found';
      exit;
    end;

  if TempVersion = 'Win XP' then
    result := DataModule1.ClientVersion('C:\Program Files\Vista\Common Files\YS_MHA_AUX.dll');

  if TempVersion = 'Win 7' then
    result := DataModule1.ClientVersion('C:\Program Files (x86)\Vista\Common Files\YS_MHA_AUX.dll');
  }
end;

function TLocalFiles.Check_DLL_Version: boolean;
// Ensure YS_MHA_AUX.dll is present and current.
var
  PCVersion                 : string;
  i,
    RequiredVersionInt,
    PCVersionInt              : integer;

begin
  result := false;
  // YS_MHA_AUX.dll version on this PC
  PCVersion := LocalFiles.PC_DLL_Version;

  if PCVersion = 'Not Found' then
    begin
    {
      ShowMessage('An important file cannot be found. ' + #13#13 +
        'YS_MHA_AUX.dll is used by MHA to handle complex instruments and ' +
        'it is not located in C:\Program Files (x86)\Vista\Common Files. ' + #13#13 +
        'Please contact IRM to have this file installed. MHA will not ' +
        'continue.');
     }
      // patch 105 modified to use gvEnvVar llh
      ShowMessage('An important file cannot be found. YS_MHA_AUX.dll is used by' + #13 +
                  'MHA to handle complex instruments and it is NOT located in ' + #13 +
                  gvEnvVar + '\Vista\Common Files. Please contact IRM'  + #13 +
                  'to have this file installed. MHA will not continue.');
      exit;
    end;

  // gvDLL_Version was set when the resource files were saved to the PC.

  for i := 1 to 4 do
    begin
      RequiredVersionInt := StrToIntDef(Piece(gvDLL_Version, '.', i), 0);
      PCVersionInt := StrToIntDef(Piece(PCVersion, '.', i), 0);

      if RequiredVersionInt > PCVersionInt then
        begin
          ShowMessage('An important file needs to be updated. ' + #13#13 +
            'YS_MHA_AUX.dll is used by MHA to handle complex instruments and ' +
            //patch 105, modified line below to use gvEnvVar llh
            //'it is located in C:\Program Files (x86)\Vista\Common Files. ' + #13#13 +
            'it is located in ' + gvEnvVar + '\Vista\Common Files. ' + #13#13 +
            'The desired version of YS_MHA_AUX.dll is: ' + gvDLL_Version + #13 +
            'Version in use: ' + PCVersion + #13#13 +
            'Please contact IRM to have this file updated. MHA will not ' +
            'continue.');
          exit;
        end;
    end;
    
  result := true;
end;

procedure TLocalFiles.CleanupDatedOfflineFiles(aName: string);
// This is a file for an incomplete instrument administered off-line that has
// expired.
begin
  if FileExists(aName) then
    SysUtils.DeleteFile(aName);
end;

procedure TLocalFiles.RemoveOffLineFilesForPt(aDFN: string);
// Remove off-line administration & answer files.
// File's name: 2~3070913-105401~AUDC.adm & 2~X16.ans
var
  sr                          : TSearchRec;
begin

  if (FindFirst(LocalFiles.AnswerFilesPath + '\*.' +
    ADMIN_FILE_EXT, faAnyFile, sr) = 0) then
    if Pos(aDFN + FILE_NAME_DELIMITER, sr.Name) > 0 then
      SysUtils.DeleteFile(LocalFiles.AnswerFilesPath + '\' + sr.name);

  while (FindNext(sr) = 0) do
    if Pos(aDFN + FILE_NAME_DELIMITER, sr.Name) > 0 then
      SysUtils.DeleteFile(LocalFiles.AnswerFilesPath + '\' + sr.name);

  SysUtils.FindClose(sr);

  if (FindFirst(LocalFiles.AnswerFilesPath + '\*.' +
    ANSWER_FILE_EXT, faAnyFile, sr) = 0) then
    if Pos(aDFN + FILE_NAME_DELIMITER, sr.Name) > 0 then
      SysUtils.DeleteFile(LocalFiles.AnswerFilesPath + '\' + sr.name);

  while (FindNext(sr) = 0) do
    if Pos(aDFN + FILE_NAME_DELIMITER, sr.Name) > 0 then
      SysUtils.DeleteFile(LocalFiles.AnswerFilesPath + '\' + sr.name);

  SysUtils.FindClose(sr);
end;


procedure TLocalFiles.CleanupCrashFiles(aDFN: string);
var
  sr                          : TSearchRec;
  TempStr                     : string;
begin
  TempStr := aDFN + FILE_NAME_DELIMITER;

  if (SysUtils.FindFirst(LocalFiles.AnswerFilesPath + '\*.' +
    CRASH_FILE_EXT, faAnyFile, sr) = 0) then
    if Pos(TempStr, sr.Name) > 0 then
      SysUtils.DeleteFile(LocalFiles.AnswerFilesPath + '\' + sr.Name);

  while (SysUtils.FindNext(sr) = 0) do
    if Pos(TempStr, sr.Name) > 0 then
      SysUtils.DeleteFile(LocalFiles.AnswerFilesPath + '\' + sr.name);

  SysUtils.FindClose(sr);
end;


function TLocalFiles.OffLineAdministrationFilesExist: BOOLEAN;
var
  sr                          : TSearchRec;
begin
  Result := (SysUtils.FindFirst(FAnswerFilesPath + '\' + '*.' + ADMIN_FILE_EXT,
    faAnyFile, sr) = 0);
  SysUtils.FindClose(sr);
end;





initialization
  LocalFiles := TLocalFiles.Create;

finalization
  LocalFiles.Free;

end.

