unit uMHA_SpecialResults_OPS;

interface

uses
  Contnrs,
  Classes,
  Windows,
  Messages,
  Controls,
  StdCtrls,
  SysUtils;

type
  TSpecialResults_OPS = class
  private
    // All potential instruments that can be on the Health Summary Form
    FUsersListOfSpecialInstruments: TObjectList;
  protected
  public
    constructor Create;
    destructor Destroy; override;

    property UsersListOfSpecialInstruments: TObjectList read FUsersListOfSpecialInstruments
      write FUsersListOfSpecialInstruments;

    function LoadSpecialResultsInstruments: string;
    function LoadUsersSpecialResultsInstruments: Boolean;
    //procedure LoadQuestionsForSpecialResults(aInstrumentName: string);

  end; // TSpecialResults_OPS

var
  SpecialResults_OPS          : TSpecialResults_OPS;


implementation

uses
  uMHA_Const,
  uIA_Const,
  uIA_Utils,
  fIA_dialogs, Dialogs,
  uMHA_Report_OPS,
  uMHA_Report_IO,
  uMHA_DM,
  uMHA_SpecialResults;


constructor TSpecialResults_OPS.Create;
begin
  inherited Create;
  FUsersListOfSpecialInstruments := TObjectList.Create;
end;


destructor TSpecialResults_OPS.Destroy;
begin
  FUsersListOfSpecialInstruments.Free;
  inherited Destroy;
end;


function TSpecialResults_OPS.LoadUsersSpecialResultsInstruments: Boolean;
// This loads the user's picks for the health summary (gvSpecialResultsList).
// Name^Height^Min^Max^SeriesCount^ShowQA^Guide Caption^ColorBand
// ColorBand is formated:
//    Max-Min-Color*Max-Min-Color*...
//
//'BAI^200^0^63^0^ Guide for interpreting BAI scores (range 0-63):' + #13 + #13 +
//    '0-9: Minimal level of anxiety. 10-18: Mild anxiety. 19-29: Moderate anxiety. 30-63: Severe anxiety.' +
//    '^63-30-clRed*29-19-clMaroon*18-10-clPurple*9-0-clGreen*~'
var
  i, j, k, x                     : Integer;
  TempHS                      : TSpecialResultInstrument;
  DLLHandle                   : THandle;
  Inst_Name,
    TempStr,
    TempStr1,
    TempStr2,
    ResultString              : string;
begin
  result := False;

  if gvSpecialResultsList = '' then
    Exit;

  FUsersListOfSpecialInstruments.Clear;
  // if MsgDlgFrm('Skip this one?', '',
  //        mtWarning, [mbYes, mbNo], 0) = mrYes then exit;

  // patch 105 added gvAuxFile which = proper C:\Program Files dir for the YS_MHA_AUX file llh
  dllHandle := LoadLibrary(PChar(gvAuxFile));

  if dllHandle = 0 then
  begin
    // patch 105 added gvAuxFile, removed C:\Program Files llh
    MsgDlgFrm('Could not find DLL used for special results.' +
      'The needed file, YS_MHA_AUX.DLL, is usually located in ' + gvAuxFile , '',
      mtWarning, [mbOk], 0);
    Exit;
  end
  else
    begin
      try
        @SpecialResultsDLL := GetProcAddress(dllHandle, 'SpecialResults');

        if @SpecialResultsDLL = nil then
          // function not found
          Exit;

        if Assigned(SpecialResultsDLL) then
          SpecialResultsDLL(ResultString);

        if ResultString = '' then
          Exit;

        // 'BAI^G~PHQ9^B~PCLM^G~SF36^G~'
        for i := 1 to NumberOfPieces(gvSpecialResultsList, RECORD_END) - 1 do
          begin
            Inst_Name := Piece(gvSpecialResultsList, RECORD_END, i);

            // ResultString is the data on all special instruments
            for j := 1 to NumberOfPieces(ResultString, RECORD_END) - 1 do
              begin
                TempStr := Piece(ResultString, RECORD_END, j);

                if Piece(Inst_Name, U, 1) = Piece(TempStr, U, 1) then
                  begin

                    TempHS := TSpecialResultInstrument.Create;
                 // Name^Height^Min^Max^SeriesCount^ShowQA^Guide Caption^ColorBand
                 // ColorBand is formated:
                 //    Max-Min-color*Max-Min-Color*...
                    with TempHS do
                      begin
                        hsTitle := Piece(TempStr, U, 1);
                        hsHeight := StrToIntDef(Piece(TempStr, U, 2), 200);
                        hsMin := StrToIntDef(Piece(TempStr, U, 3), 0);
                        hsMax := StrToIntDef(Piece(TempStr, U, 4), 10);
                        hsSeriesCount := StrToIntDef(Piece(TempStr, U, 5), 0);
                        hsGuide := Piece(TempStr, U, 6);
                        TempStr1 := Piece(TempStr, U, 7);

                        if Piece(Inst_Name, U, 2) = 'B' then
                          hsShowQA := True
                        else
                          hsShowQA := False;

                        x := NumberOfPieces(TempStr1, '*');

                        if x > 0 then
                          begin
                            for k := 1 to x - 1 do
                              begin
                                TempStr2 := '';
                                TempStr2 := Piece(TempStr1, '*', k);
                                ListOfColorBands.Add(TempStr2);
                              end;
                          end;
                      end; // with TempHS

                    FUsersListOfSpecialInstruments.Add(TempHS);
                  end; // if on user's list
              end; // for i
          end;
      finally
        FreeLibrary(dllHandle);
      end; {try..finally}

      result := True;
    end;

end;

function TSpecialResults_OPS.LoadSpecialResultsInstruments: string;
// This loads the instruments suitable for the Special Results.
// Name^Height^Min^Max^SeriesCount^ShowQA^Guide Caption^ColorBand
// ColorBand is formated:
//    Max-Min-color*Max-Min-Color*...
var
  i                           : Integer;
  DLLHandle                   : THandle;
  TempStr,
    ResultString              : string;
begin
  result := '';

  // patch 105 added gvAuxFile which = proper C:\Program Files dir for the YS_MHA_AUX file llh
  dllHandle := LoadLibrary(PChar(gvAuxFile));

  if dllHandle = 0 then
  begin
    // patch 105 added gvAuxFile, removed C:\Program Files llh
    MsgDlgFrm('Could not find DLL used for scoring instruments.' +
      'The file YS_MHA_AUX.DLL is usually located in ' + gvAuxFile , '',
      mtWarning, [mbOk], 0);
    Exit;
  end
  else
    begin
      try
        @SpecialResultsDLL := GetProcAddress(dllHandle, 'SpecialResults');

        if @SpecialResultsDLL = nil then
          // function not found
          Exit;

        if Assigned(SpecialResultsDLL) then
          SpecialResultsDLL(ResultString);

        if ResultString = '' then
          Exit;

        for i := 1 to NumberOfPieces(ResultString, RECORD_END) - 1 do
          begin
            TempStr := Piece(ResultString, RECORD_END, i);
            Result := Result + Piece(TempStr, U, 1) + U;
          end; // for i
      finally
        FreeLibrary(dllHandle);
      end; {try..finally}
    end;
end;


initialization
  SpecialResults_OPS := TSpecialResults_OPS.Create;

finalization
  SpecialResults_OPS.Free;

end.

