unit uMHA_AUX_DERS;

// Introduced in patch 108 - llh

interface

uses
  SysUtils, Math;

function ScoreAnswers_DERS(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_DERS(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  //patch 129 removed fIA_dialogs,
  //fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The following questions are scored in reverse: 1, 2, 6, 7, 8, 10, 17,20, 22, 24, 34
  (1=5)  (2=4)  (3=3)  (4=2)  (5=1)

  There are 6 scales with this instrument:
    'Non-acceptance of emotional responses'
    'Difficulties engaging in goal-directed behavior'
    'Impulse control difficulties'
    'Limited access to emotion regulation strategies'
    'Lack of emotional awareness'
    'Lack of emotional clarity'

   And both the raw and mean scores are computed for each scale.

*}

function AddSpace(StringIn: string; Position: integer): string;
begin
  result := result + StringIn;
  while Length(result) < Position do
    result := result + ' ';
end;

function PadSpace(StringIn: string; Position: integer): string;
begin
  result := result + StringIn;
  while Length(result) < Position do
    result := ' ' + result ;
end;


function ScoreAnswers_DERS(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  sc1, sc2, sc3,
    sc4, sc5, sc6,
    scm1, scm2, scm3,
    scm4, scm5, scm6,
    total                     : double ;

  i,Count,TempAnswer          : Integer;

  TempResult                  : string;
begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll

  if AnswerString = 'DERS' then
    begin
      Result := 'YES';
      exit;
    end;

  sc1 := 0 ;
  sc2 := 0 ;
  sc3 := 0 ;
  sc4 := 0 ;
  sc5 := 0 ;
  sc6 := 0 ;

  scm1 := 0 ;
  scm2 := 0 ;
  scm3 := 0 ;
  scm4 := 0 ;
  scm5 := 0 ;
  scm6 := 0 ;

  total := 0 ;

  Result := '' ;
  Count := NumberOfPieces(AnswerString, RECORD_END) ;

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := StrToIntDef(Piece(Piece(TempResult, U, 1), EQ, 2), -1);

      // if question has been skipped, can't add it to the score, so get next question response
      if (TempAnswer = 1155) then   // do nothing
      else
      begin
        // itemindex 0 = 1 so need to add 1 to TempAnswer
        if ( TempAnswer > -1 ) then
          TempAnswer := TempAnswer + 1 ;

        // reverse the scores for these questions
        if ( (i = 1) or (i = 2) or (i = 6) or (i = 7) or (i = 8) or
             (i = 10) or (i = 17) or (i = 20) or (i = 22) or (i = 24) or (i = 34) ) then
          case tempAnswer of
            1: tempAnswer := 5 ;
            2: tempAnswer := 4 ;
            3: tempAnswer := 3 ;
            4: tempAnswer := 2 ;
            5: tempAnswer := 1 ;
           else
             tempAnswer := 0 ;
           end;

        //showmessage('i = ' + inttostr(i) + #13 + 'rawScore = ' + inttostr(TempAnswer) +
        //            #13 + 'revScore = ' + inttostr(TempAnswer)) ;

        if ( (i = 11) or (i = 12) or (i = 21) or (i = 23) or (i = 25) or (i = 29)) then
          sc1 := sc1 + TempAnswer ;

        if ( (i = 13) or (i = 18) or (i = 20) or (i = 26) or (i = 33) ) then
          sc2 := sc2 + TempAnswer ;

        if ( (i = 3) or (i = 14) or (i = 19) or (i = 24) or (i = 27) or (i = 32)) then
          sc3 := sc3 + TempAnswer ;

        if ( (i = 15) or (i = 16) or (i = 22) or (i = 28) or (i = 30) or (i = 31) or (i = 35) or
             ( i = 36) ) then
          sc4 := sc4 + TempAnswer ;

        if ( (i = 2) or (i = 6) or (i = 8) or (i = 10) or (i = 17) or (i = 34)) then
          sc5 := sc5 + TempAnswer ;

        if ( (i = 1) or (i = 4) or (i = 5) or (i = 7) or (i = 9) ) then
          sc6 := sc6 + TempAnswer ;
      end;
    end ;

  // get the mean (average)
  if sc1 > 0 then scm1 := RoundTo(sc1/6, -2) ;
  if sc2 > 0 then scm2 := RoundTo(sc2/5, -2) ;
  if sc3 > 0 then scm3 := RoundTo(sc3/6, -2) ;
  if sc4 > 0 then scm4 := RoundTo(sc4/8, -2) ;
  if sc5 > 0 then scm5 := RoundTo(sc5/6, -2) ;
  if sc6 > 0 then scm6 := RoundTo(sc6/5, -2) ;

  total := sc1 + sc2 + sc3 + sc4 + sc5 + sc6 ;

  // Note: "Non-acceptance..." is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~
  Result :=
    'DERS Total Score^^' + floattoStr(total) +
                     '                           Raw Score   Mean Score' + RECORD_END +
    '  Non-acceptance of emotional responses^^' + floattostr(sc1) + U + floattostr(scm1) + RECORD_END +
    '  Difficulties engaging in goal-directed behavior^^' + floattoStr(sc2) + U + floattostr(scm2) + RECORD_END +
    '  Impulse control difficulties^^' + floattostr(sc3) + U + floattostr(scm3) + RECORD_END +
    '  Limited access to emotion regulation strategies^^' + floattostr(sc4) + U + floattostr(scm4) + RECORD_END +
    '  Lack of emotional awareness^^' + floattoStr(sc5) + U + floattostr(scm5) + RECORD_END +
    '  Lack of emotional clarity^^' + floattostr(sc6) + U + floattostr(scm6) + RECORD_END ;
end;


function ProgressNote_DERS(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr1, TempStr2,
    TempStr3, padScale,
    padScore                  : string;
begin
  Result := '';
  padScale := '' ;
  padScore := '' ;

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  for i := 1 to NumberOfPieces(ScoreString, RECORD_END) - 1 do
    begin
      TempStr1 := Piece(Piece(ScoreString, RECORD_END, i), U, 1) + ':' ;
      TempStr2 := Piece(Piece(ScoreString, RECORD_END, i), U, 3) ;
      TempStr3 := Piece(Piece(ScoreString, RECORD_END, i), U, 4) ;
      padScale := AddSpace(TempStr1,51) ;
      padScore := PadSpace(TempStr2,2) ;

      if i = 1 then
        Result := Result + TempStr1 + '  ' + TempStr2 + #13#13
      else
        Result := Result + Indent + padScale + padScore + '           ' + TempStr3 + #13 ;
      padScale := '' ;
      padScore := '' ;
    end;

end;

end.
