unit uMHA_AUX_MDQ;

interface

uses
  SysUtils;

function ScoreAnswers_MDQ(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_MDQ(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;


implementation

uses
  uMHA_AUX_Const,
  //patch 129 removed fIA_dialogs,
  //fIA_dialogs,
  uMHA_AUX_Utils,
  dialogs;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  Question 1 must be YES.
  For Question 1A through 1M, at least 7 have to be YES.
  Question 2 has to be YES.
  Question 3 has to be anything other than zero.

  If all these conditions are met, then bi-polar diagnosis is supported.


*}

function ScoreAnswers_MDQ(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  Q2, Q3                      : boolean;
  Count, i,
  TempAnswer,
    Yes_Count
                              : Integer;

  MDQ_Result,
    TempResult
                              : string;
begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'MDQ' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);
  Yes_Count := 0;
  MDQ_Result := 'Bipolar mood disorder is not supported.';
  Q2 := false;
  Q3 := false;

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := StrToIntDef(Piece(Piece(TempResult, U, 1), EQ, 2), -1);

      if (i = 1) then // Question 1
        begin
          // Question 1 skipped.

          if TempAnswer = 1155 then
            begin
              MDQ_Result := 'Question 1 was not answered. A diagnosis of a mood disorder ' +
                'could not be determined.';
              Result := 'MDQ^^' + MDQ_Result + RECORD_END;
              exit;
            end;

          // if Question 1 is NO, then we're done.
          if TempAnswer = 0 then // No
            begin
              Result := 'MDQ^^' + MDQ_Result + RECORD_END;
              exit;
            end;

          if TempAnswer = 1 then
            Yes_Count := 1;
        end
      else
        if (i < 14) and (TempAnswer = 1) then
          Inc(Yes_Count);

      if (i = 14) then // Question 2
        begin
          if TempAnswer = 1 then
            Q2 := true;

          if TempAnswer = 1155 then
            begin
              MDQ_Result := 'Question 2 was not answered. A diagnosis of a mood disorder ' +
                'could not be determined.';
              Result := 'MDQ^^' + MDQ_Result + RECORD_END;
              exit;
            end;
        end;

      if (i = 15) then // Question 3
        begin
          if TempAnswer = 1155 then
            begin
              MDQ_Result := 'Question 3 was not answered. A diagnosis of a mood disorder ' +
                'could not be determined.';
              Result := 'MDQ^^' + MDQ_Result + RECORD_END;
              exit;
            end;

          if ((TempAnswer = 2) or (TempAnswer = 3)) then
            Q3 := true;
        end;
     // showmessage('i = ' + inttostr(i) + #13 + 'TempAnswer: ' + inttostr(TempAnswer) + #13
     //              + 'Yes_Count: ' + inttoSTr(yes_count) );
    end; // for i

  if (Yes_Count > 6) and Q2 and Q3 then
    MDQ_Result := 'Bipolar mood disorder is supported.';


  // Note: "CDR" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~
  Result := 'MDQ^^' + MDQ_Result + RECORD_END;
end;


function ProgressNote_MDQ(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr                     : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result := Piece(Piece(ScoreString, RECORD_END, 1), U, 3);

end;

end.

