unit uMHA_AUX_POQ;

interface

uses
  SysUtils;

function ScoreAnswers_POQ(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_POQ(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

function ProgressNote_POQ_SR(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  //patch 129 removed fIA_dialogs,
  //fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The POQ has six scales and an overall score.
  A score is not reported if there are two or more questions not answered;
  this includes the overall score.
*}


function ScoreAnswers_POQ(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

const
  Spacer                      : string = '      ';

var
  Count, i,
    Total,
    Pain, ADL, Mobility, Vitality, NA, Fear: Integer;
  Inpatient,
    Outpatient,
    TempResult,
    TempAnswer                : string;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'POQ' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  Pain := 0;
  ADL := 0;
  Mobility := 0;
  Vitality := 0;
  NA := 0;
  Fear := 0;
  Inpatient := '';
  Outpatient := '';

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = NOT_ASKED) or
        (TempAnswer = SKIPPED) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          TempAnswer := 'X';
        end; // Skipped questions

      if (i = 2) then
        begin
          if TempAnswer <> 'X' then
            pain := StrToIntDef(TempAnswer, 0);
        end;
      if (i = 3) then
        begin
          if TempAnswer <> 'X' then
            Mobility := Mobility + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 4) then
        begin
          if TempAnswer <> 'X' then
            Mobility := Mobility + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 5) then
        begin
          if TempAnswer <> 'X' then
            Mobility := Mobility + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 6) then
        begin
          if TempAnswer <> 'X' then
            Mobility := Mobility + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 7) then
        begin
          if TempAnswer <> 'X' then
            ADL := ADL + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 8) then
        begin
          if TempAnswer <> 'X' then
            ADL := ADL + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 9) then
        begin
          if TempAnswer <> 'X' then
            ADL := ADL + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 10) then
        begin
          if TempAnswer <> 'X' then
            ADL := ADL + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 11) then
        begin
          if TempAnswer <> 'X' then
            NA := NA + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 12) then // reversed already
        begin
          if TempAnswer <> 'X' then
            Vitality := Vitality + (10 - StrToIntDef(TempAnswer, 0));
        end;
      if (i = 13) then // reversed already
        begin
          if TempAnswer <> 'X' then
            Vitality := Vitality + (10 - StrToIntDef(TempAnswer, 0));
        end;
      if (i = 14) then // reversed already
        begin
          if TempAnswer <> 'X' then
            Vitality := Vitality + (10 - StrToIntDef(TempAnswer, 0));
        end;
      if (i = 15) then
        begin
          if TempAnswer <> 'X' then
            NA := NA + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 16) then
        begin
          if TempAnswer <> 'X' then
            NA := NA + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 17) then
        begin
          if TempAnswer <> 'X' then
            Fear := Fear + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 18) then // reversed already
        begin
          if TempAnswer <> 'X' then
            Fear := Fear + (10 - StrToIntDef(TempAnswer, 0));
        end;
      if (i = 19) then
        begin
          if TempAnswer <> 'X' then
            NA := NA + StrToIntDef(TempAnswer, 0);
        end;
      if (i = 20) then
        begin
          if TempAnswer <> 'X' then
            NA := NA + StrToIntDef(TempAnswer, 0);
        end;
    end; // for i

  Total := Pain + ADL + Mobility + Vitality + NA + Fear;

  case Pain of
    0:
      begin
        Inpatient := ' 1';
        Outpatient := ' 1';
      end;
    1:
      begin
        Inpatient := ' 1';
        Outpatient := ' 1';
      end;
    2:
      begin
        Inpatient := ' 1';
        Outpatient := ' 1';
      end;
    3:
      begin
        Inpatient := ' 1';
        Outpatient := ' 2';
      end;
    4:
      begin
        Inpatient := ' 5';
        Outpatient := ' 4';
      end;
    5:
      begin
        Inpatient := '14';
        Outpatient := '11';
      end;
    6:
      begin
        Inpatient := '27';
        Outpatient := '23';
      end;
    7:
      begin
        Inpatient := '47';
        Outpatient := '43';
      end;
    8:
      begin
        Inpatient := '72';
        Outpatient := '69';
      end;
    9:
      begin
        Inpatient := '90';
        Outpatient := '86';
      end;
    10:
      begin
        Inpatient := '97';
        Outpatient := '96';
      end;
  end; // case Pain

  Result := 'Pain' + U + IntToStr(Pain) + U + Inpatient + U + Outpatient + RECORD_END;

  case ADL of
    0:
      begin
        Inpatient := ' 6';
        Outpatient := '12';
      end;
    1:
      begin
        Inpatient := ' 13';
        Outpatient := '25';
      end;
    2:
      begin
        Inpatient := '15';
        Outpatient := '28';
      end;
    3:
      begin
        Inpatient := '18';
        Outpatient := '31';
      end;
    4:
      begin
        Inpatient := '22';
        Outpatient := '36';
      end;
    5:
      begin
        Inpatient := '26';
        Outpatient := '42';
      end;
    6:
      begin
        Inpatient := '29';
        Outpatient := '47';
      end;
    7:
      begin
        Inpatient := '32';
        Outpatient := '50';
      end;
    8:
      begin
        Inpatient := '36';
        Outpatient := '53';
      end;
    9:
      begin
        Inpatient := '40';
        Outpatient := '55';
      end;
    10:
      begin
        Inpatient := '44';
        Outpatient := '57';
      end;
    11:
      begin
        Inpatient := '47';
        Outpatient := '59';
      end;
    12:
      begin
        Inpatient := '50';
        Outpatient := '61';
      end;
    13:
      begin
        Inpatient := '53';
        Outpatient := '63';
      end;
    14:
      begin
        Inpatient := '55';
        Outpatient := '64';
      end;
    15:
      begin
        Inpatient := '58';
        Outpatient := '66';
      end;
    16:
      begin
        Inpatient := '61';
        Outpatient := '67';
      end;
    17:
      begin
        Inpatient := '64';
        Outpatient := '68';
      end;
    18:
      begin
        Inpatient := '67';
        Outpatient := '70';
      end;
    19:
      begin
        Inpatient := '69';
        Outpatient := '71';
      end;
    20:
      begin
        Inpatient := '72';
        Outpatient := '73';
      end;
    21:
      begin
        Inpatient := '74';
        Outpatient := '76';
      end;
    22:
      begin
        Inpatient := '76';
        Outpatient := '78';
      end;
    23:
      begin
        Inpatient := '78';
        Outpatient := '80';
      end;
    24:
      begin
        Inpatient := '80';
        Outpatient := '83';
      end;
    25:
      begin
        Inpatient := '83';
        Outpatient := '85';
      end;
    26:
      begin
        Inpatient := '85';
        Outpatient := '86';
      end;
    27:
      begin
        Inpatient := '86';
        Outpatient := '88';
      end;
    28:
      begin
        Inpatient := '87';
        Outpatient := '89';
      end;
    29:
      begin
        Inpatient := '88';
        Outpatient := '90';
      end;
    30:
      begin
        Inpatient := '89';
        Outpatient := '91';
      end;
    31:
      begin
        Inpatient := '90';
        Outpatient := '92';
      end;
    32:
      begin
        Inpatient := '92';
        Outpatient := '93';
      end;
    33:
      begin
        Inpatient := '93';
        Outpatient := '94';
      end;
    34:
      begin
        Inpatient := '94';
        Outpatient := '95';
      end;
    35:
      begin
        Inpatient := '95';
        Outpatient := '96';
      end;
    36:
      begin
        Inpatient := '96';
        Outpatient := '96';
      end;
    37:
      begin
        Inpatient := '96';
        Outpatient := '96';
      end;
    38:
      begin
        Inpatient := '97';
        Outpatient := '97';
      end;
    39:
      begin
        Inpatient := '97';
        Outpatient := '97';
      end;
    40:
      begin
        Inpatient := '99';
        Outpatient := '99';
      end;
  end; // case  ADL
  Result := Result + 'ADL    ' + U + IntToStr(ADL) + U + Inpatient + U + Outpatient + RECORD_END;

  case Fear of
    0:
      begin
        Inpatient := ' 2';
        Outpatient := ' 2';
      end;
    1:
      begin
        Inpatient := ' 4';
        Outpatient := ' 3';
      end;
    2:
      begin
        Inpatient := ' 4';
        Outpatient := ' 4';
      end;
    3:
      begin
        Inpatient := ' 6';
        Outpatient := ' 6';
      end;
    4:
      begin
        Inpatient := ' 8';
        Outpatient := ' 7';
      end;
    5:
      begin
        Inpatient := '10';
        Outpatient := '10';
      end;
    6:
      begin
        Inpatient := '13';
        Outpatient := '14';
      end;
    7:
      begin
        Inpatient := '16';
        Outpatient := '16';
      end;
    8:
      begin
        Inpatient := '19';
        Outpatient := '19';
      end;
    9:
      begin
        Inpatient := '24';
        Outpatient := '22';
      end;
    10:
      begin
        Inpatient := '32';
        Outpatient := '28';
      end;
    11:
      begin
        Inpatient := '42';
        Outpatient := '35';
      end;
    12:
      begin
        Inpatient := '50';
        Outpatient := '41';
      end;
    13:
      begin
        Inpatient := '58';
        Outpatient := '48';
      end;
    14:
      begin
        Inpatient := '66';
        Outpatient := '54';
      end;
    15:
      begin
        Inpatient := '75';
        Outpatient := '61';
      end;
    16:
      begin
        Inpatient := '82';
        Outpatient := '69';
      end;
    17:
      begin
        Inpatient := '88';
        Outpatient := '75';
      end;
    18:
      begin
        Inpatient := '93';
        Outpatient := '83';
      end;
    19:
      begin
        Inpatient := '97';
        Outpatient := '90';
      end;
    20:
      begin
        Inpatient := '99';
        Outpatient := '96';
      end;

  end; // case Fear
  Result := Result + 'Fear' + U + IntToStr(Fear) + U + Inpatient + U + Outpatient + RECORD_END;

  case Mobility of
    0:
      begin
        Inpatient := ' 1';
        Outpatient := ' 1';
      end;
    1:
      begin
        Inpatient := ' 1';
        Outpatient := ' 1';
      end;
    2:
      begin
        Inpatient := ' 2';
        Outpatient := ' 1';
      end;
    3:
      begin
        Inpatient := ' 2';
        Outpatient := ' 2';
      end;
    4:
      begin
        Inpatient := ' 2';
        Outpatient := ' 2';
      end;
    5:
      begin
        Inpatient := ' 3';
        Outpatient := ' 2';
      end;
    6:
      begin
        Inpatient := ' 3';
        Outpatient := ' 3';
      end;
    7:
      begin
        Inpatient := ' 3';
        Outpatient := ' 3';
      end;
    8:
      begin
        Inpatient := ' 4';
        Outpatient := ' 4';
      end;
    9:
      begin
        Inpatient := ' 5';
        Outpatient := ' 5';
      end;
    10:
      begin
        Inpatient := ' 6';
        Outpatient := ' 7';
      end;
    11:
      begin
        Inpatient := ' 7';
        Outpatient := ' 9';
      end;
    12:
      begin
        Inpatient := ' 9';
        Outpatient := '10';
      end;
    13:
      begin
        Inpatient := '10';
        Outpatient := '12';
      end;
    14:
      begin
        Inpatient := '12';
        Outpatient := '14';
      end;
    15:
      begin
        Inpatient := '14';
        Outpatient := '16';
      end;
    16:
      begin
        Inpatient := '16';
        Outpatient := '19';
      end;
    17:
      begin
        Inpatient := '17';
        Outpatient := '21';
      end;
    18:
      begin
        Inpatient := '20';
        Outpatient := '25';
      end;
    19:
      begin
        Inpatient := '22';
        Outpatient := '29';
      end;
    20:
      begin
        Inpatient := '25';
        Outpatient := '32';
      end;
    21:
      begin
        Inpatient := '28';
        Outpatient := '37';
      end;
    22:
      begin
        Inpatient := '32';
        Outpatient := '41';
      end;
    23:
      begin
        Inpatient := '35';
        Outpatient := '43';
      end;
    24:
      begin
        Inpatient := '38';
        Outpatient := '46';
      end;
    25:
      begin
        Inpatient := '42';
        Outpatient := '49';
      end;
    26:
      begin
        Inpatient := '46';
        Outpatient := '52';
      end;
    27:
      begin
        Inpatient := '50';
        Outpatient := '55';
      end;
    28:
      begin
        Inpatient := '53';
        Outpatient := '57';
      end;
    29:
      begin
        Inpatient := '56';
        Outpatient := '59';
      end;
    30:
      begin
        Inpatient := '60';
        Outpatient := '61';
      end;
    31:
      begin
        Inpatient := '65';
        Outpatient := '65';
      end;
    32:
      begin
        Inpatient := '68';
        Outpatient := '68';
      end;
    33:
      begin
        Inpatient := '72';
        Outpatient := '70';
      end;
    34:
      begin
        Inpatient := '76';
        Outpatient := '73';
      end;
    35:
      begin
        Inpatient := '80';
        Outpatient := '77';
      end;
    36:
      begin
        Inpatient := '83';
        Outpatient := '80';
      end;
    37:
      begin
        Inpatient := '86';
        Outpatient := '83';
      end;
    38:
      begin
        Inpatient := '90';
        Outpatient := '87';
      end;
    39:
      begin
        Inpatient := '93';
        Outpatient := '90';
      end;
    40:
      begin
        Inpatient := '97';
        Outpatient := '95';
      end;
  end; // case Mobility
  Result := Result + 'Mobility' + U + IntToStr(Mobility) + U + Inpatient + U + Outpatient + RECORD_END;

  case NA of
    0:
      begin
        Inpatient := ' 1';
        Outpatient := ' 1';
      end;
    1:
      begin
        Inpatient := ' 1';
        Outpatient := ' 3';
      end;
    2:
      begin
        Inpatient := ' 1';
        Outpatient := ' 4';
      end;
    3:
      begin
        Inpatient := ' 2';
        Outpatient := ' 5';
      end;
    4:
      begin
        Inpatient := ' 2';
        Outpatient := ' 6';
      end;
    5:
      begin
        Inpatient := ' 2';
        Outpatient := ' 7';
      end;
    6:
      begin
        Inpatient := ' 3';
        Outpatient := ' 8';
      end;
    7:
      begin
        Inpatient := ' 4';
        Outpatient := ' 9';
      end;
    8:
      begin
        Inpatient := ' 5';
        Outpatient := '10';
      end;
    9:
      begin
        Inpatient := ' 5';
        Outpatient := '11';
      end;
    10:
      begin
        Inpatient := ' 6';
        Outpatient := '14';
      end;
    11:
      begin
        Inpatient := ' 7';
        Outpatient := '17';
      end;
    12:
      begin
        Inpatient := ' 8';
        Outpatient := '19';
      end;
    13:
      begin
        Inpatient := ' 9';
        Outpatient := '21';
      end;
    14:
      begin
        Inpatient := ' 9';
        Outpatient := '22';
      end;
    15:
      begin
        Inpatient := '10';
        Outpatient := '24';
      end;
    16:
      begin
        Inpatient := '12';
        Outpatient := '27';
      end;
    17:
      begin
        Inpatient := '15';
        Outpatient := '29';
      end;
    18:
      begin
        Inpatient := '17';
        Outpatient := '30';
      end;
    19:
      begin
        Inpatient := '18';
        Outpatient := '32';
      end;
    20:
      begin
        Inpatient := '20';
        Outpatient := '34';
      end;
    21:
      begin
        Inpatient := '22';
        Outpatient := '36';
      end;
    22:
      begin
        Inpatient := '24';
        Outpatient := '39';
      end;
    23:
      begin
        Inpatient := '27';
        Outpatient := '42';
      end;
    24:
      begin
        Inpatient := '30';
        Outpatient := '43';
      end;
    25:
      begin
        Inpatient := '32';
        Outpatient := '45';
      end;
    26:
      begin
        Inpatient := '36';
        Outpatient := '47';
      end;
    27:
      begin
        Inpatient := '39';
        Outpatient := '49';
      end;
    28:
      begin
        Inpatient := '42';
        Outpatient := '52';
      end;
    29:
      begin
        Inpatient := '45';
        Outpatient := '54';
      end;
    30:
      begin
        Inpatient := '49';
        Outpatient := '57';
      end;
    31:
      begin
        Inpatient := '52';
        Outpatient := '60';
      end;
    32:
      begin
        Inpatient := '56';
        Outpatient := '62';
      end;
    33:
      begin
        Inpatient := '60';
        Outpatient := '64';
      end;
    34:
      begin
        Inpatient := '64';
        Outpatient := '68';
      end;
    35:
      begin
        Inpatient := '68';
        Outpatient := '72';
      end;
    36:
      begin
        Inpatient := '72';
        Outpatient := '76';
      end;
    37:
      begin
        Inpatient := '75';
        Outpatient := '80';
      end;
    38:
      begin
        Inpatient := '79';
        Outpatient := '83';
      end;
    39:
      begin
        Inpatient := '82';
        Outpatient := '86';
      end;
    40:
      begin
        Inpatient := '85';
        Outpatient := '87';
      end;
    41:
      begin
        Inpatient := '88';
        Outpatient := '89';
      end;
    42:
      begin
        Inpatient := '90';
        Outpatient := '91';
      end;
    43:
      begin
        Inpatient := '92';
        Outpatient := '92';
      end;
    44:
      begin
        Inpatient := '94';
        Outpatient := '93';
      end;
    45:
      begin
        Inpatient := '95';
        Outpatient := '94';
      end;
    46:
      begin
        Inpatient := '97';
        Outpatient := '95';
      end;
    47:
      begin
        Inpatient := '98';
        Outpatient := '96';
      end;
    48:
      begin
        Inpatient := '99';
        Outpatient := '97';
      end;
    49:
      begin
        Inpatient := '99';
        Outpatient := '97';
      end;
    50:
      begin
        Inpatient := '99';
        Outpatient := '99';
      end;
  end; // case NA
  Result := Result + 'Neg Affect' + U + IntToStr(NA) + U + Inpatient + U + Outpatient + RECORD_END;

  case Vitality of
    0:
      begin
        Inpatient := ' 1';
        Outpatient := ' 1';
      end;
    1:
      begin
        Inpatient := ' 1';
        Outpatient := ' 1';
      end;
    2:
      begin
        Inpatient := ' 1';
        Outpatient := ' 1';
      end;
    3:
      begin
        Inpatient := ' 1';
        Outpatient := ' 1';
      end;
    4:
      begin
        Inpatient := ' 1';
        Outpatient := ' 1';
      end;
    5:
      begin
        Inpatient := ' 1';
        Outpatient := ' 1';
      end;
    6:
      begin
        Inpatient := ' 2';
        Outpatient := ' 2';
      end;
    7:
      begin
        Inpatient := ' 3';
        Outpatient := ' 2';
      end;
    8:
      begin
        Inpatient := ' 3';
        Outpatient := ' 3';
      end;
    9:
      begin
        Inpatient := ' 3';
        Outpatient := ' 3';
      end;
    10:
      begin
        Inpatient := ' 4';
        Outpatient := ' 5';
      end;
    11:
      begin
        Inpatient := ' 6';
        Outpatient := ' 7';
      end;
    12:
      begin
        Inpatient := ' 7';
        Outpatient := ' 9';
      end;
    13:
      begin
        Inpatient := ' 8';
        Outpatient := ' 9';
      end;
    14:
      begin
        Inpatient := '10';
        Outpatient := '11';
      end;
    15:
      begin
        Inpatient := '13';
        Outpatient := '15';
      end;
    16:
      begin
        Inpatient := '18';
        Outpatient := '19';
      end;
    17:
      begin
        Inpatient := '23';
        Outpatient := '24';
      end;
    18:
      begin
        Inpatient := '29';
        Outpatient := '30';
      end;
    19:
      begin
        Inpatient := '35';
        Outpatient := '35';
      end;
    20:
      begin
        Inpatient := '41';
        Outpatient := '41';
      end;
    21:
      begin
        Inpatient := '50';
        Outpatient := '49';
      end;
    22:
      begin
        Inpatient := '60';
        Outpatient := '57';
      end;
    23:
      begin
        Inpatient := '68';
        Outpatient := '65';
      end;
    24:
      begin
        Inpatient := '76';
        Outpatient := '72';
      end;
    25:
      begin
        Inpatient := '83';
        Outpatient := '78';
      end;
    26:
      begin
        Inpatient := '88';
        Outpatient := '82';
      end;
    27:
      begin
        Inpatient := '92';
        Outpatient := '87';
      end;
    28:
      begin
        Inpatient := '95';
        Outpatient := '92';
      end;
    29:
      begin
        Inpatient := '97';
        Outpatient := '95';
      end;
    30:
      begin
        Inpatient := '99';
        Outpatient := '98';
      end;
  end; // case Vitality

  Result := Result + 'Vitality' + U + IntToStr(Vitality) + U + Inpatient + U + Outpatient + RECORD_END;
  (*
    0, 1, 2, 3: Inpatient := ' 1%';
    4, 5: Inpatient := '10%';
    6:
      begin
        Inpatient := '25%';
        Outpatient := '25%';
      end;
    7:
      begin
        Inpatient := '50%';
        Outpatient := '50%';
      end;
    8:
      begin
        Inpatient := '75%';
        Outpatient := '75%';
      end;
    9:
      begin
        Inpatient := '90%';
        Outpatient := '90%';
      end;
    10:
      begin
        Inpatient := '99%';
        Outpatient := '99%';
      end;
  end; // Pain

  case Pain of
    0, 1, 2: Outpatient := ' 1%';
    3, 4, 5: Outpatient := '10%';
  end; // Pain

  Result := 'Pain' + U + IntToStr(Pain) + U + Inpatient + U + Outpatient + RECORD_END;

  if ADL = 0 then
    begin
      Inpatient := '10%';
      Outpatient := '10%';
    end;

  if ADL = 1 then
    begin
      Inpatient := '25%';
      Outpatient := '25%';
    end;

  if (ADL > 2) and (ADL <= 8) then
    Inpatient := '50%';
  if (ADL > 8) and (ADL <= 18) then
    Inpatient := '75%';
  if (ADL > 18) and (ADL <= 28) then
    Inpatient := '90%';
  if (ADL > 28) and (ADL <= 40) then
    Inpatient := '99%';

  if (ADL > 2) and (ADL <= 7) then
    Outpatient := '50%';
  if (ADL > 7) and (ADL <= 21) then
    Outpatient := '75%';
  if (ADL > 21) and (ADL <= 29) then
    Outpatient := '90%';
  if (ADL > 29) and (ADL <= 40) then
    Outpatient := '99%';

  // In order for the tab to be properly set, the short texts have to be at least
  // four characters.
  Result := Result + 'ADL    ' + U + IntToStr(ADL) + U + Inpatient + U + Outpatient + RECORD_END;

  if Mobility = 0 then
    begin
      Inpatient := ' 1%';
      Outpatient := ' 1%';
    end;

  if (Mobility > 0) and (Mobility <= 12) then
    Inpatient := '10%';
  if (Mobility > 12) and (Mobility <= 19) then
    Inpatient := '25%';
  if (Mobility > 19) and (Mobility <= 28) then
    Inpatient := '50%';
  if (Mobility > 28) and (Mobility <= 35) then
    Inpatient := '75%';
  if (Mobility > 35) then
    Inpatient := '99%';

  if (Mobility > 2) and (Mobility <= 12) then
    Outpatient := '10%';
  if (Mobility > 12) and (Mobility <= 18) then
    Outpatient := '25%';
  if (Mobility > 18) and (Mobility <= 25) then
    Outpatient := '50%';
  if (Mobility > 25) and (Mobility <= 35) then
    Outpatient := '75%';
  if (Mobility > 35) and (Mobility <= 39) then
    Outpatient := '90%';
  if (Mobility > 39) then
    Outpatient := '99%';

  Result := Result + 'Mobility' + U + IntToStr(Mobility) + U + Inpatient + U + Outpatient + RECORD_END;

  if (Vitality > 0) and (Vitality <= 8) then
    Inpatient := ' 1%';
  if (Vitality > 8) and (Vitality <= 14) then
    Inpatient := '10%';
  if (Vitality > 14) and (Vitality <= 18) then
    Inpatient := '25%';
  if (Vitality > 18) and (Vitality <= 22) then
    Inpatient := '50%';
  if (Vitality > 22) and (Vitality <= 25) then
    Inpatient := '75%';
  if (Vitality > 25) and (Vitality <= 28) then
    Inpatient := '90%';
  if (Vitality > 28) then
    Inpatient := '99%';

  if (Vitality > 0) and (Vitality <= 2) then
    Outpatient := ' 1%';
  if (Vitality > 2) and (Vitality <= 14) then
    Outpatient := '10%';
  if (Vitality > 14) and (Vitality <= 17) then
    Outpatient := '25%';
  if (Vitality > 17) and (Vitality <= 21) then
    Outpatient := '50%';
  if (Vitality > 21) and (Vitality <= 24) then
    Outpatient := '75%';
  if (Vitality > 24) and (Vitality <= 28) then
    Outpatient := '90%';
  if (Vitality > 28) then
    Outpatient := '99%';

  Result := Result + 'Vitality' + U + IntToStr(Vitality) + U + Inpatient + U + Outpatient + RECORD_END;

  if (NA > 0) and (NA <= 1) then
    Inpatient := ' 1%';
  if (NA > 1) and (NA <= 11) then
    Inpatient := '10%';
  if (NA > 11) and (NA <= 19) then
    Inpatient := '25%';
  if (NA > 19) and (NA <= 29) then
    Inpatient := '50%';
  if (NA > 29) and (NA <= 37) then
    Inpatient := '75%';
  if (NA > 37) and (NA <= 44) then
    Inpatient := '90%';
  if (NA > 44) then
    Inpatient := '99%';

  if (NA = 0) then
    Outpatient := ' 1%';
  if (NA > 1) and (NA <= 8) then
    Outpatient := '10%';
  if (NA > 8) and (NA <= 15) then
    Outpatient := '25%';
  if (NA > 15) and (NA <= 27) then
    Outpatient := '50%';
  if (NA > 27) and (NA <= 36) then
    Outpatient := '75%';
  if (NA > 36) and (NA <= 42) then
    Outpatient := '90%';
  if (NA > 42) then
    Outpatient := '99%';

  Result := Result + 'Neg Affect' + U + IntToStr(NA) + U + Inpatient + U + Outpatient + RECORD_END;

  if (Fear = 0) then
    Inpatient := ' 1%';
  if (Fear > 0) and (Fear <= 4) then
    Inpatient := '10%';
  if (Fear > 4) and (Fear <= 10) then
    Inpatient := '25%';
  if (Fear > 10) and (Fear <= 12) then
    Inpatient := '50%';
  if (Fear > 12) and (Fear <= 16) then
    Inpatient := '75%';
  if (Fear > 16) and (Fear <= 18) then
    Inpatient := '90%';
  if (Fear > 18) then
    Inpatient := '99%';

  if (Fear = 0) then
    Outpatient := ' 1%';
  if (Fear > 0) and (Fear <= 5) then
    Outpatient := '10%';
  if (Fear > 5) and (Fear <= 10) then
    Outpatient := '25%';
  if (Fear > 10) and (Fear <= 13) then
    Outpatient := '50%';
  if (Fear > 13) and (Fear <= 17) then
    Outpatient := '75%';
  if (Fear > 17) and (Fear <= 19) then
    Outpatient := '90%';
  if (Fear > 19) then
    Outpatient := '99%';

  Result := Result + 'Fear' + U + IntToStr(Fear) + U + Inpatient + U + Outpatient + RECORD_END;
  *)
  (*
  if (Total > 0) and (Total <= 31) then
    Inpatient := ' 1%';
  if (Total > 31) and (Total <= 60) then
    Inpatient := '10%';
  if (Total > 60) and (Total <= 77) then
    Inpatient := '25%';
  if (Total > 77) and (Total <= 97) then
    Inpatient := '50%';
  if (Total > 97) and (Total <= 121) then
    Inpatient := '75%';
  if (Total > 121) and (Total <= 138) then
    Inpatient := '90%';
  if (Total > 138) then
    Inpatient := '99%';

  if (Total > 0) and (Total <= 2) then
    outpatient := ' 1%';
  if (Total > 2) and (Total <= 51) then
    outpatient := '10%';
  if (Total > 51) and (Total <= 71) then
    outpatient := '25%';
  if (Total > 71) and (Total <= 97) then
    outpatient := '50%';
  if (Total > 97) and (Total <= 120) then
    outpatient := '75%';
  if (Total > 120) and (Total <= 145) then
    outpatient := '90%';
  if (Total > 145) then
    outpatient := '99%'; *)

  Result := Result + 'Total' + U + IntToStr(Total); // + U + Inpatient + U + Outpatient + RECORD_END;

end;


function ProgressNote_POQ(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.  <.DLL_String.> provides location for this string in the
// report.
const
  indent                      : string = '   ';
  space                       : string = '        ';
var
  i                           : Integer;
  TempStr                     : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  // |        |          |            |
  // Scale    Raw        Inpatient    Outpatient
  //         Score       Percentile   Percentile
  //
      Result :=
        indent + 'POQ Scale' +
        space + 'Raw' +
        '     Inpatient' +
        '     Outpatient' + #13;
      Result := Result +
        indent + '     ' +
        space + '   Score' +
        '     Percentile' +
        '    Percentile' + #13;

      for i := 1 to NumberOfPieces(ScoreString, RECORD_END) - 1 do
        begin
          TempStr := Piece(ScoreString, RECORD_END, i);
          Result := Result +
            indent + ExpandString(Piece(TempStr, U, 1), 10) +
            space + ExpandString(Piece(TempStr, U, 2), 3) +
            space + ExpandString(Piece(TempStr, U, 3), 3) +
            space + indent + Piece(TempStr, U, 4) + #13;
        end;


      TempStr := Piece(ScoreString, RECORD_END, NumberOfPieces(ScoreString, RECORD_END));
      Result := Result + #13 +
        indent + ExpandString(Piece(TempStr, U, 1), 10) +
        space + ExpandString(Piece(TempStr, U, 2), 3) +
        space + ExpandString(Piece(TempStr, U, 3), 3) +
        space + indent + Piece(TempStr, U, 4) + #13 + #13;

      //patch 129 llh added CR (#13) for formating, Capitalized veterans, Remedy Ticket
      Result := Result +
        indent + 'Higher percentile scores denote Veteran''s self-reports of increased symptom ' + #13 +
        'severity when compared to comparison groups of Veterans with pain treated ' + #13 +
        'in inpatient or outpatient multidisciplinary pain settings.' + #13;

      Result := Result + #13; // + #13 +
  //  indent + '------------ POQ Responses ----------' + #13 + #13;
end;

// Screen reader version
function ProgressNote_POQ_SR(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.  <.DLL_String.> provides location for this string in the
// report.
const
  indent                      : string = '   ';
  space                       : string = '        ';
var
  i                           : Integer;
  TempStr                     : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;
//'Pain' + U + IntToStr(Pain) + U + Inpatient + U + Outpatient + RECORD_END;

  for i := 1 to NumberOfPieces(ScoreString, RECORD_END) - 1 do
    begin
      TempStr := Piece(ScoreString, RECORD_END, i);
      Result := Result + Piece(TempStr, U, 1) + ' score is ' +
        Piece(TempStr, U, 2) +
        ' which is ' + Piece(TempStr, U, 3) + ' of inpatient populations and ' +
        Piece(TempStr, U, 4) + ' of outpatient populations.' + #13;
    end;

  TempStr := Piece(ScoreString, RECORD_END, NumberOfPieces(ScoreString, RECORD_END));
  Result := Result + 'Total score is ' + Piece(TempStr, U, 2) + #13;

  //patch 129 llh added CR (#13) for formating, Capitalized veterans
  Result := Result + 'Higher percentile scores denote Veteran''s self-reports of increased symptom ' + #13 +
    'severity when compared to comparison groups of Veterans with pain treated ' + #13 +
    'in inpatient or outpatient multidisciplinary pain settings.' + #13 ;
end;



end.

