unit uMHA_AUX_Utils;

interface

uses
  Classes,
  Controls,
  Windows,
  SysUtils;

type
  TFMDateTime = Double;

//function DropTrailer(AString: string): string;
function FMDateStrToDateStr(AFMDateStr: string): string;
function FMDateTimeStrToDateTimeStr(AFMDateStr: string): string;
function DateStrToFMDateStr(ADateStr: string): string;
function DateTimeToFileNameDateTime(ADateTime: TDateTime): string;
function FileNameDateTimeToDateTimeString(AFileNameDateTime: string): string;
function MBoolStrToDelphiBoolStr(MBoolStr: string): string;
function DelphiBoolStrToMBoolStr(DelphiBoolStr: string): string;
function BoolToStr(ABool: BOOLEAN): string;
function BoolToMStr(ABool: BOOLEAN): string;
function StrToBool(AString: string): BOOLEAN;
function NumberOfPieces(AString: string; ADelimiter: Char): integer;
function PieceIndex(AString: string; ASubString: string; ADelimiter: Char): integer;
function StringToPieces(AString: string; ADelimiter: Char; PieceLength: integer): string;
function IsNull(AValue: string): BOOLEAN;
function FMDOBToDOB(ADOB: string): string;
//function ASIQuestionFieldNumberToIndex(AQuestionID: string): integer;
//function ASIQuestionIndexToFieldNumber(AIndex: integer): string;
//procedure VisitURL(AURL: string);
//function ContainsAlphaNumericsOnly(AString: string): BOOLEAN;
//function WrapString(StringIn: string; Col: Integer): string;

//ORFN methods
function Piece(S: string; Delim: char; PieceNum: Integer): string;
procedure PiecesToList(x: string; ADelim: Char; AList: TStrings);
function IsFMDateTime(x: string): Boolean;
function FMDateTimeToDateTime(ADateTime: TFMDateTime): TDateTime;
function DateTimeToFMDateTime(ADateTime: TDateTime): TFMDateTime;
//procedure MixedCaseList(AList: TStrings);
//function MixedCase(x: string): string;
//function CharAt(x: string; APos: Integer): Char;
//function TextWidthByFont(AFontHandle: THandle; x: string): Integer;
//function TextHeightByFont(AFontHandle: THandle; x: string): Integer;
procedure SetPiece(var x: string; Delim: Char; PieceNum: Integer; NewPiece: string);
//function FormatFMDateTime(AFormat: string; ADateTime: TFMDateTime): string;
function Pieces(S: string; Delim: char; FirstNum, LastNum: Integer): string;
//function FormatSSN(ASSN: string): string;
//procedure InvokeOnlineSupport;
function DateStrToMMDDYYYYFormat(DateStr: string; Delimiter: char): string;
//function GetChoiceOnly(StringIn: string): string;

// Use this to ensure strings are the same width for tabbing in tables.
function ExpandString(StringIn: string; WidthIn: integer): string;

implementation

uses
  ShellAPI,
  uMHA_AUX_Const;

type
  EFMDateTimeError = class(Exception);

  (*
function WrapString(StringIn: string; Col: Integer): string;
var
  i, start, finish, Max, Count: Integer;

begin
  Count := 0;
  start := 1;
  finish := 0;
  Max := Length(StringIn);

  if Max < Col then
    begin
      Result := StringIn;
      Exit;
    end;

  Result := '';

  while Count <= Max do
    begin
      finish := finish + Col;
      if finish >= Max then
        finish := Max;

      if Finish <> Max then
        while not (StringIn[finish] in [#9, #32, #13, #10, #0]) do
          Dec(Finish);

      for i := start to finish do
        begin
          if (StringIn[i] = #13) or (StringIn[i] = #10) or (StringIn[i] = #9) then
          begin
            Inc(Count);
            Result := Result + ' ';
          end
          else
            begin
              Inc(Count);
              Result := Result + StringIn[i];
            end;
        end;

      Result := Result + U;
      Start := Finish + 1;
      if Start >= Max then
        Count := Max + 1;
    end;
end;
*)
  (*
function ASIQuestionFieldNumberToIndex(AQuestionID: string): integer;
var
  i: integer;
begin
  i := 1;

  while not (Piece(ASIFieldNumberString, U, i) = AQuestionID) do
    inc(i);

  //NB: This makes the index zero-based (as it would be in a stringlist)
  //rather than one-based (as it is with piece).
  //It's an arbitrary difference, but the functions in ASI_Loader that use this result
  //assume zero-base and it's too much trouble to rewrite them.
  Result := i - 1;
end;

function ASIQuestionIndexToFieldNumber(AIndex: integer): string;
begin
  Result := Piece(ASIFieldNumberString, U, AIndex);
end;*)

 (*
function DropTrailer(AString: string): string;
begin
  Result := Copy(AString, 1, Length(AString) - 1)
end;*)

function FMDateStrToDateStr(AFMDateStr: string): string;
var
  TempDate                    : string;
begin
  //This function differs from the one in ORFn in that it returns
  //01/01/1900 if the parameter passed to it is not a valid FMDate string.
  //This most likely would be the result of of an empty string.
  TempDate := AFMDateStr;

  if IsFMDateTime(TempDate) then
    Result := DateToStr(FMDateTimeToDateTime(StrToFloat(TempDate)))
  else
    Result := '01/01/1900';
end;

function FMDateTimeStrToDateTimeStr(AFMDateStr: string): string;
var
  TempTimeDate                : string;
begin
  //This function differs from the one in ORFn in that it returns
  //01/01/1900 if the parameter passed to it is not a valid FMDate string.
  //This most likely would be the result of of an empty string.

  TempTimeDate := AFMDateStr;

  if IsFMDateTime(TempTimeDate) then
    Result := DateTimeToStr(FMDateTimeToDateTime(StrToFloat(TempTimeDate)))
  else
    Result := '01/01/1900';
end;

function DateStrToFMDateStr(ADateStr: string): string;
var
  TempDate                    : string;
begin
  //This function differs from the one in ORFn in that it returns
  //2000101 if the parameter passed to it is an empty string.
  //The Access database ensures the date field is either empty or a valid date.
  TempDate := ADateStr;

  if (TempDate = '') then
    Result := '2000101' // FMDate for 01/01/1900
  else
    Result := FloatToStr(DateTimeToFMDateTime(StrToDate(TempDate)));
end;

function MBoolStrToDelphiBoolStr(MBoolStr: string): string;
begin
  Result := 'FALSE';

  if (MBoolStr = '') then
    Exit;

  MBoolStr := UpperCase(copy(MBoolStr, 1, 1));

  if (MBoolStr = 'Y') or
    (MBoolStr = 'YES') or
    (MBoolStr = YES) or
    (MBoolStr = 'True') or
    (MBoolStr = 'T') then
    Result := 'TRUE';

end;

function DelphiBoolStrToMBoolStr(DelphiBoolStr: string): string;
begin
  Result := 'N';

  if (DelphiBoolStr = '') then
    Exit;

  DelphiBoolStr := UpperCase(copy(DelphiBoolStr, 1, 1));

  if (DelphiBoolStr = 'T') then
    Result := 'Y';

end;

function BoolToStr(ABool: BOOLEAN): string;
begin
  if ABool then
    Result := 'TRUE'
  else
    Result := 'FALSE';
end;

function BoolToMStr(ABool: BOOLEAN): string;
begin
  if ABool then
    Result := YES
  else
    Result := NO;
end;

function StrToBool(AString: string): BOOLEAN;
var
  tmpStr                      : string;
begin

  tmpStr := UpperCase(AString);
  Result := False;

  // Cover all the bases
  if (tmpStr = 'TRUE')
    or (tmpStr = 'YES')
    or (tmpStr = 'Y') then
    Result := True;
end;

function DateTimeToFileNameDateTime(ADateTime: TDateTime): string;
var
  tmpDate                     : string;
begin
  //if (ADateTime = null) then
  if (DateToStr(ADateTime) = '') then
    tmpDate := '2000101' //the FMDate for 01/01/1900
  else
    begin
      tmpDate := FloatToStr(DateTimeToFMDateTime(ADateTime));
      tmpDate := StringReplace(tmpDate, '.', '-', [rfReplaceAll]);
    end;

  Result := tmpDate;

end;

function FileNameDateTimeToDateTimeString(AFileNameDateTime: string): string;
var
  tmpDate                     : Double;
begin
  AFileNameDateTime := StringReplace(AFileNameDateTime, '-', '.', []);
  tmpDate := StrToFloat(AFileNameDateTime);
  Result := DateTimeToStr(FMDateTimeToDateTime(tmpDate));
end;

function StringToPieces(AString: string; ADelimiter: Char; PieceLength: integer): string;
var
  i                           : integer;
begin
  if (PieceLength = 0) then
    begin
      Result := BAD_DATA;
      Exit;
    end;

  Result := '';

  for i := 1 to Length(AString) do
    begin
      if ((i mod PieceLength) = 0) then
        Result := Result + ADelimiter;
      Result := Result + copy(AString, i, 1);
    end;
end;

function Piece(S: string; Delim: char; PieceNum: Integer): string;
{ returns the Nth piece (PieceNum) of a string delimited by Delim }
var
  i                           : Integer;
  Strt, Next                  : PChar;
begin
  i := 1;
  Strt := PChar(S);
  Next := StrScan(Strt, Delim);
  while (i < PieceNum) and (Next <> nil) do
    begin
      Inc(i);
      Strt := Next + 1;
      Next := StrScan(Strt, Delim);
    end;
  if Next = nil then
    Next := StrEnd(Strt);
  if i < PieceNum then
    Result := ''
  else
    SetString(Result, Strt, Next - Strt);
end;

procedure PiecesToList(x: string; ADelim: Char; AList: TStrings);
{ adds each piece to a TStrings list, the list is cleared first }
var
  APiece                      : string;
begin
  AList.Clear;
  while Length(x) > 0 do
    begin
      APiece := Piece(x, ADelim, 1);
      AList.Add(APiece);
      Delete(x, 1, Length(APiece) + 1);
    end;
end;

function IsFMDateTime(x: string): Boolean;
var
  i                           : Integer;
begin
  Result := FALSE;
  if Length(x) < 7 then
    Exit;
  for i := 1 to 7 do
    if not (x[i] in ['0'..'9']) then
      Exit;
  if (copy(x, 4, 2) = '00') or (copy(x, 6, 2) = '00') then
    Exit;
  if (Length(x) > 7) and (x[8] <> '.') then
    Exit;
  if (Length(x) > 8) and not (x[9] in ['0'..'9']) then
    Exit;
  Result := TRUE;
end;


function FMDateTimeToDateTime(ADateTime: TFMDateTime): TDateTime;
{ converts a Fileman date/time (type double) to a Delphi date/time }
var
  ADate, ATime                : TDateTime;
  DatePart, TimePart          : string;
  tmpYear, tmpMonth, tmpDay   : integer;
begin
  DatePart := Piece(FloatToStrF(ADateTime, ffFixed, 14, 6), '.', 1);
  TimePart := Piece(FloatToStrF(ADateTime, ffFixed, 14, 6), '.', 2) + '000000';
  if Length(DatePart) <> 7 then
    raise EFMDateTimeError.Create('Invalid Fileman Date');
  if Copy(TimePart, 1, 2) = '24' then
    TimePart := '23595959';

  tmpYear := StrToIntDef(Copy(DatePart, 1, 3), 1) + 1700;
  if (tmpYear < 1) or (tmpYear > 9999) then
    tmpYear := 1;

  tmpMonth := StrToIntDef(Copy(DatePart, 4, 2), 1);
  if (tmpMonth < 1) or (tmpMonth > 12) then
    tmpMonth := 1;

  tmpDay := StrToIntDef(Copy(DatePart, 6, 2), 1);
  if (tmpDay < 1) or (tmpDay > 31) then
    tmpDay := 1;

  case tmpMonth of
    4, 6, 9, 11:
      if (tmpDay > 30) then
        tmpDay := 1;
    2:
      if (tmpYear mod 4 = 0) then
        begin
          if (tmpDay > 29) then
            tmpDay := 1;
        end
      else
        begin
          if (tmpDay > 28) then
            tmpDay := 1;
        end;
  end; //case tmpMonth

  ADate := EncodeDate(tmpYear, tmpMonth, tmpDay);

    //  ADate := EncodeDate(StrToIntDef(Copy(DatePart, 1, 3), 1) + 1700,
//    StrToIntDef(Copy(DatePart, 4, 2), 1),
//    StrToIntDef(Copy(DatePart, 6, 2), 1));

  ATime := EncodeTime(StrToInt(Copy(TimePart, 1, 2)),
    StrToInt(Copy(TimePart, 3, 2)),
    StrToInt(Copy(TimePart, 5, 2)), 0);
  Result := ADate + ATime;
end;

function DateTimeToFMDateTime(ADateTime: TDateTime): TFMDateTime;
{ converts a Delphi date/time type to a Fileman date/time (type double) }
var
  y, m, d, h, n, s, l         : Word;
  DatePart, TimePart          : Integer;
begin
  DecodeDate(ADateTime, y, m, d);
  DecodeTime(ADateTime, h, n, s, l);
  DatePart := ((y - 1700) * 10000) + (m * 100) + d;
  TimePart := (h * 10000) + (n * 100) + s;
  Result := DatePart + (TimePart / 1000000);
end;

(*
procedure MixedCaseList(AList: TStrings);
var
  i: integer;
begin
  for i := 0 to (AList.Count - 1) do AList[i] := MixedCase(AList[i]);
end;

function MixedCase(x: string): string;
var
  i: integer;
begin
  Result := x;
  for i := 2 to Length(x) do
    if (not (x[i - 1] in [' ', '''', ',', '-', '.', '/', '^'])) and (x[i] in ['A'..'Z'])
      then Result[i] := Chr(Ord(x[i]) + 32);
end;

function CharAt(x: string; APos: Integer): Char;
{ returns a character at a given position in a string or the null character if past the end }
begin
  if Length(x) < APos then Result := #0 else Result := x[APos];
end;

function TextWidthByFont(AFontHandle: THandle; x: string): Integer;
{ returns the width of a string in pixels, given a FONT handle and string }
var
  DC: HDC;
  SaveFont: HFont;
  TextSize: TSize;
begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, AFontHandle);
  GetTextExtentPoint32(DC, PChar(x), Length(x), TextSize);
  Result := TextSize.cx;
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
end;

function TextHeightByFont(AFontHandle: THandle; x: string): Integer;
var
  DC: HDC;
  SaveFont: HFont;
  TextSize: TSize;

begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, AFontHandle);
  GetTextExtentPoint32(DC, PChar(x), Length(x), TextSize);
  Result := TextSize.cy;
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
end; *)

procedure SetPiece(var x: string; Delim: Char; PieceNum: Integer; NewPiece: string);
{ sets the Nth piece (PieceNum) of a string to NewPiece, adding delimiters as necessary }
var
  i                           : Integer;
  Strt, Next                  : PChar;
begin
  i := 1;
  Strt := PChar(x);
  Next := StrScan(Strt, Delim);
  while (i < PieceNum) and (Next <> nil) do
    begin
      Inc(i);
      Strt := Next + 1;
      Next := StrScan(Strt, Delim);
    end;
  if Next = nil then
    Next := StrEnd(Strt);
  if i < PieceNum then
    x := x + StringOfChar(Delim, PieceNum - i) + NewPiece
  else
    x := Copy(x, 1, Strt - PChar(x)) + NewPiece + StrPas(Next);
end;

(*
function FormatFMDateTime(AFormat: string; ADateTime: TFMDateTime): string;
{ formats a Fileman Date/Time using (mostly) the same format string as Delphi FormatDateTime }
var
  x: string;
  y, m, d, h, n, s: Integer;

  function TrimFormatCount: Integer;
  { delete repeating characters and count how many were deleted }
  var
    c: Char;
  begin
    Result := 0;
    c := AFormat[1];
    repeat
      Delete(AFormat, 1, 1);
      Inc(Result);
    until CharAt(AFormat, 1) <> c;
  end;

begin {FormatFMDateTime}
  Result := '';
  if not (ADateTime > 0) then Exit;
  x := FloatToStrF(ADateTime, ffFixed, 15, 6) + '0000000';
  y := StrToIntDef(Copy(x, 1, 3), 0) + 1700;
  m := StrToIntDef(Copy(x, 4, 2), 0);
  d := StrToIntDef(Copy(x, 6, 2), 0);
  h := StrToIntDef(Copy(x, 9, 2), 0);
  n := StrToIntDef(Copy(x, 11, 2), 0);
  s := StrToIntDef(Copy(x, 13, 2), 0);
  while Length(AFormat) > 0 do
    case UpCase(AFormat[1]) of
      '"': begin // literal
          Delete(AFormat, 1, 1);
          while not (CharAt(AFormat, 1) in [#0, '"']) do
          begin
            Result := Result + AFormat[1];
            Delete(AFormat, 1, 1);
          end;
          if CharAt(AFormat, 1) = '"' then Delete(AFormat, 1, 1);
        end;
      'D': case TrimFormatCount of // day/date
          1: if d > 0 then Result := Result + IntToStr(d);
          2: if d > 0 then Result := Result + FormatFloat('00', d);
        end;
      'H': case TrimFormatCount of // hour
          1: Result := Result + IntToStr(h);
          2: Result := Result + FormatFloat('00', h);
        end;
      'M': case TrimFormatCount of // month
          1: if m > 0 then Result := Result + IntToStr(m);
          2: if m > 0 then Result := Result + FormatFloat('00', m);
          3: if m in [1..12] then Result := Result + MONTH_NAMES_SHORT[m];
          4: if m in [1..12] then Result := Result + MONTH_NAMES_LONG[m];
        end;
      'N': case TrimFormatCount of // minute
          1: Result := Result + IntToStr(n);
          2: Result := Result + FormatFloat('00', n);
        end;
      'S': case TrimFormatCount of // second
          1: Result := Result + IntToStr(s);
          2: Result := Result + FormatFloat('00', s);
        end;
      'Y': case TrimFormatCount of // year
          2: if y > 0 then Result := Result + Copy(IntToStr(y), 3, 2);
          4: if y > 0 then Result := Result + IntToStr(y);
        end;
    else begin // other
        Result := Result + AFormat[1];
        Delete(AFormat, 1, 1);
      end;
    end; {case}
end; {FormatFMDateTime}
*)

function Pieces(S: string; Delim: char; FirstNum, LastNum: Integer): string;
{ returns several contiguous pieces }
var
  PieceNum                    : Integer;
begin
  Result := '';
  for PieceNum := FirstNum to LastNum do
    Result := Result + Piece(S, Delim, PieceNum) + Delim;
  if Length(Result) > 0 then
    Delete(Result, Length(Result), 1);
end;

function NumberOfPieces(AString: string; ADelimiter: Char): integer;
var
  i, StrLength                : integer;
begin
  StrLength := Length(AString);

  if (StrLength = 0) then
    Result := 0
  else
    begin
      Result := 1;
      for i := 1 to StrLength do
        if (copy(AString, i, 1) = ADelimiter) then
          Result := Result + 1;
    end;
end;

function PieceIndex(AString: string; ASubString: string; ADelimiter: Char): integer;
var
  i, Index                    : integer;
  tmpString, tmpSubString     : string;
  tmpDelimiter                : Char;
begin
  tmpString := AString;
  tmpSubString := ASubString;
  tmpDelimiter := ADelimiter;
  Index := -1; //i.e., the substring is not a piece in the string

  for i := 1 to NumberOfPieces(tmpString, tmpDelimiter) do
    if (Piece(tmpString, tmpDelimiter, i) = tmpSubString) then
      begin
        Index := i;
        Break;
      end;

  Result := Index;

end;

function IsNull(AValue: string): BOOLEAN;
begin
  Result := (AValue = '');
end;

function FMDOBToDOB(ADOB: string): string;
var
  IntYear, IntMonth, IntDay   : integer;
  StrYear                     : string;
begin
  //This function allows DOB to be set even if the value stored in M is not
  //a real FM Date (which happens)
  IntYear := StrToIntDef(copy(ADOB, 1, 3), 0);
  //The string to int function will drop leading zeros, if any
  IntMonth := StrToIntDef(copy(ADOB, 4, 2), 0);
  IntDay := StrToIntDef(copy(ADOB, 6, 2), 0);

  if (IntYear = 0) then
    StrYear := '0000'
  else
    StrYear := IntToStr(IntYear + 1700);

  //Date format = mm/dd/yyyy
  Result := IntToStr(IntMonth) + '/' + IntToStr(IntDay) + '/' + StrYear;

end;

(*
procedure VisitURL(AURL: string);
var
  URLStz: array[0..255] of Char;
begin
  StrPCopy(URLStz, AURL);
  ShellExecute(HWnd_Desktop, 'Open', URLStz, nil, nil, sw_ShowNormal);
end;

function ContainsAlphaNumericsOnly(AString: string): BOOLEAN;
var
  i: integer;
  tmpChar: Char;
begin
  Result := True;

  for i := 1 to Length(AString) do
  begin
    tmpChar := CharAt(AString, i);
    if not (tmpChar in ['0'..'9'])
      and not (tmpChar in ['A'..'Z'])
      and not (tmpChar in ['a'..'z'])
      and not (tmpChar = ' ')
      and not (tmpChar = '''')
      and not (tmpChar = '(')
      and not (tmpChar = ')') then
      Result := False;
  end;

end;  *)

(*
function FormatSSN(ASSN: string): string;
begin
  if (Length(ASSN) > 8)
    and not (Copy(ASSN, 1, 1) = '*') then
    Result := Copy(ASSN, 1, 3) + '-' + Copy(ASSN, 4, 2) + '-'
      + Copy(ASSN, 6, Length(ASSN))
  else
    Result := ASSN;
end;*)


(*
procedure InvokeOnlineSupport;
begin
  ShellExecute(0, 'open', MHA_ONLINE_SUPPORT_URL,
    MHA_ONLINE_SUPPORT_URL, // need this, to pass as parameter to non-default browsers
    '', SW_SHOW);
end; *)

function DateStrToMMDDYYYYFormat(DateStr: string; Delimiter: char): string;
(*
 Take an alpha-numeric string in most any format (e.g., mm dd yy,
   mm\dd\yy, mm.dd.yyyyy, MON dd yy, MON dd, yyyy, dd Month yyyy)
   and convert it to mm/dd/yyyy.
 For two-digit years, between 0 and 9, add 2000; all others add 1900.
 Delimiter is set by user for output date format.
 "Error" is returned if the input is improper.  *)

var
  InputStr                    : string;
  Month                       : array[1..12] of string[3];
  LongMonth                   : array[1..12] of string[9];
  i, j, x                     : integer;
  TempMonth, TempDay          : string[2];
  TempYear                    : string[4];
  GotDay                      : boolean;


  function CheckValues(M, D, Y: string): string;
  var
    TestDate                  : TDateTime;
    Yr, Mo, Dy                : word;

  begin
    result := 'Error';

    try
      Dy := StrToInt(D);
      Mo := StrToInt(M);
      Yr := StrToInt(Y);
    except
      exit;
    end;

    if ((Dy < 1) or (Dy > 31)) then
      exit;
    if ((Mo < 1) or (Mo > 12)) then
      exit;

    // if between 00 and 09 add 2000
    if (Yr < 10) then
      Yr := 2000 + Yr;
    // above 09 add 1900
    if ((Yr > 09) and (Yr < 100)) then
      Yr := 1900 + Yr;

    // let windows take a crack at date.
    try
      TestDate := StrToDate(InttoStr(Mo) + '/' + IntToStr(Dy) + '/' + IntToStr(Yr));
    except
      exit;
    end; {-try-}

    result := IntToStr(Yr);
  end;

begin
  //Blank field: Allow the date to be skipped.
  if Length(DateStr) = 0 then
    begin
      Result := '';
      exit;
    end;

  Result := 'Error';

  // Garbage check
  if Length(DateStr) < 6 then
    exit;

  InputStr := DateStr;
  TempDay := '';
  TempYear := '';
  TempMonth := '';

  //if DateStr[1] in ['A'..'Z', 'a'..'z'] then
  //  begin
  Month[1] := 'JAN';
  Month[5] := 'MAY';
  Month[9] := 'SEP';
  Month[2] := 'FEB';
  Month[6] := 'JUN';
  Month[10] := 'OCT';
  Month[3] := 'MAR';
  Month[7] := 'JUL';
  Month[11] := 'NOV';
  Month[4] := 'APR';
  Month[8] := 'AUG';
  Month[12] := 'DEC';
  LongMonth[1] := 'JANUARY';
  LongMonth[5] := 'MAY';
  LongMonth[9] := 'SEPTEMBER';
  LongMonth[2] := 'FEBRUARY';
  LongMonth[6] := 'JUNE';
  LongMonth[10] := 'OCTOBER';
  LongMonth[3] := 'MARCH';
  LongMonth[7] := 'JULY';
  LongMonth[11] := 'NOVEMBER';
  LongMonth[4] := 'APRIL';
  LongMonth[8] := 'AUGUST';
  LongMonth[12] := 'DECEMBER';

  for i := 1 to 12 do
    begin
      // Look for month's name in string (full name & abbreviation)
      x := Pos(LongMonth[i], UpperCase(DateStr)); // Not found: x := 0
      if x <> 0 then
        Delete(InputStr, x, Length(LongMonth[i]))
      else
        begin
          x := Pos(Month[i], UpperCase(DateStr)); // Not found: x := 0
          if x <> 0 then
            Delete(InputStr, x, Length(Month[i]));
        end;

      // Month found and removed from InputStr. Day & Year remain separated by
      // space and/or comma
      if x <> 0 then
        begin
          TempMonth := IntToStr(i);
          InputStr := Trim(InputStr);
          TempDay := '';
          TempYear := '';
          GotDay := false;

          for j := 1 to Length(InputStr) do // Month's name removed
            begin
              if InputStr[j] in [' ', ','] then
                GotDay := true;
              if InputStr[j] in ['0'..'9'] then
                if not GotDay then
                  TempDay := TempDay + InputStr[j]
                else
                  TempYear := TempYear + InputStr[j];
            end; // for J

          TempYear := CheckValues(TempMonth, TempDay, TempYear);

          if TempYear = 'Error' then
            exit;

          if Length(TempMonth) = 1 then
            TempMonth := '0' + TempMonth;
          if Length(TempDay) = 1 then
            TempDay := '0' + TempDay;

          Result := TempMonth + Delimiter + TempDay + Delimiter + TempYear;
          exit;
        end;
    end; // for i

  // Numeric input: Windows date format is a bit rigid, this will make it a
  // little more flexible.

    // make the delimiters uniform
  for i := 1 to length(InputStr) do
    begin
      if InputStr[i] = ' ' then
        InputStr[i] := '/';
      if InputStr[i] = '-' then
        InputStr[i] := '/';
      if InputStr[i] = '\' then
        InputStr[i] := '/';
      if InputStr[i] = ',' then
        InputStr[i] := '/';
      if InputStr[i] = '.' then
        InputStr[i] := '/';
    end;

  x := 0;
  // Parse the month, day & year from string
  for i := 1 to length(InputStr) do
    begin
      if (InputStr[i] = '/') then
        Inc(x)
      else
        begin
          if InputStr[i] in ['0'..'9'] then
            begin
              if x = 0 then
                TempMonth := TempMonth + InputStr[i];
              if x = 1 then
                TempDay := TempDay + InputStr[i];
              if x = 2 then
                TempYear := TempYear + InputStr[i];
            end;
        end;
    end;

  if x = 3 then
    exit;

  TempYear := CheckValues(TempMonth, TempDay, TempYear);

  if TempYear = 'Error' then
    exit;

  if Length(TempMonth) = 1 then
    TempMonth := '0' + TempMonth;
  if Length(TempDay) = 1 then
    TempDay := '0' + TempDay;

  Result := TempMonth + Delimiter + TempDay + Delimiter + TempYear;
end;

(*
function GetChoiceOnly(StringIn: string): string;
var
  i                           : Integer;
begin
  Result := StringIn;
  i := Pos('.', StringIn);
  if i = 0 then
    i := Pos(')', StringIn);
  if i = 0 then
    i := Pos(']', StringIn);
  if (i > 0) and ((i + 1) < Length(StringIn)) then
    result := Trim(Copy(StringIn, i + 1, Length(StringIn)));
end;  *)

function ExpandString(StringIn: string; WidthIn: integer): string;
// Use this to ensure strings are the same width for tabbing in tables.
begin
  Result := StringIn;
  while Length(Result) < WidthIn do
    result := result + ' ';
end;

end.

