unit uMHA_AUX_VR12;

interface

uses
  SysUtils;

function ScoreAnswers_VR12(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_VR12(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  //patch 129 removed fIA_dialogs,
  //fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The VR-12 has two scales PCS and MCS. The scales have a range of 0-100, a mean
  of 50 and SD of 10.
*}

function ScoreAnswers_VR12(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

const
  Spacer                      : string = '      ';

var
  PCS, MCS                    : Double;
  Count, i
                              : Integer;
  TempResult,
    TempAnswer                : string;
  Complete                    : boolean;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'VR-12' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  PCS := 47.226630;
  MCS := 44.856200;
  Count := NumberOfPieces(AnswerString, RECORD_END);
  Complete := true;

  if Count < 1 then
    Exit;

  // First make sure all the questions are answered.
  // Last two questions are not scored.
  for i := 1 to Count - 3 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      if TempAnswer = '-1' then
        begin
          //Complete := false;
          break;
          Result := 'PCS^^' + 'Missing data' + RECORD_END +
            'MCS^^' + 'Missing data' + RECORD_END;
          exit;
        end;
    end; // for i

  if not complete then
    begin
      // do something with missing data
    end;

  for i := 1 to Count - 1 do // Last two questions are not scored
    begin

      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      if i = 1 then // Question 1
        begin
          //if TempAnswer = '0' then // Excellent is skipped
          if TempAnswer = '1' then // Very good
            begin
              PCS := PCS - 1.422927;
              MCS := MCS + 0.006179;
            end;
          if TempAnswer = '2' then // Good
            begin
              PCS := PCS - 3.200699;
              MCS := MCS + -0.032633;
            end;
          if TempAnswer = '3' then // Fair
            begin
              PCS := PCS - 5.668607;
              MCS := MCS - 0.151991;
            end;
          if TempAnswer = '4' then // Poor
            begin
              PCS := PCS - 7.623203;
              MCS := MCS - 0.410722;
            end;
        end;

      if i = 2 then // Question 2A
        begin
          //if TempAnswer = '0' then skipped
          if TempAnswer = '1' then // Limited a little
            begin
              PCS := PCS + 3.209097;
              MCS := MCS - 1.741941;
            end;
          if TempAnswer = '2' then // Not limited at all
            begin
              PCS := PCS + 6.440926;
              MCS := MCS - 3.391449;
            end;
        end;

      if i = 3 then // Question 2B
        begin
          //if TempAnswer = '0' then skipped
          if TempAnswer = '1' then // Limited a little
            begin
              PCS := PCS + 3.841436;
              MCS := MCS - 1.893174;
            end;
          if TempAnswer = '2' then // Not limited at all
            begin
              PCS := PCS + 6.875059;
              MCS := MCS - 3.358263;
            end;
        end;

      if i = 4 then // Question 3A
        begin
          if TempAnswer = '1' then // a little at a time
            begin
              PCS := PCS - 2.295770;
              MCS := MCS + 0.770424;
            end;
          if TempAnswer = '2' then // some of the time
            begin
              PCS := PCS - 4.220704;
              MCS := MCS + 1.342969;
            end;
          if TempAnswer = '3' then // most of the time
            begin
              PCS := PCS - 5.869204;
              MCS := MCS + 1.843018;
            end;
          if TempAnswer = '4' then // all of the time
            begin
              PCS := PCS - 6.451106;
              MCS := MCS + 2.113603;
            end;
        end;

      if i = 5 then // Question 3B
        begin
          if TempAnswer = '1' then // a little at a time
            begin
              PCS := PCS - 2.853384;
              MCS := MCS + 0.898016;
            end;
          if TempAnswer = '2' then // some of the time
            begin
              PCS := PCS - 4.751619;
              MCS := MCS + 1.519380;
            end;
          if TempAnswer = '3' then // most of the time
            begin
              PCS := PCS - 6.292369;
              MCS := MCS + 1.932001;
            end;
          if TempAnswer = '4' then // all of the time
            begin
              PCS := PCS - 6.834621;
              MCS := MCS + 2.089988;
            end;
        end;

      if i = 6 then // Question 4A
        begin
          if TempAnswer = '1' then // a little at a time
            begin
              PCS := PCS + 1.863268;
              MCS := MCS - 3.867584;
            end;
          if TempAnswer = '2' then // some at a time
            begin
              PCS := PCS + 3.491722;
              MCS := MCS - 7.704990;
            end;
          if TempAnswer = '3' then // most at a time
            begin
              PCS := PCS + 4.604420;
              MCS := MCS - 10.290840;
            end;
          if TempAnswer = '4' then // all of the time
            begin
              PCS := PCS + 4.502007;
              MCS := MCS - 10.038810;
            end;
        end;

      if i = 7 then // Question 4B
        begin
          if TempAnswer = '1' then // a little at a time
            begin
              PCS := PCS + 1.213867;
              MCS := MCS - 3.052609;
            end;
          if TempAnswer = '2' then // some at a time
            begin
              PCS := PCS + 2.227551;
              MCS := MCS - 5.676195;
            end;
          if TempAnswer = '3' then // most at a time
            begin
              PCS := PCS + 2.839852;
              MCS := MCS - 7.568439;
            end;
          if TempAnswer = '4' then // all of the time
            begin
              PCS := PCS + 2.273264;
              MCS := MCS - 6.684413;
            end;
        end;

      if i = 8 then // Question 5
        begin
          if TempAnswer = '1' then // a little bit
            begin
              PCS := PCS - 3.767011;
              MCS := MCS + 0.724378;
            end;
          if TempAnswer = '2' then // moderately
            begin
              PCS := PCS - 6.888286;
              MCS := MCS + 1.289420;
            end;
          if TempAnswer = '3' then // quite a bit
            begin
              PCS := PCS - 9.701818;
              MCS := MCS + 1.752278;
            end;
          if TempAnswer = '4' then // extremely
            begin
              PCS := PCS - 12.553300;
              MCS := MCS + 2.261750;
            end;
        end;

      if i = 9 then // Question 6A
        begin
          if TempAnswer = '1' then // most of the time
            begin
              PCS := PCS + 0.509143;
              MCS := MCS - 1.945028;
            end;
          if TempAnswer = '2' then // good bit of the time
            begin
              PCS := PCS + 1.250000;
              MCS := MCS - 3.920049;
            end;
          if TempAnswer = '3' then // some of the time
            begin
              PCS := PCS + 2.136413;
              MCS := MCS - 6.051385;
            end;
          if TempAnswer = '4' then // little of the time
            begin
              PCS := PCS + 3.068895;
              MCS := MCS - 8.191803;
            end;
          if TempAnswer = '5' then // none of the time
            begin
              PCS := PCS + 3.758398;
              MCS := MCS - 9.805100;
            end;
        end;

      if i = 10 then // Question 6B
        begin
          if TempAnswer = '1' then // most of the time
            begin
              PCS := PCS - 0.487705;
              MCS := MCS - 0.863361;
            end;
          if TempAnswer = '2' then // good bit of the time
            begin
              PCS := PCS - 1.054558;
              MCS := MCS - 1.997290;
            end;
          if TempAnswer = '3' then // some of the time
            begin
              PCS := PCS - 1.570157;
              MCS := MCS - 3.313938;
            end;
          if TempAnswer = '4' then // little of the time
            begin
              PCS := PCS - 2.004446;
              MCS := MCS - 4.671423;
            end;
          if TempAnswer = '5' then // none of the time
            begin
              PCS := PCS - 2.565244;
              MCS := MCS - 6.016106;
            end;
        end;

      if i = 11 then // Question 6C
        begin
          if TempAnswer = '1' then // most of the time
            begin
              PCS := PCS - 0.733526;
              MCS := MCS + 2.825623;
            end;
          if TempAnswer = '2' then // good bit of the time
            begin
              PCS := PCS - 1.840210;
              MCS := MCS + 6.163902;
            end;
          if TempAnswer = '3' then // some of the time
            begin
              PCS := PCS - 3.020777;
              MCS := MCS + 9.500628;
            end;
          if TempAnswer = '4' then // little of the time
            begin
              PCS := PCS - 3.943621;
              MCS := MCS + 12.128690;
            end;
          if TempAnswer = '5' then // none of the time
            begin
              PCS := PCS - 4.854536;
              MCS := MCS + 14.706530;
            end;
        end;

      if i = 12 then // Question 7
        begin
          if TempAnswer = '1' then // most of the time
            begin
              PCS := PCS + 0.214456;
              MCS := MCS + 2.148606;
            end;
          if TempAnswer = '2' then // some of the time
            begin
              PCS := PCS + 0.270629;
              MCS := MCS + 4.989030;
            end;
          if TempAnswer = '3' then // little of the time
            begin
              PCS := PCS + 0.523565;
              MCS := MCS + 7.583853;
            end;
          if TempAnswer = '4' then // none of the time
            begin
              PCS := PCS + 0.772322;
              MCS := MCS + 10.251920;
            end;
        end;
    end; // for i


  // Note: "PCS" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~
  Result := 'PCS^^' + FloatToStrF(PCS, ffFixed, 20, 2) + RECORD_END +
    'MCS^^' + FloatToStrF(MCS, ffFixed, 20, 2) + RECORD_END;
end;


function ProgressNote_VR12(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.
var
  PCS, MCS                     : string;

begin
  Result := '';
  PCS  := '';
  MCS := '';

  if (ScoreString = '') then
    Exit;

  PCS := Piece(Piece(ScoreString, RECORD_END, 1), U, 3); // 6~
  MCS := Piece(Piece(ScoreString, RECORD_END, 2), U, 3); // 6~

  Delete(PCS, Length(PCS), 1);
  Delete(MCS, Length(MCS), 1);

  Result := #13 + #13 +
    'Physical Component Summary Score: ' + PCS + #13 +
    '    Higher scores indicate better health. Range is 1 to 72, Mean = 40, SD = 12. '+ #13 + #13 +
    'Mental Component Summary Score: ' + MCS + #13 +
    '    Higher scores indicate better health. Range is -2 to 76, Mean = 50, SD = 11. ' + #13 + #13;
end;




end.

