unit uIA_AvailableInstrument;

interface

uses
  Contnrs,
  SysUtils,
  classes;

type
  TAvailableInstrument = class
  private
    FAvailInstrumentName,
      FID,
      FR_Privilege,
      FLastEditDate: string;

    FRequiresLicense,
      FLicenseIsCurrent,
      FCopyrightStatusIsOK,
      FCanBeAdministered,
      FIsLegacy,
      FIsNationalInstrument,
      FHasBeenOperational: Boolean;

    FIsOperational: Char;
  protected
  public
    property AvailInstrumentName: string read FAvailInstrumentName write FAvailInstrumentName;
    property AvailInstrumentID: string read FID write FID;
    property R_Privilege: string read FR_Privilege write FR_Privilege;
    property LastEditDate: string read FLastEditDate write FLastEditDate;
    property RequiresLicense: BOOLEAN read FRequiresLicense write
      FRequiresLicense;
    property LicenseIsCurrent: BOOLEAN read FLicenseIsCurrent write
      FLicenseIsCurrent;
    property CopyrightStatusIsOK: BOOLEAN read FCopyrightStatusIsOK write
      FCopyrightStatusIsOK;
    property IsLegacy: BOOLEAN read FIsLegacy write FIsLegacy;
    property IsNationalInstrument: BOOLEAN read FIsNationalInstrument write
      FIsNationalInstrument;
    property CanBeAdministered: BOOLEAN read FCanBeAdministered write
      FCanBeAdministered;
    property HasBeenOperational: BOOLEAN read FHasBeenOperational write
      FHasBeenOperational;
    property IsOperational: Char read FIsOperational write FIsOperational;

    procedure Load(ARecord: string);
    procedure Clear;
  end; { TAvailableInstrument }

var
  AvailableInstrument         : TAvailableInstrument;

implementation

uses
  uIA_Const,
  uIA_Utils;


procedure TAvailableInstrument.Load(ARecord: string);
// Used in IAT
var
  tmpStr,
    tmpValue                  : string;
begin
  tmpStr := '';
  // TEST NAME =
  //	(1) LAST EDIT DATE
  //	(2) ^OPERATIONAL
  //	(3) ^REQUIRES LICENSE
  // 	(4) ^LICENSE CURRENT
  //	(5) ^IS LEGACY
  //	(6) ^IEN
  //	(7) ^R PRIVILEGE
  // 	(8) ^IS NATIONAL
  //	(9) ^HAS BEEN OPERATIONAL

  FAvailInstrumentName := Piece(ARecord, '=', 1);
  tmpValue := Piece(ARecord, '=', 2);
  FLastEditDate := Piece(tmpValue, U, 1);

  FIsOperational := 'U'; // Default
  tmpStr := Piece(tmpValue, U, 2); // Could be Y, N or U
  if Length(tmpStr) <> 0 then
    if CharInSet(tmpStr[1], ['Y', 'N', 'U']) then
      FIsOperational := tmpStr[1];

  FRequiresLicense := StrToBool(Piece(tmpValue, U, 3));
  FLicenseIsCurrent := StrToBool(Piece(tmpValue, U, 4));
  FIsLegacy := StrToBool(Piece(tmpValue, U, 5));
  FID := Piece(tmpValue, U, 6);
  FR_Privilege := UpperCase(Piece(tmpValue, U, 7));
  FIsNationalInstrument := StrToBool(Piece(tmpValue, U, 8));
  FHasBeenOperational := StrToBool(Piece(tmpValue, U, 9));

  if FRequiresLicense then
    FCopyrightStatusIsOK := FLicenseIsCurrent
  else
    FCopyrightStatusIsOK := True;

  FCanBeAdministered := (FIsOperational = 'Y') and FCopyrightStatusIsOK;

end;


procedure TAvailableInstrument.Clear;
// Used in IAT
begin
  FAvailInstrumentName := '';
  FLicenseIsCurrent := False;
  FRequiresLicense := False;
  FR_Privilege := '';
  FLastEditDate := '';
  FID := '';
  FCanBeAdministered := False;
  FIsOperational := 'U';
  FIsLegacy := False;
  FCanBeAdministered := False;
  FCopyrightStatusIsOK := False;
  //FRequiresYSPKey := False;
  FIsNationalInstrument := False;
  FHasBeenOperational := False;
end;


end.

 