unit uIA_AvailableInstrument_OPS;

interface

uses
  Classes,
  Dialogs,
  Trpcb,
  Sysutils;

type

  TAvailableInstrument_OPS = class
  private
    FList: TStringList;
    FCount: Integer;
  protected
  public
    property Count: Integer read FCount write FCount;

    destructor Destroy; override;
    constructor Create;
    function Load(Broker: TRPCBroker): BOOLEAN;

    procedure LoadAvailableInstrumentByName(AName: string);
    procedure LoadAvailableInstrumentByIndex(AIndex: integer);
    function AdministratableNames(OrdererHasYSPKey: BOOLEAN): string;
    function EditableNames(CanEditNationalInstruments: BOOLEAN; CanEditScales: BOOLEAN): string;
    function IsANewName(AName: string): BOOLEAN;
    function LoadSingleInstrumentByName(NameIn: string): boolean;
  published
  end; { TAvailableInstrument_OPS }

var
  AvailableInstrument_OPS     : TAvailableInstrument_OPS;

implementation

uses
  uIA_Const,
  uIA_Utils,
  uIA_AvailableInstrument,
  uIA_AvailableInstrument_IO;


constructor TAvailableInstrument_OPS.Create;
begin
  inherited Create;
  FList := TStringList.Create;
  AvailableInstrument := TAvailableInstrument.Create;
end;


destructor TAvailableInstrument_OPS.Destroy;
begin
  if AvailableInstrument <> nil then
    AvailableInstrument.Free;
  if FList <> nil then
    FList.Free;
  inherited Destroy;
end;


function TAvailableInstrument_OPS.Load(Broker: TRPCBroker): BOOLEAN;
begin
  PiecesToList(ReadAvailableInstruments(Broker), RECORD_END, FList);
  Result := True;

  if (FList.Count < 1)
    or (FList[0] = BAD_DATA) then
    begin
      Result := False;
      FList.Clear;
    end;

  FCount := FList.Count;

end;

procedure TAvailableInstrument_OPS.LoadAvailableInstrumentByName(AName: string);
begin
  LoadAvailableInstrumentByIndex(FList.IndexOfName(AName));
end;

function TAvailableInstrument_OPS.LoadSingleInstrumentByName(NameIn: string): boolean;
var
  i                           : integer;
begin
  result := false;
  for i := 0 to FList.Count - 1 do
    begin
      if UpperCase(NameIn) = UpperCase(Piece(FList[i], EQ, 1)) then
        begin
          AvailableInstrument.Load(FList[i]);
          Result := true;
          exit;
        end;
    end;
end;

procedure TAvailableInstrument_OPS.LoadAvailableInstrumentByIndex(AIndex: integer);
begin
  if (AIndex < 0) or (AIndex >= FCount) then
    AvailableInstrument.Clear
  else
    AvailableInstrument.Load(FList[AIndex]);
end;

function TAvailableInstrument_OPS.AdministratableNames(OrdererHasYSPKey: BOOLEAN): string;
var
  i                           : integer;
  tmpResult                   : string;
begin
  tmpResult := '';

  for i := 0 to FCount - 1 do
    begin
      LoadAvailableInstrumentByIndex(i);
      if not AvailableInstrument.CanBeAdministered then
        Continue;

//      if AvailableInstrument.RequiresYSPKey
//        and not OrdererHasYSPKey then
//        Continue;

      tmpResult := tmpResult + AvailableInstrument.AvailInstrumentName + U;

    end;

  Result := DropTrailer(tmpResult);

end;

function TAvailableInstrument_OPS.EditableNames(CanEditNationalInstruments: BOOLEAN;
  CanEditScales: BOOLEAN): string;
var
  i                           : integer;
  tmpResult                   : string;
begin
  tmpResult := '';

  for i := 0 to FCount - 1 do
    begin
      LoadAvailableInstrumentByIndex(i);

      if AvailableInstrument.IsNationalInstrument
        and not CanEditNationalInstruments then
        Continue;

      if AvailableInstrument.HasBeenOperational
        and not CanEditScales then
        Continue;

      tmpResult := tmpResult + AvailableInstrument.AvailInstrumentName + U;

    end;

  Result := DropTrailer(tmpResult);

end;

function TAvailableInstrument_OPS.IsANewName(AName: string): BOOLEAN;
var
  tmpName                     : string;
  i:integer;
begin
  tmpName := AName;
  Result := (FList.IndexOfName(tmpName) = -1);
end;

initialization
  AvailableInstrument_OPS := TAvailableInstrument_OPS.Create;

finalization
  AvailableInstrument_OPS.Free;

end.

