unit uIA_Instrument;

interface

uses
  Contnrs,
  classes;

type
  TInstrument = class
  private
    FIEN,
      FInstrumentName,
      FPrintTitle,
      FVersion,
      FAuthor,
      FPublisher,
      FCopyright,
      FPublicationDate,
      FReference,
      FNormSample,
      FR_Privilege,
      FPurpose,
      FTargetPopulation,
      FEntryDate,
      FEnteredBy,
      FA_Privilege,
      FTIU_Title,
      FConsult_Title,
      FDLL_Version,
      FDLL_Date,
      FLastEditedBy,
      FLastEditFMDate,
      FLastEditDate
      : string;

    FDaysToRestart,
      FNumberOfQuestions: Integer;
    FIsOperational: CHAR;

    FIsNationalTest,
      FSubmitToNationalDB,
      FRequiresSignature,
      FRequiresLicense,
      FLicenseIsCurrent,
      //FHasBeenOperational,
    FIsASI,
      FIsLegacy,
      FIsCopyrighted,
      FWriteFullText,
      FIsUnderDevelopment,
      FGenerateProgressNote: Boolean;

    FListOfQuestions,
      FListOfIntroductions,
      FListOfSections,
      FListOfRules,
      FListOfAnswers,
      FListOfScaleGroups,
      FListOfScales,
      FListOfKeys: TObjectList;
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;

    {** Does an administration have to be signed by a staff person?}
    property RequiresSignature: BOOLEAN read FRequiresSignature write FRequiresSignature;
    {** Journal article, publisher manual, etc}
    property Reference: string read FReference write FReference;
    {** Is a license (or permission) required to administer instrument?
    NB: If yes, then cannot be released under FOIA}
    property RequiresLicense: BOOLEAN read FRequiresLicense write FRequiresLicense;
    {** Software is operational}
    property IsOperational: CHAR read FIsOperational write FIsOperational;
    {**Is the license current (relevant only if license is required)}
    property LicenseIsCurrent: BOOLEAN read FLicenseIsCurrent write FLicenseIsCurrent;
    property IsASI: BOOLEAN read FIsASI write FIsASI;
    property IsLegacy: BOOLEAN read FIsLegacy write FIsLegacy;
    property NumberOfQuestions: Integer read FNumberOfQuestions write
      FNumberOfQuestions;
    property DaysToRestart: Integer read FDaysToRestart write
      FDaysToRestart;
    property IEN: string read FIEN write FIEN;
    {** No duplicates! If instrument content (questions or choices) changes,
    then a new instrument with a new name must be created.}
    property InstrumentName: string read FInstrumentName write FInstrumentName;
    property PrintTitle: string read FPrintTitle write FPrintTitle;
    property Publisher: string read FPublisher write FPublisher;
    {** The version number in the test manual (e.g., ASI Version 5).}
    property Version: string read FVersion write FVersion;
    property Author: string read FAuthor write FAuthor;
    {** The copyright notice.}
    property Copyright: string read FCopyright write FCopyright;
    property PublicationDate: string read FPublicationDate write
      FPublicationDate;
    property NormSample: string read FNormSample write FNormSample;
    {** Results Privilege (eg. YSP)--What permission is required to see instrument results.}
    property R_Privilege: string read FR_Privilege write FR_Privilege;
    {** What is the test used for, e.g., an alcohol history, personality test, etc.}
    property DLL_Version: string read FDLL_Version write FDLL_Version;
    {** If YS_MHA_AUX.dll scores, verifies, or creates progress note, list the version.}
    {** 1.0.3.100 as example. If not used, leave blank.}
    property DLL_Date: string read FDLL_Date write FDLL_Date;
    property Purpose: string read FPurpose write FPurpose;
    {** The clinical population for which the test is appropriate.}
    property TargetPopulation: string read FTargetPopulation write
      FTargetPopulation;
    property EntryDate: string read FEntryDate write FEntryDate;
    {** The name (and/or DFN?) of the person who entered the test into VistA.
    Needed to track locally developed tests.}
    property EnteredBy: string read FEnteredBy write FEnteredBy;
    {** Admin Privilege--What permission is required to administer the instrument.}
    property A_Privilege: string read FA_Privilege write FA_Privilege;
    {** The name (and/or DFN?) of the person who  last edited the test in VistA.}
    property LastEditedBy: string read FLastEditedBy write FLastEditedBy;
    property LastEditDate: string read FLastEditDate write FLastEditDate;
    property TIU_Title: string read FTIU_Title write FTIU_Title;
    property Consult_Title: string read FConsult_Title write FConsult_Title;

    {** Is this a local test or one that we released nationally.}
    property IsNationalTest: BOOLEAN read FIsNationalTest write FIsNationalTest;
    property LastEditFMDate: string read FLastEditFMDate write FLastEditFMDate;
    property SubmitToNationalDB: Boolean read FSubmitToNationalDB write FSubmitToNationalDB;
    property IsCopyrighted: BOOLEAN read FIsCopyrighted write FIsCopyrighted;
    property WriteFullText: BOOLEAN read FWriteFullText write FWriteFullText;
    property IsUnderDevelopment: BOOLEAN read FIsUnderDevelopment write FIsUnderDevelopment;
    property GenerateProgressNote: BOOLEAN read FGenerateProgressNote write FGenerateProgressNote;

    property ListOfQuestions: TObjectList read FListOfQuestions write FListOfQuestions;
    property ListOfIntroductions: TObjectList read FListOfIntroductions write FListOfIntroductions;
    property ListOfSections: TObjectList read FListOfSections write FListOfSections;
    property ListOfRules: TObjectList read FListOfRules write FListOfRules;
    property ListOfScaleGroups: TObjectList read FListOfScaleGroups write FListOfScaleGroups;
    property ListOfScales: TObjectList read FListOfScales write FListOfScales;
    property ListOfAnswers: TObjectList read FListOfAnswers write FListOfAnswers;
    property ListOfKeys: TObjectList read FListOfKeys write FListOfKeys;
  end; { TInstrument }

implementation


constructor TInstrument.Create;
begin
  inherited Create;

  FListOfQuestions := TObjectList.Create;
  FListOfIntroductions := TObjectList.Create;
  FListOfSections := TObjectList.Create;
  FListOfRules := TObjectList.Create;
  FListOfAnswers := TObjectList.Create;
  FListOfScaleGroups := TObjectList.Create;
  FListOfScales := TObjectList.Create;
  FListOfKeys := TObjectList.Create;
  Clear;

end;

destructor TInstrument.Destroy;
begin

  FListOfIntroductions.Free; // This also frees the Introduction objects
  FListOfSections.Free;
  FListOfQuestions.Free;
  FListOfRules.Free;
  FListOfAnswers.Free;
  FListOfScaleGroups.Free;
  FListOfScales.Free;
  FListOfKeys.Free;

  inherited Destroy;
end;

procedure TInstrument.Clear;
begin
  FIEN := '';
  FInstrumentName := '';
  FPrintTitle := '';
  FPublisher := '';
  FVersion := '';
  FAuthor := '';
  FCopyright := '';
  FPublicationDate := '';
  FReference := '';
  FNormSample := '';
  FR_Privilege := '';
  FPurpose := '';
  FTargetPopulation := '';
  FEntryDate := '';
  FEnteredBy := '';
  FA_Privilege := '';
  FLastEditedBy := '';
  FLastEditDate := '';
  FIsOperational := 'N';
  FIsNationalTest := False;
  FSubmitToNationalDB := False;
  FRequiresSignature := False;
  FRequiresLicense := False;
  FLicenseIsCurrent := False;
  FNumberOfQuestions := 0;
  FDaysToRestart := 2;
  FIsCopyrighted := False;
  FWriteFullText := False;
  FIsASI := False;
  FGenerateProgressNote := True;
  FTIU_Title := '';
  FConsult_Title := '';
  FDLL_Version := '';
  FDLL_Date := '';
end;

end.

