unit uIA_Question;

(*

1) The relationship between Question and Introduction is a congruence between
their respective ObjectLists. ListOfIntroductions[1] is for ListOfQuestions[1],
and so on.

The ObjectList starts at zero. The Introduction text appears on the form before
the Question it is linked with. Both ObjectLists are read from Vista in
uID_Question_IO.

2) MultiChoice is contained within Question. For questions with multiple choice
answers (e.g., radio groups and combo boxes) the multiple choice answers are
stored in a StringList, Choices, as in Question.MultiChoice.Choices has this
format:
     Sequence^ChoiceID^Text^Legacy Value

*)

interface

uses
  Classes,
  SysUtils,
  uIA_Display;

type
  TMultiChoice = class
  private
    FNumberOfPicks: Integer;
    FNumberOfChoices: Integer;
    FChoices: TStringList; // Contains: Sequence^Question.QuestionID^Text^Legacy Value^VUID
    // Sequence starts at one.
  protected
  public

    destructor Destroy; override;
    constructor Create;
    procedure Clear;

    property NumberOfPicks: Integer read FNumberOfPicks write FNumberOfPicks;
    property Choices: TStringList read FChoices write FChoices;
    property NumberOfChoices: Integer read FNumberOfChoices write FNumberOfChoices;
  end; { TMultiChoice }


  TResponseType = (rtMultiChoice, rtInteger, rtString, rtDate, rtMemo,
    rtCurrency, rtVista, rtMaskEdit, rtTrackBar, rtCheckList);

  TChoiceIdentifier = (ciNone, ciZeroBased, ciOneBased);


  TQuestion = class
    ResponseType: TResponseType;
    ChoiceIdentifier: TChoiceIdentifier;
    MultiChoice: TMultiChoice;
    Display: TDisplay;
    AnswerDisplay: TDisplay;

  private
    FQuestionID,
      FQuestionVUID,
      FqText,
      FDesignator,
      FqHint,
      FChoiceTypeID,
      FqCaption,
      FInstrumentContentsSequenceField,
      FInstrumentContentsID: string;
    FqTag,
      FqMax,
      FqMin: Integer;
    FIsRequired,
      FIsLastQuestion,
      FIsFirstQuestion: Boolean;
  protected
  public
    destructor Destroy; override;
    constructor Create;

    property qText: string read FqText write FqText;
    property QuestionID: string read FQuestionID write FQuestionID;
    property QuestionVUID: string read FQuestionVUID write FQuestionVUID;
    property qMin: integer read FqMin write FqMin;
    property qMax: integer read FqMax write FqMax;
    property IsRequired: BOOLEAN read FIsRequired write FIsRequired;
    property Designator: string read FDesignator write FDesignator;
    property qHint: string read FqHint write FqHint;
    property qTag: Integer read FqTag write FqTag;
    property qCaption: string read FqCaption write FqCaption;
    property InstrumentContentsID: string read FInstrumentContentsID write FInstrumentContentsID;
    property IsLastQuestion: BOOLEAN read FIsLastQuestion write FIsLastQuestion;
    property IsFirstQuestion: BOOLEAN read FIsFirstQuestion write FIsFirstQuestion;
    property InstrumentContentsSequenceField: string read FInstrumentContentsSequenceField write FInstrumentContentsSequenceField;
    property ChoiceTypeID: string read FChoiceTypeID write FChoiceTypeID;
    procedure Clear;
  end; { TQuestion }

implementation


constructor TQuestion.Create;
begin
  inherited create;
  MultiChoice := TMultiChoice.Create;
  Display := TDisplay.Create;
  AnswerDisplay := TDisplay.Create;
end;


destructor TQuestion.Destroy;
begin
  if MultiChoice <> nil then
    MultiChoice.Free;
  if Display <> nil then
    Display.Free;
  if AnswerDisplay <> nil then
    AnswerDisplay.Free;
  inherited Destroy;
end;


procedure TQuestion.Clear;
begin
  FqText := '';
  ResponseType := rtMultiChoice;
  FQuestionID := '';
  FQuestionVUID := '';
  FqMax := 0;
  FChoiceTypeID := '';
  FIsRequired := False;
  FqMin := 0;
  FDesignator := '';
  FqHint := '';
  FInstrumentContentsID := '';
  FqTag := -1;
  FqCaption := '';
  FIsFirstQuestion := False;
  FIsLastQuestion := False;
  FInstrumentContentsSequenceField := '';
end;


//------------------------------------------------------------------------------
//    Multiple Choice
//------------------------------------------------------------------------------

constructor TMultiChoice.Create;
begin
  inherited Create;
  FChoices := TStringList.Create;
end;

destructor TMultiChoice.Destroy;
begin
  if FChoices <> nil then
    FChoices.Free;
  inherited Destroy;
end;

procedure TMultiChoice.Clear;
begin
  FChoices.Clear;
  FNumberOfChoices := 0;
  FNumberOfPicks := 1;
end;


end.

